@@ -22,6 +22,7 @@ boundaries of the face.
 
 .. [1] *Snakes: Active contour models*. Kass, M.; Witkin, A.; Terzopoulos, D.
        International Journal of Computer Vision 1 (4): 321 (1988).
+       DOI:`10.1007/BF00133570`
 """
 
 import numpy as np
@@ -36,20 +37,23 @@ img = data.astronaut()
 img = rgb2gray(img)
 
 s = np.linspace(0, 2*np.pi, 400)
-x = 220 + 100*np.cos(s)
-y = 100 + 100*np.sin(s)
-init = np.array([x, y]).T
+r = 100 + 100*np.sin(s)
+c = 220 + 100*np.cos(s)
+init = np.array([r, c]).T
 
 snake = active_contour(gaussian(img, 3),
-                       init, alpha=0.015, beta=10, gamma=0.001)
+                       init, alpha=0.015, beta=10, gamma=0.001,
+                       coordinates='rc')
 
 fig, ax = plt.subplots(figsize=(7, 7))
 ax.imshow(img, cmap=plt.cm.gray)
-ax.plot(init[:, 0], init[:, 1], '--r', lw=3)
-ax.plot(snake[:, 0], snake[:, 1], '-b', lw=3)
+ax.plot(init[:, 1], init[:, 0], '--r', lw=3)
+ax.plot(snake[:, 1], snake[:, 0], '-b', lw=3)
 ax.set_xticks([]), ax.set_yticks([])
 ax.axis([0, img.shape[1], img.shape[0], 0])
 
+plt.show()
+
 ######################################################################
 # Here we initialize a straight line between two points, `(5, 136)` and
 # `(424, 50)`, and require that the spline has its end points there by giving
@@ -58,17 +62,18 @@ ax.axis([0, img.shape[1], img.shape[0], 0])
 
 img = data.text()
 
-x = np.linspace(5, 424, 100)
-y = np.linspace(136, 50, 100)
-init = np.array([x, y]).T
+r = np.linspace(136, 50, 100)
+c = np.linspace(5, 424, 100)
+init = np.array([r, c]).T
 
-snake = active_contour(gaussian(img, 1), init, bc='fixed',
-                       alpha=0.1, beta=1.0, w_line=-5, w_edge=0, gamma=0.1)
+snake = active_contour(gaussian(img, 1), init, boundary_condition='fixed',
+                       alpha=0.1, beta=1.0, w_line=-5, w_edge=0, gamma=0.1,
+                       coordinates='rc')
 
 fig, ax = plt.subplots(figsize=(9, 5))
 ax.imshow(img, cmap=plt.cm.gray)
-ax.plot(init[:, 0], init[:, 1], '--r', lw=3)
-ax.plot(snake[:, 0], snake[:, 1], '-b', lw=3)
+ax.plot(init[:, 1], init[:, 0], '--r', lw=3)
+ax.plot(snake[:, 1], snake[:, 0], '-b', lw=3)
 ax.set_xticks([]), ax.set_yticks([])
 ax.axis([0, img.shape[1], img.shape[0], 0])
 
