@@ -12,9 +12,9 @@ iterations, which needs to be hand-tuned.
 
 .. [1] William Hadley Richardson, "Bayesian-Based Iterative
        Method of Image Restoration",
-       J. Opt. Soc. Am. A 27, 1593-1607 (1972), DOI:10.1364/JOSA.62.000055
+       J. Opt. Soc. Am. A 27, 1593-1607 (1972), :DOI:`10.1364/JOSA.62.000055`
 
-.. [2] http://en.wikipedia.org/wiki/Richardson%E2%80%93Lucy_deconvolution
+.. [2] https://en.wikipedia.org/wiki/Richardson%E2%80%93Lucy_deconvolution
 """
 import numpy as np
 import matplotlib.pyplot as plt
