@@ -48,17 +48,17 @@ ax = axes.ravel()
 
 l_resized = ndi.zoom(l, 2, order=3)
 ax[0].set_title("Original rescaled with\n spline interpolation (order=3)")
-ax[0].imshow(l_resized, extent=(0, 128, 128, 0), interpolation='nearest',
+ax[0].imshow(l_resized, extent=(0, 128, 128, 0),
              cmap=cm.Greys_r)
 
 ax[1].set_title("Block view with\n local mean pooling")
-ax[1].imshow(mean_view, interpolation='nearest', cmap=cm.Greys_r)
+ax[1].imshow(mean_view, cmap=cm.Greys_r)
 
 ax[2].set_title("Block view with\n local max pooling")
-ax[2].imshow(max_view, interpolation='nearest', cmap=cm.Greys_r)
+ax[2].imshow(max_view, cmap=cm.Greys_r)
 
 ax[3].set_title("Block view with\n local median pooling")
-ax[3].imshow(median_view, interpolation='nearest', cmap=cm.Greys_r)
+ax[3].imshow(median_view, cmap=cm.Greys_r)
 
 for a in ax:
     a.set_axis_off()
