@@ -68,15 +68,15 @@ overlay_h = color.label2rgb(label_h_maxima, img, alpha=0.7, bg_label=0,
 # a new figure with 3 subplots
 fig, ax = plt.subplots(1, 3, figsize=(15, 5))
 
-ax[0].imshow(img, cmap='gray', interpolation='none')
+ax[0].imshow(img, cmap='gray')
 ax[0].set_title('Original image')
 ax[0].axis('off')
 
-ax[1].imshow(overlay, interpolation='none')
+ax[1].imshow(overlay)
 ax[1].set_title('Local Maxima')
 ax[1].axis('off')
 
-ax[2].imshow(overlay_h, interpolation='none')
+ax[2].imshow(overlay_h)
 ax[2].set_title('h maxima for h = %.2f' % h)
 ax[2].axis('off')
 plt.show()
