@@ -14,7 +14,7 @@ found.
 
 See Wikipedia_ for more details on the algorithm.
 
-.. _Wikipedia: http://en.wikipedia.org/wiki/Watershed_(image_processing)
+.. _Wikipedia: https://en.wikipedia.org/wiki/Watershed_(image_processing)
 
 """
 
@@ -49,17 +49,17 @@ fig, axes = plt.subplots(nrows=2, ncols=2, figsize=(8, 8),
                          sharex=True, sharey=True)
 ax = axes.ravel()
 
-ax[0].imshow(image, cmap=plt.cm.gray, interpolation='nearest')
+ax[0].imshow(image, cmap=plt.cm.gray)
 ax[0].set_title("Original")
 
-ax[1].imshow(gradient, cmap=plt.cm.nipy_spectral, interpolation='nearest')
+ax[1].imshow(gradient, cmap=plt.cm.nipy_spectral)
 ax[1].set_title("Local Gradient")
 
-ax[2].imshow(markers, cmap=plt.cm.nipy_spectral, interpolation='nearest')
+ax[2].imshow(markers, cmap=plt.cm.nipy_spectral)
 ax[2].set_title("Markers")
 
-ax[3].imshow(image, cmap=plt.cm.gray, interpolation='nearest')
-ax[3].imshow(labels, cmap=plt.cm.nipy_spectral, interpolation='nearest', alpha=.7)
+ax[3].imshow(image, cmap=plt.cm.gray)
+ax[3].imshow(labels, cmap=plt.cm.nipy_spectral, alpha=.7)
 ax[3].set_title("Segmented")
 
 for a in ax:
