@@ -6,7 +6,6 @@ RAG Thresholding
 This example constructs a Region Adjacency Graph (RAG) and merges regions
 which are similar in color. We construct a RAG and define edges as the
 difference in mean color. We then join regions with similar mean color.
-
 """
 
 from skimage import data, io, segmentation, color
@@ -23,8 +22,13 @@ g = graph.rag_mean_color(img, labels1)
 labels2 = graph.cut_threshold(labels1, g, 29)
 out2 = color.label2rgb(labels2, img, kind='avg')
 
-plt.figure()
-io.imshow(out1)
-plt.figure()
-io.imshow(out2)
-io.show()
+fig, ax = plt.subplots(nrows=2, sharex=True, sharey=True,
+                       figsize=(6, 8))
+
+ax[0].imshow(out1)
+ax[1].imshow(out2)
+
+for a in ax:
+    a.axis('off')
+
+plt.tight_layout()
