@@ -75,14 +75,13 @@ As with SLIC, there is an additional *compactness* argument that makes it
 harder for markers to flood faraway pixels. This makes the watershed regions
 more regularly shaped. [5]_
 
-.. [4] http://en.wikipedia.org/wiki/Watershed_%28image_processing%29
+.. [4] https://en.wikipedia.org/wiki/Watershed_%28image_processing%29
 
 .. [5] Peer Neubert & Peter Protzel (2014). Compact Watershed and
        Preemptive SLIC: On Improving Trade-offs of Superpixel Segmentation
-       Algorithms. ICPR 2014, pp 996-1001. DOI:10.1109/ICPR.2014.181
+       Algorithms. ICPR 2014, pp 996-1001. :DOI:`10.1109/ICPR.2014.181`
        https://www.tu-chemnitz.de/etit/proaut/forschung/rsrc/cws_pSLIC_ICPR.pdf
 """
-from __future__ import print_function
 
 import matplotlib.pyplot as plt
 import numpy as np
@@ -102,9 +101,9 @@ segments_quick = quickshift(img, kernel_size=3, max_dist=6, ratio=0.5)
 gradient = sobel(rgb2gray(img))
 segments_watershed = watershed(gradient, markers=250, compactness=0.001)
 
-print("Felzenszwalb number of segments: {}".format(len(np.unique(segments_fz))))
-print('SLIC number of segments: {}'.format(len(np.unique(segments_slic))))
-print('Quickshift number of segments: {}'.format(len(np.unique(segments_quick))))
+print(f"Felzenszwalb number of segments: {len(np.unique(segments_fz))}")
+print(f"SLIC number of segments: {len(np.unique(segments_slic))}")
+print(f"Quickshift number of segments: {len(np.unique(segments_quick))}")
 
 fig, ax = plt.subplots(2, 2, figsize=(10, 10), sharex=True, sharey=True)
 
