@@ -66,10 +66,12 @@ Suggested CSS definitions
 
 """
 import os
+import re
 import shutil
 import token
 import tokenize
 import traceback
+import itertools
 
 import numpy as np
 import matplotlib
@@ -80,10 +82,10 @@ from skimage import io
 from skimage import transform
 from skimage.util.dtype import dtype_range
 
-from notebook import Notebook
+from notebook_doc import Notebook
 
 from docutils.core import publish_parts
-
+from sphinx.domains.python import PythonDomain
 
 LITERALINCLUDE = """
 .. literalinclude:: {src_name}
@@ -132,8 +134,8 @@ GALLERY_IMAGE_TEMPLATE = """
 class Path(str):
     """Path object for manipulating directory and file paths."""
 
-    def __init__(self, path):
-        super(Path, self).__init__(path)
+    def __new__(self, path):
+        return str.__new__(self, path)
 
     @property
     def isdir(self):
@@ -184,6 +186,11 @@ def setup(app):
 def generate_example_galleries(app):
     cfg = app.builder.config
 
+    if isinstance(cfg.source_suffix, list):
+        cfg.source_suffix_str = cfg.source_suffix[0]
+    else:
+        cfg.source_suffix_str = cfg.source_suffix
+
     doc_src = Path(os.path.abspath(app.builder.srcdir)) # path/to/doc/source
 
     if isinstance(cfg.plot2rst_paths, tuple):
@@ -203,18 +210,17 @@ def generate_examples_and_gallery(example_dir, rst_dir, cfg):
     rst_dir.makedirs()
 
     # we create an index.rst with all examples
-    gallery_index = file(rst_dir.pjoin('index'+cfg.source_suffix), 'w')
-
-    # Here we don't use an os.walk, but we recurse only twice: flat is
-    # better than nested.
-    write_gallery(gallery_index, example_dir, rst_dir, cfg)
-    for d in sorted(example_dir.listdir()):
-        example_sub = example_dir.pjoin(d)
-        if example_sub.isdir:
-            rst_sub = rst_dir.pjoin(d)
-            rst_sub.makedirs()
-            write_gallery(gallery_index, example_sub, rst_sub, cfg, depth=1)
-    gallery_index.flush()
+    with open(rst_dir.pjoin('index'+cfg.source_suffix_str), 'w') as gallery_index:
+        # Here we don't use an os.walk, but we recurse only twice: flat is
+        # better than nested.
+        write_gallery(gallery_index, example_dir, rst_dir, cfg)
+        for d in sorted(example_dir.listdir()):
+            example_sub = example_dir.pjoin(d)
+            if example_sub.isdir:
+                rst_sub = rst_dir.pjoin(d)
+                rst_sub.makedirs()
+                write_gallery(gallery_index, example_sub, rst_sub, cfg, depth=1)
+        gallery_index.flush()
 
 
 def write_gallery(gallery_index, src_dir, rst_dir, cfg, depth=0):
@@ -233,7 +239,7 @@ def write_gallery(gallery_index, src_dir, rst_dir, cfg, depth=0):
     cfg : config object
         Sphinx config object created by Sphinx.
     """
-    index_name = cfg.plot2rst_index_name + cfg.source_suffix
+    index_name = cfg.plot2rst_index_name + cfg.source_suffix_str
     gallery_template = src_dir.pjoin(index_name)
     if not os.path.exists(gallery_template):
         print(src_dir)
@@ -244,7 +250,8 @@ def write_gallery(gallery_index, src_dir, rst_dir, cfg, depth=0):
         print(80*'_')
         return
 
-    gallery_description = file(gallery_template).read()
+    with open(gallery_template) as f:
+        gallery_description = f.read()
     gallery_index.write('\n\n%s\n\n' % gallery_description)
 
     rst_dir.makedirs()
@@ -256,7 +263,8 @@ def write_gallery(gallery_index, src_dir, rst_dir, cfg, depth=0):
     else:
         sub_dir_list = src_dir.psplit()[-depth:]
         sub_dir = Path('/'.join(sub_dir_list) + '/')
-    gallery_index.write(TOCTREE_TEMPLATE % (sub_dir + '\n   '.join(ex_names)))
+    joiner = '\n   %s' % sub_dir
+    gallery_index.write(TOCTREE_TEMPLATE % (sub_dir + joiner.join(ex_names)))
 
     for src_name in examples:
 
@@ -325,13 +333,16 @@ def write_example(src_name, src_dir, rst_dir, cfg):
     image_path = image_dir.pjoin(base_image_name + '_{0}.png')
 
     basename, py_ext = os.path.splitext(src_name)
-    rst_path = rst_dir.pjoin(basename + cfg.source_suffix)
+
+    rst_path = rst_dir.pjoin(basename + cfg.source_suffix_str)
     notebook_path = notebook_dir.pjoin(basename + '.ipynb')
 
     if _plots_are_current(src_path, image_path) and rst_path.exists and \
         notebook_path.exists:
         return
 
+    print('plot2rst: %s' % basename)
+
     blocks = split_code_and_text_blocks(example_file)
     if blocks[0][2].startswith('#!'):
         blocks.pop(0) # don't add shebang line to rst file.
@@ -360,9 +371,8 @@ def write_example(src_name, src_dir, rst_dir, cfg):
     ipnotebook_name = './notebook/' + ipnotebook_name
     example_rst += NOTEBOOK_LINK.format(ipnotebook_name)
 
-    f = open(rst_path, 'w')
-    f.write(example_rst)
-    f.flush()
+    with open(rst_path, 'w') as f:
+        f.write(example_rst)
 
     thumb_path = thumb_dir.pjoin(src_name[:-3] + '.png')
     first_image_file = image_dir.pjoin(figure_list[0].lstrip('/'))
@@ -380,15 +390,57 @@ def write_example(src_name, src_dir, rst_dir, cfg):
     # Export example to IPython notebook
     nb = Notebook()
 
-    for (cell_type, _, content) in blocks:
-        content = content.rstrip('\n')
+    # Add sphinx roles to the examples, otherwise docutils
+    # cannot compile the ReST for the notebook
+    sphinx_roles = PythonDomain.roles.keys()
+    preamble = '\n'.join('.. role:: py:{0}(literal)\n'.format(role)
+                         for role in sphinx_roles)
 
+    # Grab all references to inject them in cells where needed
+    ref_regexp = re.compile('\n(\.\. \[(\d+)\].*(?:\n[ ]{7,8}.*)+)')
+    math_role_regexp = re.compile(':math:`(.*?)`')
+
+    text = '\n'.join((content for (cell_type, _, content) in blocks
+                     if cell_type != 'code'))
+
+    references = re.findall(ref_regexp, text)
+
+    for (cell_type, _, content) in blocks:
         if cell_type == 'code':
             nb.add_cell(content, cell_type='code')
         else:
-            content = content.replace('"""', '')
+            if content.startswith('r'):
+                content = content.replace('r"""', '')
+                escaped = False
+            else:
+                content = content.replace('"""', '')
+                escaped = True
+
+            if not escaped:
+                content = content.replace("\\", "\\\\")
+
+            content = content.replace('.. seealso::', '**See also:**')
+            content = re.sub(math_role_regexp, r'$\1$', content)
+
+            # Remove math directive when rendering notebooks
+            # until we implement a smarter way of capturing and replacing
+            # its content
+            content = content.replace('.. math::', '')
+
+            if not content.strip():
+                continue
+
+            content = (preamble + content).rstrip('\n')
             content = '\n'.join([line for line in content.split('\n') if
                                  not line.startswith('.. image')])
+
+            # Remove reference links until we can figure out a better way to
+            # preserve them
+            for (reference, ref_id) in references:
+                ref_tag = '[{0}]_'.format(ref_id)
+                if ref_tag in content:
+                    content = content.replace(ref_tag, ref_tag[:-1])
+
             html = publish_parts(content, writer_name='html')['html_body']
             nb.add_cell(html, cell_type='markdown')
 
