@@ -29,8 +29,6 @@ measure
     Measurement of image properties, e.g., similarity and contours.
 morphology
     Morphological operations, e.g., opening or skeletonization.
-novice
-    Simplified interface for teaching purposes.
 restoration
     Restoration algorithms, e.g., deconvolution algorithms, denoising, etc.
 segmentation
@@ -68,56 +66,13 @@ dtype_limits
 
 """
 
-import imp
-import functools
-import warnings
 import sys
 
-__version__ = '0.14.2'
 
+__version__ = '0.16.2'
 
-try:
-    imp.find_module('pytest')
-except ImportError:
-    def _test(doctest=False, verbose=False):
-        """This would run all unit tests, but pytest couldn't be
-        imported so the test suite can not run.
-        """
-        raise ImportError("Could not load pytest. Unit tests not available.")
-
-else:
-    def _test(doctest=False, verbose=False):
-        """Run all unit tests."""
-        import pytest
-        import warnings
-        args = ['--pyargs', 'skimage']
-        if verbose:
-            args.extend(['-v', '-s'])
-        if doctest:
-            args.extend(['--doctest-modules'])
-            # Make sure warnings do not break the doc tests
-            with warnings.catch_warnings():
-                warnings.simplefilter("ignore")
-                success = pytest.main(args)
-        else:
-            success = pytest.main(args)
-        # Return sys.exit code
-        if success:
-            return 0
-        else:
-            return 1
-
-
-# do not use `test` as function name as this leads to a recursion problem with
-# the nose test suite
-test = _test
-test_verbose = functools.partial(test, verbose=True)
-test_verbose.__doc__ = test.__doc__
-doctest = functools.partial(test, doctest=True)
-doctest.__doc__ = doctest.__doc__
-doctest_verbose = functools.partial(test, doctest=True, verbose=True)
-doctest_verbose.__doc__ = doctest.__doc__
-
+from ._shared.version_requirements import ensure_python_version
+ensure_python_version((3, 5))
 
 # Logic for checking for improper install and importing while in the source
 # tree when package has not been installed inplace.
@@ -130,11 +85,12 @@ directory and you need to try from another location."""
 _STANDARD_MSG = """
 Your install of scikit-image appears to be broken.
 Try re-installing the package following the instructions at:
-http://scikit-image.org/docs/stable/install.html """
+https://scikit-image.org/docs/stable/install.html """
 
 
 def _raise_build_error(e):
     # Raise a comprehensible error
+    import os.path as osp
     local_dir = osp.split(__file__)[0]
     msg = _STANDARD_MSG
     if local_dir == "skimage":
@@ -145,6 +101,7 @@ def _raise_build_error(e):
 It seems that scikit-image has not been built correctly.
 %s""" % (e, msg))
 
+
 try:
     # This variable is injected in the __builtins__ by the build
     # process. It used to enable importing subpackages of skimage when
@@ -163,6 +120,7 @@ else:
         del geometry
     except ImportError as e:
         _raise_build_error(e)
+
     # All skimage root imports go here
     from .util.dtype import (img_as_float32,
                              img_as_float64,
@@ -172,9 +130,7 @@ else:
                              img_as_ubyte,
                              img_as_bool,
                              dtype_limits)
-
-    from .util.lookfor import lookfor
     from .data import data_dir
+    from .util.lookfor import lookfor
 
-
-del warnings, functools, imp, sys
+del sys
