@@ -1,14 +1,14 @@
-from skimage._shared.utils import (copy_func, assert_nD)
+from skimage._shared.utils import (copy_func, check_nD)
 import numpy.testing as npt
 import numpy as np
 from skimage._shared import testing
 
 
-def test_assert_nD():
+def test_check_nD():
     z = np.random.random(200**2).reshape((200, 200))
     x = z[10:30, 30:10]
     with testing.raises(ValueError):
-        assert_nD(x, 2)
+        check_nD(x, 2)
 
 
 def test_copyfunc():
