@@ -5,13 +5,11 @@ import numpy as np
 import types
 import numbers
 
-import six
-
 from ..util import img_as_float
 from ._warnings import all_warnings, warn
 
 __all__ = ['deprecated', 'get_bound_method_class', 'all_warnings',
-           'safe_as_int', 'assert_nD', 'warn']
+           'safe_as_int', 'check_nD', 'check_shape_equality', 'warn']
 
 
 class skimage_deprecation(Warning):
@@ -59,7 +57,7 @@ class deprecated(object):
         @functools.wraps(func)
         def wrapped(*args, **kwargs):
             if self.behavior == 'warn':
-                func_code = six.get_function_code(func)
+                func_code = func.__code__
                 warnings.simplefilter('always', skimage_deprecation)
                 warnings.warn_explicit(msg,
                                        category=skimage_deprecation,
@@ -153,7 +151,14 @@ def safe_as_int(val, atol=1e-3):
     return np.round(val).astype(np.int64)
 
 
-def assert_nD(array, ndim, arg_name='image'):
+def check_shape_equality(im1, im2):
+    """Raise an error if the shape do not match."""
+    if not im1.shape == im2.shape:
+        raise ValueError('Input images must have the same dimensions.')
+    return
+
+
+def check_nD(array, ndim, arg_name='image'):
     """
     Verify an array meets the desired ndims and array isn't empty.
 
@@ -189,9 +194,8 @@ def copy_func(f, name=None):
         Name of new function.
 
     """
-    return types.FunctionType(six.get_function_code(f),
-                              six.get_function_globals(f), name or f.__name__,
-                              six.get_function_defaults(f), six.get_function_closure(f))
+    return types.FunctionType(f.__code__, f.__globals__, name or f.__name__,
+                              f.__defaults__, f.__closure__)
 
 
 def check_random_state(seed):
@@ -222,7 +226,7 @@ def check_random_state(seed):
 
 
 def convert_to_float(image, preserve_range):
-    """Convert input image to double image with the appropriate range.
+    """Convert input image to float image with the appropriate range.
 
     Parameters
     ----------
@@ -230,16 +234,24 @@ def convert_to_float(image, preserve_range):
         Input image.
     preserve_range : bool
         Determines if the range of the image should be kept or transformed
-        using img_as_float.
+        using img_as_float. Also see
+        https://scikit-image.org/docs/dev/user_guide/data_types.html
+
+    Notes:
+    ------
+    * Input images with `float32` data type are not upcast.
 
     Returns
     -------
     image : ndarray
         Transformed version of the input.
+
     """
     if preserve_range:
-        image = image.astype(np.double)
+        # Convert image to double only if it is not single or double
+        # precision float
+        if image.dtype.char not in 'df':
+            image = image.astype(float)
     else:
         image = img_as_float(image)
     return image
-
