@@ -14,10 +14,9 @@ The delta-E notation comes from the German word for "Sensation" (Empfindung).
 
 Reference
 ---------
-http://en.wikipedia.org/wiki/Color_difference
+https://en.wikipedia.org/wiki/Color_difference
 
 """
-from __future__ import division
 
 import numpy as np
 
@@ -41,7 +40,7 @@ def deltaE_cie76(lab1, lab2):
 
     References
     ----------
-    .. [1] http://en.wikipedia.org/wiki/Color_difference
+    .. [1] https://en.wikipedia.org/wiki/Color_difference
     .. [2] A. R. Robertson, "The CIE 1976 color-difference formulae,"
            Color Res. Appl. 2, 7-11 (1977).
     """
@@ -100,7 +99,7 @@ def deltaE_ciede94(lab1, lab2, kH=1, kC=1, kL=1, k1=0.045, k2=0.015):
 
     References
     ----------
-    .. [1] http://en.wikipedia.org/wiki/Color_difference
+    .. [1] https://en.wikipedia.org/wiki/Color_difference
     .. [2] http://www.brucelindbloom.com/index.html?Eqn_DeltaE_CIE94.html
     """
     L1, C1 = np.rollaxis(lab2lch(lab1), -1)[:2]
@@ -152,9 +151,9 @@ def deltaE_ciede2000(lab1, lab2, kL=1, kC=1, kH=1):
 
     References
     ----------
-    .. [1] http://en.wikipedia.org/wiki/Color_difference
+    .. [1] https://en.wikipedia.org/wiki/Color_difference
     .. [2] http://www.ece.rochester.edu/~gsharma/ciede2000/ciede2000noteCRNA.pdf
-           (doi:10.1364/AO.33.008069)
+           :DOI:`10.1364/AO.33.008069`
     .. [3] M. Melgosa, J. Quesada, and E. Hita, "Uniformity of some recent
            color metrics tested with an accurate color-difference tolerance
            dataset," Appl. Opt. 33, 8069-8077 (1994).
@@ -278,7 +277,7 @@ def deltaE_cmc(lab1, lab2, kL=1, kC=1):
 
     References
     ----------
-    .. [1] http://en.wikipedia.org/wiki/Color_difference
+    .. [1] https://en.wikipedia.org/wiki/Color_difference
     .. [2] http://www.brucelindbloom.com/index.html?Eqn_DeltaE_CIE94.html
     .. [3] F. J. J. Clarke, R. McDonald, and B. Rigg, "Modification to the
            JPC79 colour-difference formula," J. Soc. Dyers Colour. 100, 128-132
