@@ -1,4 +1,3 @@
-from __future__ import division
 import numpy as np
 
 from ..color import rgb2gray
@@ -19,7 +18,63 @@ DTYPE_RANGE.update({'uint10': (0, 2 ** 10 - 1),
                     'float': dtype_range[np.float64]})
 
 
-def histogram(image, nbins=256):
+def _offset_array(arr, low_boundary, high_boundary):
+    """Offset the array to get the lowest value at 0 if negative."""
+    if low_boundary < 0:
+        offset = low_boundary
+        dyn_range = high_boundary - low_boundary
+        # get smallest dtype that can hold both minimum and offset maximum
+        offset_dtype = np.promote_types(np.min_scalar_type(dyn_range),
+                                        np.min_scalar_type(low_boundary))
+        if arr.dtype != offset_dtype:
+            # prevent overflow errors when offsetting
+            arr = arr.astype(offset_dtype)
+        arr = arr - offset
+    else:
+        offset = 0
+    return arr, offset
+
+
+def _bincount_histogram(image, source_range):
+    """
+    Efficient histogram calculation for an image of integers.
+
+    This function is significantly more efficient than np.histogram but
+    works only on images of integers. It is based on np.bincount.
+
+    Parameters
+    ----------
+    image : array
+        Input image.
+    source_range : string
+        'image' determines the range from the input image.
+        'dtype' determines the range from the expected range of the images
+        of that data type.
+
+    Returns
+    -------
+    hist : array
+        The values of the histogram.
+    bin_centers : array
+        The values at the center of the bins.
+    """
+    if source_range not in ['image', 'dtype']:
+        raise ValueError('Incorrect value for `source_range` argument: {}'.format(source_range))
+    if source_range == 'image':
+        image_min = int(image.min().astype(np.int64))
+        image_max = int(image.max().astype(np.int64))
+    elif source_range == 'dtype':
+        image_min, image_max = dtype_limits(image, clip_negative=False)
+    image, offset = _offset_array(image, image_min, image_max)
+    hist = np.bincount(image.ravel(), minlength=image_max - image_min + 1)
+    bin_centers = np.arange(image_min, image_max + 1)
+    if source_range == 'image':
+        idx = max(image_min, 0)
+        hist = hist[idx:]
+    return hist, bin_centers
+
+
+def histogram(image, nbins=256, source_range='image', normalize=False):
     """Return histogram of image.
 
     Unlike `numpy.histogram`, this function returns the centers of bins and
@@ -34,9 +89,15 @@ def histogram(image, nbins=256):
     ----------
     image : array
         Input image.
-    nbins : int
+    nbins : int, optional
         Number of bins used to calculate histogram. This value is ignored for
         integer arrays.
+    source_range : string, optional
+        'image' (default) determines the range from the input image.
+        'dtype' determines the range from the expected range of the images
+        of that data type.
+    normalize : bool, optional
+        If True, normalize the histogram by the sum of its values.
 
     Returns
     -------
@@ -64,30 +125,23 @@ def histogram(image, nbins=256):
              "computed on the flattened image. You can instead "
              "apply this function to each color channel.")
 
+    image = image.flatten()
     # For integer types, histogramming with bincount is more efficient.
     if np.issubdtype(image.dtype, np.integer):
-        offset = 0
-        image_min = np.min(image)
-        if image_min < 0:
-            offset = image_min
-            image_range = np.max(image).astype(np.int64) - image_min
-            # get smallest dtype that can hold both minimum and offset maximum
-            offset_dtype = np.promote_types(np.min_scalar_type(image_range),
-                                            np.min_scalar_type(image_min))
-            if image.dtype != offset_dtype:
-                # prevent overflow errors when offsetting
-                image = image.astype(offset_dtype)
-            image = image - offset
-        hist = np.bincount(image.ravel())
-        bin_centers = np.arange(len(hist)) + offset
-
-        # clip histogram to start with a non-zero bin
-        idx = np.nonzero(hist)[0][0]
-        return hist[idx:], bin_centers[idx:]
+        hist, bin_centers = _bincount_histogram(image, source_range)
     else:
-        hist, bin_edges = np.histogram(image.flat, bins=nbins)
+        if source_range == 'image':
+            hist_range = None
+        elif source_range == 'dtype':
+            hist_range = dtype_limits(image, clip_negative=False)
+        else:
+            ValueError('Wrong value for the `source_range` argument')
+        hist, bin_edges = np.histogram(image, bins=nbins, range=hist_range)
         bin_centers = (bin_edges[:-1] + bin_edges[1:]) / 2.
-        return hist, bin_centers
+
+    if normalize:
+        hist = hist / np.sum(hist)
+    return hist, bin_centers
 
 
 def cumulative_distribution(image, nbins=256):
@@ -97,7 +151,7 @@ def cumulative_distribution(image, nbins=256):
     ----------
     image : array
         Image array.
-    nbins : int
+    nbins : int, optional
         Number of bins for image histogram.
 
     Returns
@@ -113,7 +167,7 @@ def cumulative_distribution(image, nbins=256):
 
     References
     ----------
-    .. [1] http://en.wikipedia.org/wiki/Cumulative_distribution_function
+    .. [1] https://en.wikipedia.org/wiki/Cumulative_distribution_function
 
     Examples
     --------
@@ -156,8 +210,8 @@ def equalize_hist(image, nbins=256, mask=None):
 
     References
     ----------
-    .. [1] http://www.janeriksolem.net/2009/06/histogram-equalization-with-python-and.html
-    .. [2] http://en.wikipedia.org/wiki/Histogram_equalization
+    .. [1] http://www.janeriksolem.net/histogram-equalization-with-python-and.html
+    .. [2] https://en.wikipedia.org/wiki/Histogram_equalization
 
     """
     if mask is not None:
@@ -176,7 +230,7 @@ def intensity_range(image, range_values='image', clip_negative=False):
     ----------
     image : array
         Input image.
-    range_values : str or 2-tuple
+    range_values : str or 2-tuple, optional
         The image intensity range is configured by this parameter.
         The possible values for this parameter are enumerated below.
 
@@ -193,7 +247,7 @@ def intensity_range(image, range_values='image', clip_negative=False):
             intensity range explicitly. This option is included for functions
             that use `intensity_range` to support all desired range types.
 
-    clip_negative : bool
+    clip_negative : bool, optional
         If True, clip the negative range (i.e. return 0 for min intensity)
         even if the image dtype allows negative values.
     """
@@ -223,7 +277,7 @@ def rescale_intensity(image, in_range='image', out_range='dtype'):
     ----------
     image : array
         Image array.
-    in_range, out_range : str or 2-tuple
+    in_range, out_range : str or 2-tuple, optional
         Min and max intensity values of input and output image.
         The possible values for this parameter are enumerated below.
 
@@ -294,8 +348,9 @@ def rescale_intensity(image, in_range='image', out_range='dtype'):
 
     image = np.clip(image, imin, imax)
 
-    image = (image - imin) / float(imax - imin)
-    return dtype(image * (omax - omin) + omin)
+    if imin != imax:
+        image = (image - imin) / float(imax - imin)
+    return np.asarray(image * (omax - omin) + omin, dtype=dtype)
 
 
 def _assert_non_negative(image):
@@ -317,9 +372,9 @@ def adjust_gamma(image, gamma=1, gain=1):
     ----------
     image : ndarray
         Input image.
-    gamma : float
+    gamma : float, optional
         Non negative real number. Default value is 1.
-    gain : float
+    gain : float, optional
         The constant multiplier. Default value is 1.
 
     Returns
@@ -341,7 +396,7 @@ def adjust_gamma(image, gamma=1, gain=1):
 
     References
     ----------
-    .. [1] http://en.wikipedia.org/wiki/Gamma_correction
+    .. [1] https://en.wikipedia.org/wiki/Gamma_correction
 
     Examples
     --------
@@ -361,7 +416,7 @@ def adjust_gamma(image, gamma=1, gain=1):
     scale = float(dtype_limits(image, True)[1] - dtype_limits(image, True)[0])
 
     out = ((image / scale) ** gamma) * scale * gain
-    return dtype(out)
+    return out.astype(dtype)
 
 
 def adjust_log(image, gain=1, inv=False):
@@ -375,9 +430,9 @@ def adjust_log(image, gain=1, inv=False):
     ----------
     image : ndarray
         Input image.
-    gain : float
+    gain : float, optional
         The constant multiplier. Default value is 1.
-    inv : float
+    inv : float, optional
         If True, it performs inverse logarithmic correction,
         else correction will be logarithmic. Defaults to False.
 
@@ -404,7 +459,7 @@ def adjust_log(image, gain=1, inv=False):
         return dtype(out)
 
     out = np.log2(1 + image / scale) * scale * gain
-    return dtype(out)
+    return out.astype(dtype)
 
 
 def adjust_sigmoid(image, cutoff=0.5, gain=10, inv=False):
@@ -419,13 +474,13 @@ def adjust_sigmoid(image, cutoff=0.5, gain=10, inv=False):
     ----------
     image : ndarray
         Input image.
-    cutoff : float
+    cutoff : float, optional
         Cutoff of the sigmoid function that shifts the characteristic curve
         in horizontal direction. Default value is 0.5.
-    gain : float
+    gain : float, optional
         The constant multiplier in exponential's power of sigmoid function.
         Default value is 10.
-    inv : bool
+    inv : bool, optional
         If True, returns the negative sigmoid correction. Defaults to False.
 
     Returns
@@ -453,12 +508,12 @@ def adjust_sigmoid(image, cutoff=0.5, gain=10, inv=False):
         return dtype(out)
 
     out = (1 / (1 + np.exp(gain * (cutoff - image / scale)))) * scale
-    return dtype(out)
+    return out.astype(dtype)
 
 
 def is_low_contrast(image, fraction_threshold=0.05, lower_percentile=1,
                     upper_percentile=99, method='linear'):
-    """Detemine if an image is low contrast.
+    """Determine if an image is low contrast.
 
     Parameters
     ----------
@@ -483,7 +538,7 @@ def is_low_contrast(image, fraction_threshold=0.05, lower_percentile=1,
 
     References
     ----------
-    .. [1] http://scikit-image.org/docs/dev/user_guide/data_types.html
+    .. [1] https://scikit-image.org/docs/dev/user_guide/data_types.html
 
     Examples
     --------
