@@ -14,14 +14,14 @@ Original author: Lee Kamentsky
 
 import numpy as np
 import scipy.ndimage as ndi
-from scipy.ndimage import (gaussian_filter,
-                           generate_binary_structure, binary_erosion, label)
-from .. import dtype_limits
-from .._shared.utils import assert_nD
+from scipy.ndimage import generate_binary_structure, binary_erosion, label
+from ..filters import gaussian
+from .. import dtype_limits, img_as_float
+from .._shared.utils import check_nD
 
 
 def smooth_with_function_and_mask(image, function, mask):
-    """Smooth an image with a linear function, ignoring masked pixels
+    """Smooth an image with a linear function, ignoring masked pixels.
 
     Parameters
     ----------
@@ -58,12 +58,12 @@ def canny(image, sigma=1., low_threshold=None, high_threshold=None, mask=None,
     -----------
     image : 2D array
         Grayscale input image to detect edges on; can be of any dtype.
-    sigma : float
+    sigma : float, optional
         Standard deviation of the Gaussian filter.
-    low_threshold : float
+    low_threshold : float, optional
         Lower bound for hysteresis thresholding (linking edges).
         If None, low_threshold is set to 10% of dtype's max.
-    high_threshold : float
+    high_threshold : float, optional
         Upper bound for hysteresis thresholding (linking edges).
         If None, high_threshold is set to 20% of dtype's max.
     mask : array, dtype=bool, optional
@@ -108,6 +108,7 @@ def canny(image, sigma=1., low_threshold=None, high_threshold=None, mask=None,
     -----------
     .. [1] Canny, J., A Computational Approach To Edge Detection, IEEE Trans.
            Pattern Analysis and Machine Intelligence, 8:679-714, 1986
+           :DOI:`10.1109/TPAMI.1986.4767851`
     .. [2] William Green's Canny tutorial
            http://dasl.unlv.edu/daslDrexel/alumni/bGreen/www.pages.drexel.edu/_weg22/can_tut.html
 
@@ -153,19 +154,24 @@ def canny(image, sigma=1., low_threshold=None, high_threshold=None, mask=None,
     # mask by one and then mask the output. We also mask out the border points
     # because who knows what lies beyond the edge of the image?
     #
-    assert_nD(image, 2)
+    check_nD(image, 2)
+    dtype_max = dtype_limits(image, clip_negative=False)[1]
 
     if low_threshold is None:
-        low_threshold = 0.1 * dtype_limits(image, clip_negative=False)[1]
+        low_threshold = 0.1
+    else:
+        low_threshold = low_threshold / dtype_max
 
     if high_threshold is None:
-        high_threshold = 0.2 * dtype_limits(image, clip_negative=False)[1]
+        high_threshold = 0.2
+    else:
+        high_threshold = high_threshold / dtype_max
 
     if mask is None:
         mask = np.ones(image.shape, dtype=bool)
 
     def fsmooth(x):
-        return gaussian_filter(x, sigma, mode='constant')
+        return img_as_float(gaussian(x, sigma, mode='constant'))
 
     smoothed = smooth_with_function_and_mask(image, fsmooth, mask)
     jsobel = ndi.sobel(smoothed, axis=1)
@@ -271,6 +277,7 @@ def canny(image, sigma=1., low_threshold=None, high_threshold=None, mask=None,
     #
     high_mask = local_maxima & (magnitude >= high_threshold)
     low_mask = local_maxima & (magnitude >= low_threshold)
+
     #
     # Segment the low-mask, then only keep low-segments that have
     # some high_mask component in them
