@@ -1,11 +1,12 @@
 import numpy as np
 from scipy import sqrt, pi, arctan2, cos, sin, exp
 from scipy.ndimage import gaussian_filter
-import skimage.color
-from skimage import img_as_float, draw
+from .. import img_as_float, draw
+from ..color import gray2rgb
+from .._shared.utils import check_nD
 
 
-def daisy(img, step=4, radius=15, rings=3, histograms=8, orientations=8,
+def daisy(image, step=4, radius=15, rings=3, histograms=8, orientations=8,
           normalization='l1', sigmas=None, ring_radii=None, visualize=False):
     '''Extract DAISY feature descriptors densely for the given image.
 
@@ -26,8 +27,8 @@ def daisy(img, step=4, radius=15, rings=3, histograms=8, orientations=8,
 
     Parameters
     ----------
-    img : (M, N) array
-        Input image (greyscale).
+    image : (M, N) array
+        Input image (grayscale).
     step : int, optional
         Distance between descriptor sampling points.
     radius : int, optional
@@ -90,14 +91,12 @@ def daisy(img, step=4, radius=15, rings=3, histograms=8, orientations=8,
     .. [1] Tola et al. "Daisy: An efficient dense descriptor applied to wide-
            baseline stereo." Pattern Analysis and Machine Intelligence, IEEE
            Transactions on 32.5 (2010): 815-830.
-    .. [2] http://cvlab.epfl.ch/alumni/tola/daisy.html
+    .. [2] http://cvlab.epfl.ch/software/daisy
     '''
 
-    # Validate image format.
-    if img.ndim != 2:
-        raise ValueError('Only grey-level images are supported.')
+    check_nD(image, 2, 'img')
 
-    img = img_as_float(img)
+    image = img_as_float(image)
 
     # Validate parameters.
     if sigmas is not None and ring_radii is not None \
@@ -116,10 +115,10 @@ def daisy(img, step=4, radius=15, rings=3, histograms=8, orientations=8,
         raise ValueError('Invalid normalization method.')
 
     # Compute image derivatives.
-    dx = np.zeros(img.shape)
-    dy = np.zeros(img.shape)
-    dx[:, :-1] = np.diff(img, n=1, axis=1)
-    dy[:-1, :] = np.diff(img, n=1, axis=0)
+    dx = np.zeros(image.shape)
+    dy = np.zeros(image.shape)
+    dx[:, :-1] = np.diff(image, n=1, axis=1)
+    dy[:-1, :] = np.diff(image, n=1, axis=0)
 
     # Compute gradient orientation and magnitude and their contribution
     # to the histograms.
@@ -128,7 +127,7 @@ def daisy(img, step=4, radius=15, rings=3, histograms=8, orientations=8,
     orientation_kappa = orientations / pi
     orientation_angles = [2 * o * pi / orientations - pi
                           for o in range(orientations)]
-    hist = np.empty((orientations,) + img.shape, dtype=float)
+    hist = np.empty((orientations,) + image.shape, dtype=float)
     for i, o in enumerate(orientation_angles):
         # Weigh bin contribution by the circular normal distribution
         hist[i, :, :] = exp(orientation_kappa * cos(grad_ori - o))
@@ -146,8 +145,8 @@ def daisy(img, step=4, radius=15, rings=3, histograms=8, orientations=8,
     # Assemble descriptor grid.
     theta = [2 * pi * j / histograms for j in range(histograms)]
     desc_dims = (rings * histograms + 1) * orientations
-    descs = np.empty((desc_dims, img.shape[0] - 2 * radius,
-                      img.shape[1] - 2 * radius))
+    descs = np.empty((desc_dims, image.shape[0] - 2 * radius,
+                      image.shape[1] - 2 * radius))
     descs[:orientations, :, :] = hist_smooth[0, :, radius:-radius,
                                              radius:-radius]
     idx = orientations
@@ -178,24 +177,24 @@ def daisy(img, step=4, radius=15, rings=3, histograms=8, orientations=8,
                 descs[:, :, i:i + orientations] /= norms[:, :, np.newaxis]
 
     if visualize:
-        descs_img = skimage.color.gray2rgb(img)
+        descs_img = gray2rgb(image)
         for i in range(descs.shape[0]):
             for j in range(descs.shape[1]):
                 # Draw center histogram sigma
-                color = (1, 0, 0)
+                color = [1, 0, 0]
                 desc_y = i * step + radius
                 desc_x = j * step + radius
-                coords = draw.circle_perimeter(desc_y, desc_x, int(sigmas[0]))
-                draw.set_color(descs_img, coords, color)
+                rows, cols, val = draw.circle_perimeter_aa(desc_y, desc_x, int(sigmas[0]))
+                draw.set_color(descs_img, (rows, cols), color, alpha=val)
                 max_bin = np.max(descs[i, j, :])
                 for o_num, o in enumerate(orientation_angles):
                     # Draw center histogram bins
                     bin_size = descs[i, j, o_num] / max_bin
                     dy = sigmas[0] * bin_size * sin(o)
                     dx = sigmas[0] * bin_size * cos(o)
-                    coords = draw.line(desc_y, desc_x, int(desc_y + dy),
-                                       int(desc_x + dx))
-                    draw.set_color(descs_img, coords, color)
+                    rows, cols, val = draw.line_aa(desc_y, desc_x, int(desc_y + dy),
+                                                   int(desc_x + dx))
+                    draw.set_color(descs_img, (rows, cols), color, alpha=val)
                 for r_num, r in enumerate(ring_radii):
                     color_offset = float(1 + r_num) / rings
                     color = (1 - color_offset, 1, color_offset)
@@ -203,9 +202,9 @@ def daisy(img, step=4, radius=15, rings=3, histograms=8, orientations=8,
                         # Draw ring histogram sigmas
                         hist_y = desc_y + int(round(r * sin(t)))
                         hist_x = desc_x + int(round(r * cos(t)))
-                        coords = draw.circle_perimeter(hist_y, hist_x,
-                                                       int(sigmas[r_num + 1]))
-                        draw.set_color(descs_img, coords, color)
+                        rows, cols, val = draw.circle_perimeter_aa(hist_y, hist_x,
+                                                                   int(sigmas[r_num + 1]))
+                        draw.set_color(descs_img, (rows, cols), color, alpha=val)
                         for o_num, o in enumerate(orientation_angles):
                             # Draw histogram bins
                             bin_size = descs[i, j, orientations + r_num *
@@ -214,10 +213,10 @@ def daisy(img, step=4, radius=15, rings=3, histograms=8, orientations=8,
                             bin_size /= max_bin
                             dy = sigmas[r_num + 1] * bin_size * sin(o)
                             dx = sigmas[r_num + 1] * bin_size * cos(o)
-                            coords = draw.line(hist_y, hist_x,
-                                               int(hist_y + dy),
-                                               int(hist_x + dx))
-                            draw.set_color(descs_img, coords, color)
+                            rows, cols, val = draw.line_aa(hist_y, hist_x,
+                                                           int(hist_y + dy),
+                                                           int(hist_x + dx))
+                            draw.set_color(descs_img, (rows, cols), color, alpha=val)
         return descs, descs_img
     else:
         return descs
