@@ -1,4 +1,3 @@
-from __future__ import division
 import numpy as np
 from scipy.ndimage import gaussian_filter, gaussian_laplace
 import math
@@ -8,18 +7,18 @@ from ..util import img_as_float
 from .peak import peak_local_max
 from ._hessian_det_appx import _hessian_matrix_det
 from ..transform import integral_image
-from .._shared.utils import assert_nD
+from .._shared.utils import check_nD
 
 
 # This basic blob detection algorithm is based on:
 # http://www.cs.utah.edu/~jfishbau/advimproc/project1/ (04.04.2013)
-# Theory behind: http://en.wikipedia.org/wiki/Blob_detection (04.04.2013)
+# Theory behind: https://en.wikipedia.org/wiki/Blob_detection (04.04.2013)
 
 
 def _compute_disk_overlap(d, r1, r2):
     """
-    Compute surface overlap between two disks of radii ``r1`` and ``r2``,
-    with centers separated by a distance ``d``.
+    Compute fraction of surface overlap between two disks of radii
+    ``r1`` and ``r2``, with centers separated by a distance ``d``.
 
     Parameters
     ----------
@@ -32,8 +31,8 @@ def _compute_disk_overlap(d, r1, r2):
 
     Returns
     -------
-    vol: float
-        Volume of the overlap between the two disks.
+    fraction: float
+        Fraction of area of the overlap between the two disks.
     """
 
     ratio1 = (d ** 2 + r1 ** 2 - r2 ** 2) / (2 * d * r1)
@@ -55,8 +54,8 @@ def _compute_disk_overlap(d, r1, r2):
 
 def _compute_sphere_overlap(d, r1, r2):
     """
-    Compute volume overlap between two spheres of radii ``r1`` and ``r2``,
-    with centers separated by a distance ``d``.
+    Compute volume overlap fraction between two spheres of radii
+    ``r1`` and ``r2``, with centers separated by a distance ``d``.
 
     Parameters
     ----------
@@ -69,8 +68,8 @@ def _compute_sphere_overlap(d, r1, r2):
 
     Returns
     -------
-    vol: float
-        Volume of the overlap between the two spheres.
+    fraction: float
+        Fraction of volume of the overlap between the two spheres.
 
     Notes
     -----
@@ -82,10 +81,11 @@ def _compute_sphere_overlap(d, r1, r2):
     return vol / (4./3 * math.pi * min(r1, r2) ** 3)
 
 
-def _blob_overlap(blob1, blob2):
+def _blob_overlap(blob1, blob2, *, sigma_dim=1):
     """Finds the overlapping area fraction between two blobs.
 
-    Returns a float representing fraction of overlapped area.
+    Returns a float representing fraction of overlapped area. Note that 0.0
+    is *always* returned for dimension greater than 3.
 
     Parameters
     ----------
@@ -99,35 +99,49 @@ def _blob_overlap(blob1, blob2):
         where ``row, col`` (or ``(pln, row, col)``) are coordinates
         of blob and ``sigma`` is the standard deviation of the Gaussian kernel
         which detected the blob.
+    sigma_dim : int, optional
+        The dimensionality of the sigma value. Can be 1 or the same as the
+        dimensionality of the blob space (2 or 3).
 
     Returns
     -------
     f : float
         Fraction of overlapped area (or volume in 3D).
     """
-    n_dim = len(blob1) - 1
-    root_ndim = sqrt(n_dim)
+    ndim = len(blob1) - sigma_dim
+    if ndim > 3:
+        return 0.0
+    root_ndim = sqrt(ndim)
 
-    # extent of the blob is given by sqrt(2)*scale
-    r1 = blob1[-1] * root_ndim
-    r2 = blob2[-1] * root_ndim
+    # we divide coordinates by sigma * sqrt(ndim) to rescale space to isotropy,
+    # giving spheres of radius = 1 or < 1.
+    if blob1[-1] > blob2[-1]:
+        max_sigma = blob1[-sigma_dim:]
+        r1 = 1
+        r2 = blob2[-1] / blob1[-1]
+    else:
+        max_sigma = blob2[-sigma_dim:]
+        r2 = 1
+        r1 = blob1[-1] / blob2[-1]
+    pos1 = blob1[:ndim] / (max_sigma * root_ndim)
+    pos2 = blob2[:ndim] / (max_sigma * root_ndim)
 
-    d = sqrt(np.sum((blob1[:-1] - blob2[:-1])**2))
-    if d > r1 + r2:
-        return 0
+    d = np.sqrt(np.sum((pos2 - pos1)**2))
+    if d > r1 + r2:  # centers farther than sum of radii, so no overlap
+        return 0.0
 
-    # one blob is inside the other, the smaller blob must die
+    # one blob is inside the other
     if d <= abs(r1 - r2):
-        return 1
+        return 1.0
 
-    if n_dim == 2:
+    if ndim == 2:
         return _compute_disk_overlap(d, r1, r2)
 
-    else:  # http://mathworld.wolfram.com/Sphere-SphereIntersection.html
+    else:  # ndim=3 http://mathworld.wolfram.com/Sphere-SphereIntersection.html
         return _compute_sphere_overlap(d, r1, r2)
 
 
-def _prune_blobs(blobs_array, overlap):
+def _prune_blobs(blobs_array, overlap, *, sigma_dim=1):
     """Eliminated blobs with area overlap.
 
     Parameters
@@ -142,22 +156,27 @@ def _prune_blobs(blobs_array, overlap):
     overlap : float
         A value between 0 and 1. If the fraction of area overlapping for 2
         blobs is greater than `overlap` the smaller blob is eliminated.
+    sigma_dim : int, optional
+        The number of columns in ``blobs_array`` corresponding to sigmas rather
+        than positions.
 
     Returns
     -------
     A : ndarray
         `array` with overlapping blobs removed.
     """
-    sigma = blobs_array[:, -1].max()
-    distance = 2 * sigma * sqrt(blobs_array.shape[1] - 1)
-    tree = spatial.cKDTree(blobs_array[:, :-1])
+    sigma = blobs_array[:, -sigma_dim:].max()
+    distance = 2 * sigma * sqrt(blobs_array.shape[1] - sigma_dim)
+    tree = spatial.cKDTree(blobs_array[:, :-sigma_dim])
     pairs = np.array(list(tree.query_pairs(distance)))
     if len(pairs) == 0:
         return blobs_array
     else:
         for (i, j) in pairs:
             blob1, blob2 = blobs_array[i], blobs_array[j]
-            if _blob_overlap(blob1, blob2) > overlap:
+            if _blob_overlap(blob1, blob2, sigma_dim=sigma_dim) > overlap:
+                # note: this test works even in the anisotropic case because
+                # all sigmas increase together.
                 if blob1[-1] > blob2[-1]:
                     blob2[-1] = 0
                 else:
@@ -167,8 +186,8 @@ def _prune_blobs(blobs_array, overlap):
 
 
 def blob_dog(image, min_sigma=1, max_sigma=50, sigma_ratio=1.6, threshold=2.0,
-             overlap=.5,):
-    """Finds blobs in the given grayscale image.
+             overlap=.5, *, exclude_border=False):
+    r"""Finds blobs in the given grayscale image.
 
     Blobs are found using the Difference of Gaussian (DoG) method [1]_.
     For each blob found, the method returns its coordinates and the standard
@@ -179,12 +198,16 @@ def blob_dog(image, min_sigma=1, max_sigma=50, sigma_ratio=1.6, threshold=2.0,
     image : 2D or 3D ndarray
         Input grayscale image, blobs are assumed to be light on dark
         background (white on black).
-    min_sigma : float, optional
-        The minimum standard deviation for Gaussian Kernel. Keep this low to
-        detect smaller blobs.
-    max_sigma : float, optional
-        The maximum standard deviation for Gaussian Kernel. Keep this high to
-        detect larger blobs.
+    min_sigma : scalar or sequence of scalars, optional
+        The minimum standard deviation for Gaussian kernel. Keep this low to
+        detect smaller blobs. The standard deviations of the Gaussian filter
+        are given for each axis as a sequence, or as a single number, in
+        which case it is equal for all axes.
+    max_sigma : scalar or sequence of scalars, optional
+        The maximum standard deviation for Gaussian kernel. Keep this high to
+        detect larger blobs. The standard deviations of the Gaussian filter
+        are given for each axis as a sequence, or as a single number, in
+        which case it is equal for all axes.
     sigma_ratio : float, optional
         The ratio between the standard deviation of Gaussian Kernels used for
         computing the Difference of Gaussians
@@ -195,19 +218,25 @@ def blob_dog(image, min_sigma=1, max_sigma=50, sigma_ratio=1.6, threshold=2.0,
     overlap : float, optional
         A value between 0 and 1. If the area of two blobs overlaps by a
         fraction greater than `threshold`, the smaller blob is eliminated.
+    exclude_border : int or bool, optional
+        If nonzero int, `exclude_border` excludes blobs from
+        within `exclude_border`-pixels of the border of the image.
 
     Returns
     -------
-    A : (n, image.ndim + 1) ndarray
-        A 2d array with each row representing 3 values for a 2D image,
-        and 4 values for a 3D image: ``(r, c, sigma)`` or ``(p, r, c, sigma)``
-        where ``(r, c)`` or ``(p, r, c)`` are coordinates of the blob and
-        ``sigma`` is the standard deviation of the Gaussian kernel which
-        detected the blob.
+    A : (n, image.ndim + sigma) ndarray
+        A 2d array with each row representing 2 coordinate values for a 2D
+        image, and 3 coordinate values for a 3D image, plus the sigma(s) used.
+        When a single sigma is passed, outputs are:
+        ``(r, c, sigma)`` or ``(p, r, c, sigma)`` where ``(r, c)`` or
+        ``(p, r, c)`` are coordinates of the blob and ``sigma`` is the standard
+        deviation of the Gaussian kernel which detected the blob. When an
+        anisotropic gaussian is used (sigmas per dimension), the detected sigma
+        is returned for each dimension.
 
     References
     ----------
-    .. [1] http://en.wikipedia.org/wiki/Blob_detection#The_difference_of_Gaussians_approach
+    .. [1] https://en.wikipedia.org/wiki/Blob_detection#The_difference_of_Gaussians_approach
 
     Examples
     --------
@@ -245,8 +274,22 @@ def blob_dog(image, min_sigma=1, max_sigma=50, sigma_ratio=1.6, threshold=2.0,
     """
     image = img_as_float(image)
 
+    # if both min and max sigma are scalar, function returns only one sigma
+    scalar_sigma = np.isscalar(max_sigma) and np.isscalar(min_sigma)
+
+    # Gaussian filter requires that sequence-type sigmas have same
+    # dimensionality as image. This broadcasts scalar kernels
+    if np.isscalar(max_sigma):
+        max_sigma = np.full(image.ndim, max_sigma, dtype=float)
+    if np.isscalar(min_sigma):
+        min_sigma = np.full(image.ndim, min_sigma, dtype=float)
+
+    # Convert sequence types to array
+    min_sigma = np.asarray(min_sigma, dtype=float)
+    max_sigma = np.asarray(max_sigma, dtype=float)
+
     # k such that min_sigma*(sigma_ratio**k) > max_sigma
-    k = int(log(float(max_sigma) / min_sigma, sigma_ratio)) + 1
+    k = int(np.mean(np.log(max_sigma / min_sigma) / np.log(sigma_ratio) + 1))
 
     # a geometric progression of standard deviations for gaussian kernels
     sigma_list = np.array([min_sigma * (sigma_ratio ** i)
@@ -255,9 +298,9 @@ def blob_dog(image, min_sigma=1, max_sigma=50, sigma_ratio=1.6, threshold=2.0,
     gaussian_images = [gaussian_filter(image, s) for s in sigma_list]
 
     # computing difference between two successive Gaussian blurred images
-    # multiplying with standard deviation provides scale invariance
+    # multiplying with average standard deviation provides scale invariance
     dog_images = [(gaussian_images[i] - gaussian_images[i + 1])
-                  * sigma_list[i] for i in range(k)]
+                  * np.mean(sigma_list[i]) for i in range(k)]
 
     image_cube = np.stack(dog_images, axis=-1)
 
@@ -265,20 +308,33 @@ def blob_dog(image, min_sigma=1, max_sigma=50, sigma_ratio=1.6, threshold=2.0,
     local_maxima = peak_local_max(image_cube, threshold_abs=threshold,
                                   footprint=np.ones((3,) * (image.ndim + 1)),
                                   threshold_rel=0.0,
-                                  exclude_border=False)
+                                  exclude_border=exclude_border)
     # Catch no peaks
     if local_maxima.size == 0:
         return np.empty((0, 3))
+
     # Convert local_maxima to float64
     lm = local_maxima.astype(np.float64)
-    # Convert the last index to its corresponding scale value
-    lm[:, -1] = sigma_list[local_maxima[:, -1]]
-    return _prune_blobs(lm, overlap)
+
+    # translate final column of lm, which contains the index of the
+    # sigma that produced the maximum intensity value, into the sigma
+    sigmas_of_peaks = sigma_list[local_maxima[:, -1]]
+
+    if scalar_sigma:
+        # select one sigma column, keeping dimension
+        sigmas_of_peaks = sigmas_of_peaks[:, 0:1]
+
+    # Remove sigma index and replace with sigmas
+    lm = np.hstack([lm[:, :-1], sigmas_of_peaks])
+
+    sigma_dim = sigmas_of_peaks.shape[1]
+
+    return _prune_blobs(lm, overlap, sigma_dim=sigma_dim)
 
 
 def blob_log(image, min_sigma=1, max_sigma=50, num_sigma=10, threshold=.2,
-             overlap=.5, log_scale=False):
-    """Finds blobs in the given grayscale image.
+             overlap=.5, log_scale=False, *, exclude_border=False):
+    r"""Finds blobs in the given grayscale image.
 
     Blobs are found using the Laplacian of Gaussian (LoG) method [1]_.
     For each blob found, the method returns its coordinates and the standard
@@ -289,12 +345,16 @@ def blob_log(image, min_sigma=1, max_sigma=50, num_sigma=10, threshold=.2,
     image : 2D or 3D ndarray
         Input grayscale image, blobs are assumed to be light on dark
         background (white on black).
-    min_sigma : float, optional
-        The minimum standard deviation for Gaussian Kernel. Keep this low to
-        detect smaller blobs.
-    max_sigma : float, optional
-        The maximum standard deviation for Gaussian Kernel. Keep this high to
-        detect larger blobs.
+    min_sigma : scalar or sequence of scalars, optional
+        the minimum standard deviation for Gaussian kernel. Keep this low to
+        detect smaller blobs. The standard deviations of the Gaussian filter
+        are given for each axis as a sequence, or as a single number, in
+        which case it is equal for all axes.
+    max_sigma : scalar or sequence of scalars, optional
+        The maximum standard deviation for Gaussian kernel. Keep this high to
+        detect larger blobs. The standard deviations of the Gaussian filter
+        are given for each axis as a sequence, or as a single number, in
+        which case it is equal for all axes.
     num_sigma : int, optional
         The number of intermediate values of standard deviations to consider
         between `min_sigma` and `max_sigma`.
@@ -309,19 +369,25 @@ def blob_log(image, min_sigma=1, max_sigma=50, num_sigma=10, threshold=.2,
         If set intermediate values of standard deviations are interpolated
         using a logarithmic scale to the base `10`. If not, linear
         interpolation is used.
+    exclude_border : int or bool, optional
+        If nonzero int, `exclude_border` excludes blobs from
+        within `exclude_border`-pixels of the border of the image.
 
     Returns
     -------
-    A : (n, image.ndim + 1) ndarray
-        A 2d array with each row representing 3 values for a 2D image,
-        and 4 values for a 3D image: ``(r, c, sigma)`` or ``(p, r, c, sigma)``
-        where ``(r, c)`` or ``(p, r, c)`` are coordinates of the blob and
-        ``sigma`` is the standard deviation of the Gaussian kernel which
-        detected the blob.
+    A : (n, image.ndim + sigma) ndarray
+        A 2d array with each row representing 2 coordinate values for a 2D
+        image, and 3 coordinate values for a 3D image, plus the sigma(s) used.
+        When a single sigma is passed, outputs are:
+        ``(r, c, sigma)`` or ``(p, r, c, sigma)`` where ``(r, c)`` or
+        ``(p, r, c)`` are coordinates of the blob and ``sigma`` is the standard
+        deviation of the Gaussian kernel which detected the blob. When an
+        anisotropic gaussian is used (sigmas per dimension), the detected sigma
+        is returned for each dimension.
 
     References
     ----------
-    .. [1] http://en.wikipedia.org/wiki/Blob_detection#The_Laplacian_of_Gaussian
+    .. [1] https://en.wikipedia.org/wiki/Blob_detection#The_Laplacian_of_Gaussian
 
     Examples
     --------
@@ -354,31 +420,66 @@ def blob_log(image, min_sigma=1, max_sigma=50, num_sigma=10, threshold=.2,
     """
     image = img_as_float(image)
 
+    # if both min and max sigma are scalar, function returns only one sigma
+    scalar_sigma = (
+        True if np.isscalar(max_sigma) and np.isscalar(min_sigma) else False
+    )
+
+    # Gaussian filter requires that sequence-type sigmas have same
+    # dimensionality as image. This broadcasts scalar kernels
+    if np.isscalar(max_sigma):
+        max_sigma = np.full(image.ndim, max_sigma, dtype=float)
+    if np.isscalar(min_sigma):
+        min_sigma = np.full(image.ndim, min_sigma, dtype=float)
+
+    # Convert sequence types to array
+    min_sigma = np.asarray(min_sigma, dtype=float)
+    max_sigma = np.asarray(max_sigma, dtype=float)
+
     if log_scale:
-        start, stop = log(min_sigma, 10), log(max_sigma, 10)
-        sigma_list = np.logspace(start, stop, num_sigma)
+        # for anisotropic data, we use the "highest resolution/variance" axis
+        standard_axis = np.argmax(min_sigma)
+        start = np.log10(min_sigma[standard_axis])
+        stop = np.log10(max_sigma[standard_axis])
+        scale = np.logspace(start, stop, num_sigma)[:, np.newaxis]
+        sigma_list = scale * min_sigma / np.max(min_sigma)
     else:
-        sigma_list = np.linspace(min_sigma, max_sigma, num_sigma)
+        scale = np.linspace(0, 1, num_sigma)[:, np.newaxis]
+        sigma_list = scale * (max_sigma - min_sigma) + min_sigma
 
     # computing gaussian laplace
-    # s**2 provides scale invariance
-    gl_images = [-gaussian_laplace(image, s) * s ** 2 for s in sigma_list]
+    # average s**2 provides scale invariance
+    gl_images = [-gaussian_laplace(image, s) * np.mean(s) ** 2
+                 for s in sigma_list]
 
     image_cube = np.stack(gl_images, axis=-1)
 
     local_maxima = peak_local_max(image_cube, threshold_abs=threshold,
                                   footprint=np.ones((3,) * (image.ndim + 1)),
                                   threshold_rel=0.0,
-                                  exclude_border=False)
+                                  exclude_border=exclude_border)
 
     # Catch no peaks
     if local_maxima.size == 0:
         return np.empty((0, 3))
+
     # Convert local_maxima to float64
     lm = local_maxima.astype(np.float64)
-    # Convert the last index to its corresponding scale value
-    lm[:, -1] = sigma_list[local_maxima[:, -1]]
-    return _prune_blobs(lm, overlap)
+
+    # translate final column of lm, which contains the index of the
+    # sigma that produced the maximum intensity value, into the sigma
+    sigmas_of_peaks = sigma_list[local_maxima[:, -1]]
+
+    if scalar_sigma:
+        # select one sigma column, keeping dimension
+        sigmas_of_peaks = sigmas_of_peaks[:, 0:1]
+
+    # Remove sigma index and replace with sigmas
+    lm = np.hstack([lm[:, :-1], sigmas_of_peaks])
+
+    sigma_dim = sigmas_of_peaks.shape[1]
+
+    return _prune_blobs(lm, overlap, sigma_dim=sigma_dim)
 
 
 def blob_doh(image, min_sigma=1, max_sigma=30, num_sigma=10, threshold=0.01,
@@ -424,7 +525,7 @@ def blob_doh(image, min_sigma=1, max_sigma=30, num_sigma=10, threshold=0.01,
 
     References
     ----------
-    .. [1] http://en.wikipedia.org/wiki/Blob_detection#The_determinant_of_the_Hessian
+    .. [1] https://en.wikipedia.org/wiki/Blob_detection#The_determinant_of_the_Hessian
 
     .. [2] Herbert Bay, Andreas Ess, Tinne Tuytelaars, Luc Van Gool,
            "SURF: Speeded Up Robust Features"
@@ -463,7 +564,7 @@ def blob_doh(image, min_sigma=1, max_sigma=30, num_sigma=10, threshold=0.01,
     this method can't be used for detecting blobs of radius less than `3px`
     due to the box filters used in the approximation of Hessian Determinant.
     """
-    assert_nD(image, 2)
+    check_nD(image, 2)
 
     image = img_as_float(image)
     image = integral_image(image)
