@@ -103,7 +103,7 @@ def structure_tensor(image, sigma=1, mode='constant', cval=0):
     return Axx, Axy, Ayy
 
 
-def hessian_matrix(image, sigma=1, mode='constant', cval=0, order=None):
+def hessian_matrix(image, sigma=1, mode='constant', cval=0, order='rc'):
     """Compute Hessian matrix.
 
     The Hessian matrix is defined as::
@@ -112,7 +112,7 @@ def hessian_matrix(image, sigma=1, mode='constant', cval=0, order=None):
             [Hrc Hcc]
 
     which is computed by convolving the image with the second derivatives
-    of the Gaussian kernel in the respective x- and y-directions.
+    of the Gaussian kernel in the respective r- and c-directions.
 
     Parameters
     ----------
@@ -126,11 +126,11 @@ def hessian_matrix(image, sigma=1, mode='constant', cval=0, order=None):
     cval : float, optional
         Used in conjunction with mode 'constant', the value outside
         the image boundaries.
-    order : {'xy', 'rc'}, optional
+    order : {'rc', 'xy'}, optional
         This parameter allows for the use of reverse or forward order of
-        the image axes in gradient computation. 'xy' indicates the usage
-        of the last axis initially (Hxx, Hxy, Hyy), whilst 'rc' indicates
-        the use of the first axis initially (Hrr, Hrc, Hcc).
+        the image axes in gradient computation. 'rc' indicates the use of
+        the first axis initially (Hrr, Hrc, Hcc), whilst 'xy' indicates the
+        usage of the last axis initially (Hxx, Hxy, Hyy)
 
     Returns
     -------
@@ -146,7 +146,7 @@ def hessian_matrix(image, sigma=1, mode='constant', cval=0, order=None):
     >>> from skimage.feature import hessian_matrix
     >>> square = np.zeros((5, 5))
     >>> square[2, 2] = 4
-    >>> Hrr, Hrc, Hcc = hessian_matrix(square, sigma=0.1, order = 'rc')
+    >>> Hrr, Hrc, Hcc = hessian_matrix(square, sigma=0.1, order='rc')
     >>> Hrc
     array([[ 0.,  0.,  0.,  0.,  0.],
            [ 0.,  1.,  0., -1.,  0.],
@@ -160,18 +160,6 @@ def hessian_matrix(image, sigma=1, mode='constant', cval=0, order=None):
     gaussian_filtered = ndi.gaussian_filter(image, sigma=sigma,
                                             mode=mode, cval=cval)
 
-    if order is None:
-        if image.ndim == 2:
-            # The legacy 2D code followed (x, y) convention, so we swap the axis
-            # order to maintain compatibility with old code
-            warn('deprecation warning: the default order of the hessian matrix values '
-                 'will be "row-column" instead of "xy" starting in skimage version 0.15. '
-                 'Use order="rc" or order="xy" to set this explicitly')
-            order = 'xy'
-        else:
-            order = 'rc'
-
-
     gradients = np.gradient(gaussian_filtered)
     axes = range(image.ndim)
 
@@ -296,7 +284,7 @@ def structure_tensor_eigvals(Axx, Axy, Ayy):
     return _image_orthogonal_matrix22_eigvals(Axx, Axy, Ayy)
 
 
-def hessian_matrix_eigvals(H_elems, Hxy=None, Hyy=None, Hxx=None):
+def hessian_matrix_eigvals(H_elems):
     """Compute Eigenvalues of Hessian matrix.
 
     Parameters
@@ -304,12 +292,6 @@ def hessian_matrix_eigvals(H_elems, Hxy=None, Hyy=None, Hxx=None):
     H_elems : list of ndarray
         The upper-diagonal elements of the Hessian matrix, as returned
         by `hessian_matrix`.
-    Hxy : ndarray, deprecated
-        Element of the Hessian matrix for each pixel in the input image.
-    Hyy : ndarray, deprecated
-        Element of the Hessian matrix for each pixel in the input image.
-    Hxx : ndarray, deprecated
-        Element of the Hessian matrix for each pixel in the input image.
 
     Returns
     -------
@@ -331,13 +313,6 @@ def hessian_matrix_eigvals(H_elems, Hxy=None, Hyy=None, Hxx=None):
            [ 0.,  1.,  0.,  1.,  0.],
            [ 0.,  0.,  2.,  0.,  0.]])
     """
-    if Hxy is not None:
-        if Hxx is None:
-            Hxx = H_elems
-        H_elems = [Hxx, Hxy, Hyy]
-        warn('The API of `hessian_matrix_eigvals` has changed. Use a list of '
-             'elements instead of separate arguments. The old version of the '
-             'API will be removed in version 0.16.')
     if len(H_elems) == 3:  # Use fast Cython code for 2D
         eigvals = np.array(_image_orthogonal_matrix22_eigvals(*H_elems))
     else:
@@ -399,7 +374,7 @@ def shape_index(image, sigma=1, mode='constant', cval=0):
     .. [1] Koenderink, J. J. & van Doorn, A. J.,
            "Surface shape and curvature scales",
            Image and Vision Computing, 1992, 10, 557-564.
-           DOI:10.1016/0262-8856(92)90076-F
+           :DOI:`10.1016/0262-8856(92)90076-F`
 
     Examples
     --------
@@ -447,6 +422,11 @@ def corner_kitchen_rosenfeld(image, mode='constant', cval=0):
     response : ndarray
         Kitchen and Rosenfeld response image.
 
+    References
+    ----------
+    .. [1] Kitchen, L., & Rosenfeld, A. (1982). Gray-level corner detection.
+           Pattern recognition letters, 1(2), 95-102.
+           :DOI:`10.1016/0167-8655(82)90020-4`
     """
 
     imx, imy = _compute_derivatives(image, mode=mode, cval=cval)
@@ -503,8 +483,7 @@ def corner_harris(image, method='k', k=0.05, eps=1e-6, sigma=1):
 
     References
     ----------
-    .. [1] http://kiwi.cs.dal.ca/~dparks/CornerDetection/harris.htm
-    .. [2] http://en.wikipedia.org/wiki/Corner_detection
+    .. [1] https://en.wikipedia.org/wiki/Corner_detection
 
     Examples
     --------
@@ -573,8 +552,7 @@ def corner_shi_tomasi(image, sigma=1):
 
     References
     ----------
-    .. [1] http://kiwi.cs.dal.ca/~dparks/CornerDetection/harris.htm
-    .. [2] http://en.wikipedia.org/wiki/Corner_detection
+    .. [1] https://en.wikipedia.org/wiki/Corner_detection
 
     Examples
     --------
@@ -639,8 +617,12 @@ def corner_foerstner(image, sigma=1):
 
     References
     ----------
-    .. [1] http://www.ipb.uni-bonn.de/uploads/tx_ikgpublication/foerstner87.fast.pdf
-    .. [2] http://en.wikipedia.org/wiki/Corner_detection
+    .. [1] Förstner, W., & Gülch, E. (1987, June). A fast operator for detection and
+           precise location of distinct points, corners and centres of circular
+           features. In Proc. ISPRS intercommission conference on fast processing of
+           photogrammetric data (pp. 281-305).
+           https://cseweb.ucsd.edu/classes/sp02/cse252/foerstner/foerstner.pdf
+    .. [2] https://en.wikipedia.org/wiki/Corner_detection
 
     Examples
     --------
@@ -695,13 +677,13 @@ def corner_fast(image, n=12, threshold=0.15):
     ----------
     image : 2D ndarray
         Input image.
-    n : int
+    n : int, optional
         Minimum number of consecutive pixels out of 16 pixels on the circle
         that should all be either brighter or darker w.r.t testpixel.
         A point c on the circle is darker w.r.t test pixel p if
         `Ic < Ip - threshold` and brighter if `Ic > Ip + threshold`. Also
         stands for the n in `FAST-n` corner detector.
-    threshold : float
+    threshold : float, optional
         Threshold used in deciding whether the pixels on the circle are
         brighter, darker or similar w.r.t. the test pixel. Decrease the
         threshold when more corners are desired and vice-versa.
@@ -713,8 +695,10 @@ def corner_fast(image, n=12, threshold=0.15):
 
     References
     ----------
-    .. [1] Edward Rosten and Tom Drummond
-           "Machine Learning for high-speed corner detection",
+    .. [1] Rosten, E., & Drummond, T. (2006, May). Machine learning for high-speed
+           corner detection. In European conference on computer vision (pp. 430-443).
+           Springer, Berlin, Heidelberg.
+           :DOI:`10.1007/11744023_34`
            http://www.edwardrosten.com/work/rosten_2006_machine.pdf
     .. [2] Wikipedia, "Features from accelerated segment test",
            https://en.wikipedia.org/wiki/Features_from_accelerated_segment_test
@@ -779,9 +763,12 @@ def corner_subpix(image, corners, window_size=11, alpha=0.99):
 
     References
     ----------
-    .. [1] http://www.ipb.uni-bonn.de/uploads/tx_ikgpublication/\
-           foerstner87.fast.pdf
-    .. [2] http://en.wikipedia.org/wiki/Corner_detection
+    .. [1] Förstner, W., & Gülch, E. (1987, June). A fast operator for detection and
+           precise location of distinct points, corners and centres of circular
+           features. In Proc. ISPRS intercommission conference on fast processing of
+           photogrammetric data (pp. 281-305).
+           https://cseweb.ucsd.edu/classes/sp02/cse252/foerstner/foerstner.pdf
+    .. [2] https://en.wikipedia.org/wiki/Corner_detection
 
     Examples
     --------
@@ -939,6 +926,17 @@ def corner_peaks(image, min_distance=1, threshold_abs=None, threshold_rel=0.1,
     * : *
         See :py:meth:`skimage.feature.peak_local_max`.
 
+    See also
+    --------
+    skimage.feature.peak_local_max
+
+    Notes
+    -----
+    The `num_peaks` limit is applied before suppression of
+    connected peaks. If you want to limit the number of peaks
+    after suppression, you should set `num_peaks=np.inf` and
+    post-process the output of this function.
+
     Examples
     --------
     >>> from skimage.feature import peak_local_max
@@ -1000,8 +998,7 @@ def corner_moravec(image, window_size=1):
 
     References
     ----------
-    .. [1] http://kiwi.cs.dal.ca/~dparks/CornerDetection/moravec.htm
-    .. [2] http://en.wikipedia.org/wiki/Corner_detection
+    .. [1] https://en.wikipedia.org/wiki/Corner_detection
 
     Examples
     --------
