@@ -1,10 +1,10 @@
-# -*- coding: utf-8 -*- """
 """
 Port of Manuel Guizar's code from:
 http://www.mathworks.com/matlabcentral/fileexchange/18401-efficient-subpixel-image-registration-by-cross-correlation
 """
 
 import numpy as np
+from .._shared.fft import fftmodule as fft
 
 
 def _upsampled_dft(data, upsampled_region_size,
@@ -28,7 +28,7 @@ def _upsampled_dft(data, upsampled_region_size,
 
     Parameters
     ----------
-    data : 2D ndarray
+    data : array
         The input data array (DFT of original data) to upsample.
     upsampled_region_size : integer or tuple of integers, optional
         The size of the region to be sampled.  If one integer is provided, it
@@ -41,7 +41,7 @@ def _upsampled_dft(data, upsampled_region_size,
 
     Returns
     -------
-    output : 2D ndarray
+    output : ndarray
             The upsampled DFT of the specified region.
     """
     # if people pass in an integer, expand it to a list of equal-sized sections
@@ -59,20 +59,19 @@ def _upsampled_dft(data, upsampled_region_size,
             raise ValueError("number of axis offsets must be equal to input "
                              "data's number of dimensions.")
 
-    col_kernel = np.exp(
-        (-1j * 2 * np.pi / (data.shape[1] * upsample_factor)) *
-        (np.fft.ifftshift(np.arange(data.shape[1]))[:, None] -
-         np.floor(data.shape[1] / 2)).dot(
-             np.arange(upsampled_region_size[1])[None, :] - axis_offsets[1])
-    )
-    row_kernel = np.exp(
-        (-1j * 2 * np.pi / (data.shape[0] * upsample_factor)) *
-        (np.arange(upsampled_region_size[0])[:, None] - axis_offsets[0]).dot(
-            np.fft.ifftshift(np.arange(data.shape[0]))[None, :] -
-            np.floor(data.shape[0] / 2))
-    )
+    im2pi = 1j * 2 * np.pi
 
-    return row_kernel.dot(data).dot(col_kernel)
+    dim_properties = list(zip(data.shape, upsampled_region_size, axis_offsets))
+
+    for (n_items, ups_size, ax_offset) in dim_properties[::-1]:
+        kernel = ((np.arange(ups_size) - ax_offset)[:, None]
+                  * fft.fftfreq(n_items, upsample_factor))
+        kernel = np.exp(-im2pi * kernel)
+
+        # Equivalent to:
+        #   data[i, j, k] = kernel[i, :] @ data[j, k].T
+        data = np.tensordot(kernel, data, axes=(1, -1))
+    return data
 
 
 def _compute_phasediff(cross_correlation_max):
@@ -107,7 +106,7 @@ def _compute_error(cross_correlation_max, src_amp, target_amp):
 
 
 def register_translation(src_image, target_image, upsample_factor=1,
-                         space="real"):
+                         space="real", return_error=True):
     """
     Efficient subpixel image translation registration by cross-correlation.
 
@@ -119,9 +118,9 @@ def register_translation(src_image, target_image, upsample_factor=1,
 
     Parameters
     ----------
-    src_image : ndarray
+    src_image : array
         Reference image.
-    target_image : ndarray
+    target_image : array
         Image to register.  Must be same dimensionality as ``src_image``.
     upsample_factor : int, optional
         Upsampling factor. Images will be registered to within
@@ -132,6 +131,9 @@ def register_translation(src_image, target_image, upsample_factor=1,
         Defines how the algorithm interprets input data.  "real" means data
         will be FFT'd to compute the correlation, while "fourier" data will
         bypass FFT of input data.  Case insensitive.
+    return_error : bool, optional
+        Returns error and phase difference if on,
+        otherwise only shifts are returned
 
     Returns
     -------
@@ -149,30 +151,23 @@ def register_translation(src_image, target_image, upsample_factor=1,
     ----------
     .. [1] Manuel Guizar-Sicairos, Samuel T. Thurman, and James R. Fienup,
            "Efficient subpixel image registration algorithms,"
-           Optics Letters 33, 156-158 (2008). DOI:10.1364/OL.33.000156
+           Optics Letters 33, 156-158 (2008). :DOI:`10.1364/OL.33.000156`
     .. [2] James R. Fienup, "Invariant error metrics for image reconstruction"
-           Optics Letters 36, 8352-8357 (1997). DOI:10.1364/AO.36.008352
+           Optics Letters 36, 8352-8357 (1997). :DOI:`10.1364/AO.36.008352`
     """
     # images must be the same shape
     if src_image.shape != target_image.shape:
         raise ValueError("Error: images must be same size for "
                          "register_translation")
 
-    # only 2D data makes sense right now
-    if src_image.ndim != 2 and upsample_factor > 1:
-        raise NotImplementedError("Error: register_translation only supports "
-                                  "subpixel registration for 2D images")
-
     # assume complex data is already in Fourier space
     if space.lower() == 'fourier':
         src_freq = src_image
         target_freq = target_image
     # real data needs to be fft'd.
     elif space.lower() == 'real':
-        src_image = np.array(src_image, dtype=np.complex128, copy=False)
-        target_image = np.array(target_image, dtype=np.complex128, copy=False)
-        src_freq = np.fft.fftn(src_image)
-        target_freq = np.fft.fftn(target_image)
+        src_freq = fft.fftn(src_image)
+        target_freq = fft.fftn(target_image)
     else:
         raise ValueError("Error: register_translation only knows the \"real\" "
                          "and \"fourier\" values for the ``space`` argument.")
@@ -180,7 +175,7 @@ def register_translation(src_image, target_image, upsample_factor=1,
     # Whole-pixel shift - Compute cross-correlation by an IFFT
     shape = src_freq.shape
     image_product = src_freq * target_freq.conj()
-    cross_correlation = np.fft.ifftn(image_product)
+    cross_correlation = fft.ifftn(image_product)
 
     # Locate maximum
     maxima = np.unravel_index(np.argmax(np.abs(cross_correlation)),
@@ -191,9 +186,10 @@ def register_translation(src_image, target_image, upsample_factor=1,
     shifts[shifts > midpoints] -= np.array(shape)[shifts > midpoints]
 
     if upsample_factor == 1:
-        src_amp = np.sum(np.abs(src_freq) ** 2) / src_freq.size
-        target_amp = np.sum(np.abs(target_freq) ** 2) / target_freq.size
-        CCmax = cross_correlation.max()
+        if return_error:
+            src_amp = np.sum(np.abs(src_freq) ** 2) / src_freq.size
+            target_amp = np.sum(np.abs(target_freq) ** 2) / target_freq.size
+            CCmax = cross_correlation[maxima]
     # If upsampling > 1, then refine estimate with matrix multiply DFT
     else:
         # Initial shift estimate in upsampled grid
@@ -211,19 +207,21 @@ def register_translation(src_image, target_image, upsample_factor=1,
                                            sample_region_offset).conj()
         cross_correlation /= normalization
         # Locate maximum and map back to original pixel grid
-        maxima = np.array(np.unravel_index(
-                              np.argmax(np.abs(cross_correlation)),
-                              cross_correlation.shape),
-                          dtype=np.float64)
-        maxima -= dftshift
+        maxima = np.unravel_index(np.argmax(np.abs(cross_correlation)),
+                                  cross_correlation.shape)
+        CCmax = cross_correlation[maxima]
+
+        maxima = np.array(maxima, dtype=np.float64) - dftshift
+
         shifts = shifts + maxima / upsample_factor
-        CCmax = cross_correlation.max()
-        src_amp = _upsampled_dft(src_freq * src_freq.conj(),
-                                 1, upsample_factor)[0, 0]
-        src_amp /= normalization
-        target_amp = _upsampled_dft(target_freq * target_freq.conj(),
-                                    1, upsample_factor)[0, 0]
-        target_amp /= normalization
+
+        if return_error:
+            src_amp = _upsampled_dft(src_freq * src_freq.conj(),
+                                     1, upsample_factor)[0, 0]
+            src_amp /= normalization
+            target_amp = _upsampled_dft(target_freq * target_freq.conj(),
+                                        1, upsample_factor)[0, 0]
+            target_amp /= normalization
 
     # If its only one row or column the shift along that dimension has no
     # effect. We set to zero.
@@ -231,5 +229,8 @@ def register_translation(src_image, target_image, upsample_factor=1,
         if shape[dim] == 1:
             shifts[dim] = 0
 
-    return shifts, _compute_error(CCmax, src_amp, target_amp),\
-        _compute_phasediff(CCmax)
+    if return_error:
+        return shifts, _compute_error(CCmax, src_amp, target_amp),\
+            _compute_phasediff(CCmax)
+    else:
+        return shifts
