@@ -1,12 +1,15 @@
 import numpy as np
-from numpy.testing import (assert_array_equal, assert_raises,
-                           assert_almost_equal, assert_warns)
-
+from skimage._shared.testing import assert_array_equal
+from skimage._shared.testing import assert_almost_equal, assert_warns
 from skimage import data
 from skimage import img_as_float
+from skimage import draw
 from skimage.color import rgb2gray
 from skimage.morphology import octagon
 from skimage._shared.testing import test_parallel
+from skimage._shared._warnings import expected_warnings
+from skimage._shared import testing
+import pytest
 
 from skimage.feature import (corner_moravec, corner_harris, corner_shi_tomasi,
                              corner_subpix, peak_local_max, corner_peaks,
@@ -17,6 +20,15 @@ from skimage.feature import (corner_moravec, corner_harris, corner_shi_tomasi,
                              hessian_matrix_det, shape_index)
 
 
+@pytest.fixture
+def im3d():
+    r = 10
+    pad = 10
+    im3 = draw.ellipsoid(r, r, r)
+    im3 = np.pad(im3, pad, mode='constant').astype(np.uint8)
+    return im3
+
+
 def test_structure_tensor():
     square = np.zeros((5, 5))
     square[2, 2] = 1
@@ -60,9 +72,6 @@ def test_hessian_matrix():
                                        [0, 0,  0, 0, 0],
                                        [0, 0,  2, 0, 0]]))
 
-    matrix2d = np.random.rand(3, 3)
-    assert_warns(UserWarning, hessian_matrix, matrix2d, sigma=0.1)
-
 
 def test_hessian_matrix_3d():
     cube = np.zeros((5, 5, 5))
@@ -97,8 +106,8 @@ def test_structure_tensor_eigvals():
 def test_hessian_matrix_eigvals():
     square = np.zeros((5, 5))
     square[2, 2] = 4
-    Hrr, Hrc, Hcc = hessian_matrix(square, sigma=0.1, order='rc')
-    l1, l2 = hessian_matrix_eigvals(Hrr, Hrc, Hcc)
+    H = hessian_matrix(square, sigma=0.1, order='rc')
+    l1, l2 = hessian_matrix_eigvals(H)
     assert_almost_equal(l1, np.array([[0, 0,  2, 0, 0],
                                       [0, 1,  0, 1, 0],
                                       [2, 0, -2, 0, 2],
@@ -111,6 +120,26 @@ def test_hessian_matrix_eigvals():
                                       [0,  0,  0,  0, 0]]))
 
 
+def test_hessian_matrix_eigvals_3d(im3d):
+    H = hessian_matrix(im3d)
+    E = hessian_matrix_eigvals(H)
+    # test descending order:
+    e0, e1, e2 = E
+    assert np.all(e0 >= e1) and np.all(e1 >= e2)
+
+    E0, E1, E2 = E[:, E.shape[1] // 2]  # cross section
+    row_center, col_center = np.array(E0.shape) // 2
+    circles = [draw.circle_perimeter(row_center, col_center, radius,
+                                     shape=E0.shape)
+               for radius in range(1, E0.shape[1] // 2 - 1)]
+    response0 = np.array([np.mean(E0[c]) for c in circles])
+    response2 = np.array([np.mean(E2[c]) for c in circles])
+    # eigenvalues are negative just inside the sphere, positive just outside
+    assert np.argmin(response2) < np.argmax(response0)
+    assert np.min(response2) < 0
+    assert np.max(response0) > 0
+
+
 @test_parallel()
 def test_hessian_matrix_det():
     image = np.zeros((5, 5))
@@ -119,10 +148,30 @@ def test_hessian_matrix_det():
     assert_almost_equal(det, 0, decimal=3)
 
 
+def test_hessian_matrix_det_3d(im3d):
+    D = hessian_matrix_det(im3d)
+    D0 = D[D.shape[0] // 2]
+    row_center, col_center = np.array(D0.shape) // 2
+    # testing in 3D is hard. We test this by showing that you get the
+    # expected flat-then-low-then-high 2nd derivative response in a circle
+    # around the midplane of the sphere.
+    circles = [draw.circle_perimeter(row_center, col_center, r, shape=D0.shape)
+               for r in range(1, D0.shape[1] // 2 - 1)]
+    response = np.array([np.mean(D0[c]) for c in circles])
+    lowest = np.argmin(response)
+    highest = np.argmax(response)
+    assert lowest < highest
+    assert response[lowest] < 0
+    assert response[highest] > 0
+
+
 def test_shape_index():
+    # software floating point arm doesn't raise a warning on divide by zero
+    # https://github.com/scikit-image/scikit-image/issues/3335
     square = np.zeros((5, 5))
     square[2, 2] = 4
-    s = shape_index(square, sigma=0.1)
+    with expected_warnings([r'divide by zero|\A\Z', r'invalid value|\A\Z']):
+        s = shape_index(square, sigma=0.1)
     assert_almost_equal(
         s, np.array([[ np.nan, np.nan,   -0.5, np.nan, np.nan],
                      [ np.nan,      0, np.nan,      0, np.nan],
@@ -332,7 +381,8 @@ def test_blank_image_nans():
 
 def test_corner_fast_image_unsupported_error():
     img = np.zeros((20, 20, 3))
-    assert_raises(ValueError, corner_fast, img)
+    with testing.raises(ValueError):
+        corner_fast(img)
 
 
 @test_parallel()
@@ -382,14 +432,18 @@ def test_corner_fast_astronaut():
 
 def test_corner_orientations_image_unsupported_error():
     img = np.zeros((20, 20, 3))
-    assert_raises(ValueError, corner_orientations, img,
-                  np.asarray([[7, 7]]), np.ones((3, 3)))
+    with testing.raises(ValueError):
+        corner_orientations(
+            img,
+            np.asarray([[7, 7]]), np.ones((3, 3)))
 
 
 def test_corner_orientations_even_shape_error():
     img = np.zeros((20, 20))
-    assert_raises(ValueError, corner_orientations, img,
-                  np.asarray([[7, 7]]), np.ones((4, 4)))
+    with testing.raises(ValueError):
+        corner_orientations(
+            img,
+            np.asarray([[7, 7]]), np.ones((4, 4)))
 
 
 @test_parallel()
@@ -423,8 +477,3 @@ def test_corner_orientations_square():
     expected_orientations_degree = np.array([  45.,  135.,  -45., -135.])
     assert_array_equal(actual_orientations_degrees,
                        expected_orientations_degree)
-
-
-if __name__ == '__main__':
-    from numpy import testing
-    testing.run_module_suite()
