@@ -70,7 +70,8 @@ def test_keypoints_orb_less_than_desired_no_of_keypoints():
 
 
 @xfail(condition=arch32,
-       reason=('Known test failure on 32-bit platforms. See links for details:'
+       reason=('Known test failure on 32-bit platforms. See links for '
+               'details: '
                'https://github.com/scikit-image/scikit-image/issues/3091 '
                'https://github.com/scikit-image/scikit-image/issues/2529'))
 def test_descriptor_orb():
@@ -106,6 +107,10 @@ def test_descriptor_orb():
     detector_extractor.detect_and_extract(img)
     assert_equal(exp_descriptors,
                  detector_extractor.descriptors[100:120, 10:20])
+    assert detector_extractor.keypoints.shape[0] == detector_extractor.descriptors.shape[0]
+    assert detector_extractor.keypoints.shape[0] == detector_extractor.orientations.shape[0]
+    assert detector_extractor.keypoints.shape[0] == detector_extractor.responses.shape[0]
+    assert detector_extractor.keypoints.shape[0] == detector_extractor.scales.shape[0]
 
 
 def test_no_descriptors_extracted_orb():
