@@ -13,7 +13,7 @@ class TestPeakLocalMax():
         trivial = np.zeros((25, 25))
         peak_indices = peak.peak_local_max(trivial, min_distance=1, indices=True)
         assert type(peak_indices) is np.ndarray
-        assert not peak_indices     # inherent boolean-ness of empty list
+        assert peak_indices.size == 0
         peaks = peak.peak_local_max(trivial, min_distance=1, indices=False)
         assert (peaks.astype(np.bool) == trivial).all()
 
@@ -48,7 +48,7 @@ class TestPeakLocalMax():
         assert_array_almost_equal(peaks, [(3, 3)])
 
     def test_constant_image(self):
-        image = 128 * np.ones((20, 20), dtype=np.uint8)
+        image = np.full((20, 20), 128, dtype=np.uint8)
         peaks = peak.peak_local_max(image, min_distance=1)
         assert len(peaks) == 0
 
@@ -445,3 +445,15 @@ class TestProminentPeaks(unittest.TestCase):
                                      min_distance=1, threshold_rel=0,
                                      indices=False, exclude_border=False)
         assert np.all(labels == labelsin)
+
+    def test_many_objects(self):
+        mask = np.zeros([500, 500], dtype=bool)
+        x, y = np.indices((500, 500))
+        x_c = x // 20 * 20 + 10
+        y_c = y // 20 * 20 + 10
+        mask[(x - x_c) ** 2 + (y - y_c) ** 2 < 8 ** 2] = True
+        labels, num_objs = ndi.label(mask)
+        dist = ndi.distance_transform_edt(mask)
+        local_max = peak.peak_local_max(dist, min_distance=20, indices=True,
+                                        exclude_border=False, labels=labels)
+        assert len(local_max) == 625
