@@ -1,15 +1,45 @@
 import os
 import numpy as np
-from skimage._shared.testing import assert_equal
+from skimage._shared.testing import (assert_equal, assert_array_equal,
+                                     assert_allclose)
 from skimage._shared import testing
 
 import skimage
-from skimage import img_as_ubyte, img_as_float
+from skimage.util import img_as_ubyte, img_as_float
 from skimage import data, util, morphology
 from skimage.morphology import grey, disk
 from skimage.filters import rank
+from skimage.filters.rank import __all__ as all_rank_filters
 from skimage._shared._warnings import expected_warnings
-from skimage._shared.testing import test_parallel, xfail, arch32
+from skimage._shared.testing import test_parallel, arch32, parametrize, xfail
+from pytest import param
+
+
+def test_otsu_edge_case():
+    # This is an edge case that causes OTSU to appear to misbehave
+    # Pixel [1, 1] may take a value of of 41 or 81. Both should be considered
+    # valid. The value will change depending on the particular implementation
+    # of OTSU.
+    # To better understand, see
+    # https://mybinder.org/v2/gist/hmaarrfk/4afae1cfded1d78e44c9e4f58285d552/master
+
+    selem = np.array([[0, 1, 0],
+                      [1, 1, 1],
+                      [0, 1, 0]], dtype=np.uint8)
+
+    img = np.array([[  0,  41,   0],
+                    [ 30,  81, 106],
+                    [  0, 147,   0]], dtype=np.uint8)
+
+    result = rank.otsu(img, selem)
+    assert result[1, 1] in [41, 81]
+
+    img = np.array([[  0, 214,   0],
+                    [229, 104, 141],
+                    [  0, 172,   0]], dtype=np.uint8)
+    result = rank.otsu(img, selem)
+    assert result[1, 1] in [141, 172]
+
 
 
 class TestRank():
@@ -20,85 +50,41 @@ class TestRank():
         self.image = np.random.rand(25, 25)
         # Set again the seed for the other tests.
         np.random.seed(0)
+        self.selem = morphology.disk(1)
+        self.refs = np.load(os.path.join(skimage.data_dir,
+                                         "rank_filter_tests.npz"))
 
-    @xfail(condition=arch32,
-           reason=('Known test failure on 32-bit platforms. See links for '
-                   'details: '
-                   'https://github.com/scikit-image/scikit-image/issues/3091 '
-                   'https://github.com/scikit-image/scikit-image/issues/2528'))
-    def test_all(self):
-        @test_parallel()
-        def check_all():
-            selem = morphology.disk(1)
-            refs = np.load(os.path.join(skimage.data_dir, "rank_filter_tests.npz"))
+    @parametrize('filter', all_rank_filters)
+    def test_rank_filter(self, filter):
+        @test_parallel(warnings_matching=['Possible precision loss'])
+        def check():
+            expected = self.refs[filter]
+            result = getattr(rank, filter)(self.image, self.selem)
+            if filter == "entropy":
+                # There may be some arch dependent rounding errors
+                # See the discussions in
+                # https://github.com/scikit-image/scikit-image/issues/3091
+                # https://github.com/scikit-image/scikit-image/issues/2528
+                assert_allclose(expected, result, atol=0, rtol=1E-15)
+            elif filter == "otsu":
+                # OTSU May also have some optimization dependent failures
+                # See the discussions in
+                # https://github.com/scikit-image/scikit-image/issues/3091
+                # Pixel 3, 5 was found to be problematic. It can take either
+                # a value of 41 or 81 depending on the specific optimizations
+                # used.
+                assert result[3, 5] in [41, 81]
+                result[3, 5] = 81
+                # Pixel [19, 18] is also found to be problematic for the same
+                # reason.
+                assert result[19, 18] in [141, 172]
+                result[19, 18] = 172
+                assert_array_equal(expected, result)
+            else:
+                assert_array_equal(expected, result)
 
-            assert_equal(refs["autolevel"],
-                         rank.autolevel(self.image, selem))
-            assert_equal(refs["autolevel_percentile"],
-                         rank.autolevel_percentile(self.image, selem))
-            assert_equal(refs["bottomhat"],
-                         rank.bottomhat(self.image, selem))
-            assert_equal(refs["equalize"],
-                         rank.equalize(self.image, selem))
-            assert_equal(refs["gradient"],
-                         rank.gradient(self.image, selem))
-            assert_equal(refs["gradient_percentile"],
-                         rank.gradient_percentile(self.image, selem))
-            assert_equal(refs["maximum"],
-                         rank.maximum(self.image, selem))
-            assert_equal(refs["mean"],
-                         rank.mean(self.image, selem))
-            assert_equal(refs["geometric_mean"],
-                         rank.geometric_mean(self.image, selem)),
-            assert_equal(refs["mean_percentile"],
-                         rank.mean_percentile(self.image, selem))
-            assert_equal(refs["mean_bilateral"],
-                         rank.mean_bilateral(self.image, selem))
-            assert_equal(refs["subtract_mean"],
-                         rank.subtract_mean(self.image, selem))
-            assert_equal(refs["subtract_mean_percentile"],
-                         rank.subtract_mean_percentile(self.image, selem))
-            assert_equal(refs["median"],
-                         rank.median(self.image, selem))
-            assert_equal(refs["minimum"],
-                         rank.minimum(self.image, selem))
-            assert_equal(refs["modal"],
-                         rank.modal(self.image, selem))
-            assert_equal(refs["enhance_contrast"],
-                         rank.enhance_contrast(self.image, selem))
-            assert_equal(refs["enhance_contrast_percentile"],
-                         rank.enhance_contrast_percentile(self.image, selem))
-            assert_equal(refs["pop"],
-                         rank.pop(self.image, selem))
-            assert_equal(refs["pop_percentile"],
-                         rank.pop_percentile(self.image, selem))
-            assert_equal(refs["pop_bilateral"],
-                         rank.pop_bilateral(self.image, selem))
-            assert_equal(refs["sum"],
-                         rank.sum(self.image, selem))
-            assert_equal(refs["sum_bilateral"],
-                         rank.sum_bilateral(self.image, selem))
-            assert_equal(refs["sum_percentile"],
-                         rank.sum_percentile(self.image, selem))
-            assert_equal(refs["threshold"],
-                         rank.threshold(self.image, selem))
-            assert_equal(refs["threshold_percentile"],
-                         rank.threshold_percentile(self.image, selem))
-            assert_equal(refs["tophat"],
-                         rank.tophat(self.image, selem))
-            assert_equal(refs["noise_filter"],
-                         rank.noise_filter(self.image, selem))
-            assert_equal(refs["entropy"],
-                         rank.entropy(self.image, selem))
-            assert_equal(refs["otsu"],
-                         rank.otsu(self.image, selem))
-            assert_equal(refs["percentile"],
-                         rank.percentile(self.image, selem))
-            assert_equal(refs["windowed_histogram"],
-                         rank.windowed_histogram(self.image, selem))
+        check()
 
-        with expected_warnings(['precision loss', 'non-integer|\A\Z']):
-            check_all()
 
     def test_random_sizes(self):
         # make sure the size is not a problem
@@ -182,10 +168,11 @@ class TestRank():
         out = np.empty((100, 100), dtype=np.uint16)
         mask = np.ones((100, 100), dtype=np.uint8)
 
-        for i in range(5):
-            image = np.ones((100, 100), dtype=np.uint16) * 255 * 2 ** i
-            if i > 3:
-                expected = ["Bitdepth of"]
+        for i in range(8, 13):
+            max_val = 2 ** i - 1
+            image = np.full((100, 100), max_val, dtype=np.uint16)
+            if i > 10:
+                expected = ["Bad rank filter performance"]
             else:
                 expected = []
             with expected_warnings(expected):
@@ -244,11 +231,11 @@ class TestRank():
     def test_pass_on_bitdepth(self):
         # should pass because data bitdepth is not too high for the function
 
-        image = np.ones((100, 100), dtype=np.uint16) * 2 ** 11
+        image = np.full((100, 100), 2 ** 11, dtype=np.uint16)
         elem = np.ones((3, 3), dtype=np.uint8)
         out = np.empty_like(image)
         mask = np.ones(image.shape, dtype=np.uint8)
-        with expected_warnings(["Bitdepth of"]):
+        with expected_warnings(["Bad rank filter performance"]):
             rank.maximum(image=image, selem=elem, out=out, mask=mask)
 
 
@@ -302,7 +289,7 @@ class TestRank():
         for method in methods:
             func = getattr(rank, method)
             out_u = func(image_uint, disk(3))
-            with expected_warnings(['precision loss']):
+            with expected_warnings(["Possible precision loss"]):
                 out_f = func(image_float, disk(3))
             assert_equal(out_u, out_f)
 
@@ -315,9 +302,8 @@ class TestRank():
         image = img_as_ubyte(data.camera())[::2, ::2]
         image[image > 127] = 0
         image_s = image.astype(np.int8)
-        with expected_warnings(['sign loss', 'precision loss']):
-            image_u = img_as_ubyte(image_s)
-            assert_equal(image_u, img_as_ubyte(image_s))
+        image_u = img_as_ubyte(image_s)
+        assert_equal(image_u, img_as_ubyte(image_s))
 
         methods = ['autolevel', 'bottomhat', 'equalize', 'gradient', 'maximum',
                    'mean', 'geometric_mean', 'subtract_mean', 'median', 'minimum',
@@ -325,30 +311,26 @@ class TestRank():
 
         for method in methods:
             func = getattr(rank, method)
-
-            with expected_warnings(['sign loss', 'precision loss']):
-                out_u = func(image_u, disk(3))
+            out_u = func(image_u, disk(3))
+            with expected_warnings(["Possible precision loss"]):
                 out_s = func(image_s, disk(3))
             assert_equal(out_u, out_s)
 
-
-    def test_compare_8bit_vs_16bit(self):
+    @parametrize('method',
+                 ['autolevel', 'bottomhat', 'equalize', 'gradient', 'maximum',
+                  'mean', 'subtract_mean', 'median', 'minimum', 'modal',
+                  'enhance_contrast', 'pop', 'threshold', 'tophat'])
+    def test_compare_8bit_vs_16bit(self, method):
         # filters applied on 8-bit image ore 16-bit image (having only real 8-bit
         # of dynamic) should be identical
-
         image8 = util.img_as_ubyte(data.camera())[::2, ::2]
         image16 = image8.astype(np.uint16)
         assert_equal(image8, image16)
 
-        methods = ['autolevel', 'bottomhat', 'equalize', 'gradient', 'maximum',
-                   'mean', 'subtract_mean', 'median', 'minimum', 'modal',
-                   'enhance_contrast', 'pop', 'threshold', 'tophat']
-
-        for method in methods:
-            func = getattr(rank, method)
-            f8 = func(image8, disk(3))
-            f16 = func(image16, disk(3))
-            assert_equal(f8, f16)
+        func = getattr(rank, method)
+        f8 = func(image8, disk(3))
+        f16 = func(image16, disk(3))
+        assert_equal(f8, f16)
 
 
     def test_trivial_selem8(self):
@@ -530,11 +512,11 @@ class TestRank():
         selem = np.ones((64, 64), dtype=np.uint8)
         data = np.zeros((65, 65), dtype=np.uint16)
         data[:64, :64] = np.reshape(np.arange(4096), (64, 64))
-        with expected_warnings(['Bitdepth of 11']):
+        with expected_warnings(['Bad rank filter performance']):
             assert(np.max(rank.entropy(data, selem)) == 12)
 
         # make sure output is of dtype double
-        with expected_warnings(['Bitdepth of 11']):
+        with expected_warnings(['Bad rank filter performance']):
             out = rank.entropy(data, np.ones((16, 16), dtype=np.uint8))
         assert out.dtype == np.double
 
@@ -569,8 +551,8 @@ class TestRank():
         for bitdepth in range(17):
             value = 2 ** bitdepth - 1
             image[10, 10] = value
-            if bitdepth > 11:
-                expected = ['Bitdepth of %s' % (bitdepth - 1)]
+            if bitdepth >= 11:
+                expected = ['Bad rank filter performance']
             else:
                 expected = []
             with expected_warnings(expected):
@@ -732,3 +714,11 @@ class TestRank():
         assert_equal(rank.median(a), rank.median(a, full_selem))
         assert rank.median(a)[1, 1] == 0
         assert rank.median(a, disk(1))[1, 1] == 1
+
+
+    def test_majority(self):
+        img = data.camera()
+        elem = np.ones((3, 3), dtype=np.uint8)
+        expected = rank.windowed_histogram(
+            img, elem).argmax(-1).astype(np.uint8)
+        assert_equal(expected, rank.majority(img, elem))
