@@ -41,9 +41,8 @@ def test_sobel_zeros():
 
 def test_sobel_mask():
     """Sobel on a masked array should be zero."""
-    np.random.seed(0)
     result = filters.sobel(np.random.uniform(size=(10, 10)),
-                           np.zeros((10, 10), bool))
+                           np.zeros((10, 10), dtype=bool))
     assert (np.all(result == 0))
 
 
@@ -52,7 +51,7 @@ def test_sobel_horizontal():
     i, j = np.mgrid[-5:6, -5:6]
     image = (i >= 0).astype(float)
     result = filters.sobel(image) * np.sqrt(2)
-    # Fudge the eroded points
+    # Check if result match transform direction
     i[np.abs(j) == 5] = 10000
     assert_allclose(result[i == 0], 1)
     assert (np.all(result[np.abs(i) > 1] == 0))
@@ -70,15 +69,14 @@ def test_sobel_vertical():
 
 def test_sobel_h_zeros():
     """Horizontal sobel on an array of all zeros."""
-    result = filters.sobel_h(np.zeros((10, 10)), np.ones((10, 10), bool))
+    result = filters.sobel_h(np.zeros((10, 10)), np.ones((10, 10), dtype=bool))
     assert (np.all(result == 0))
 
 
 def test_sobel_h_mask():
     """Horizontal Sobel on a masked array should be zero."""
-    np.random.seed(0)
     result = filters.sobel_h(np.random.uniform(size=(10, 10)),
-                             np.zeros((10, 10), bool))
+                             np.zeros((10, 10), dtype=bool))
     assert (np.all(result == 0))
 
 
@@ -87,7 +85,7 @@ def test_sobel_h_horizontal():
     i, j = np.mgrid[-5:6, -5:6]
     image = (i >= 0).astype(float)
     result = filters.sobel_h(image)
-    # Fudge the eroded points
+    # Check if result match transform direction
     i[np.abs(j) == 5] = 10000
     assert (np.all(result[i == 0] == 1))
     assert (np.all(result[np.abs(i) > 1] == 0))
@@ -103,15 +101,14 @@ def test_sobel_h_vertical():
 
 def test_sobel_v_zeros():
     """Vertical sobel on an array of all zeros."""
-    result = filters.sobel_v(np.zeros((10, 10)), np.ones((10, 10), bool))
+    result = filters.sobel_v(np.zeros((10, 10)), np.ones((10, 10), dtype=bool))
     assert_allclose(result, 0)
 
 
 def test_sobel_v_mask():
     """Vertical Sobel on a masked array should be zero."""
-    np.random.seed(0)
     result = filters.sobel_v(np.random.uniform(size=(10, 10)),
-                             np.zeros((10, 10), bool))
+                             np.zeros((10, 10), dtype=bool))
     assert_allclose(result, 0)
 
 
@@ -120,7 +117,7 @@ def test_sobel_v_vertical():
     i, j = np.mgrid[-5:6, -5:6]
     image = (j >= 0).astype(float)
     result = filters.sobel_v(image)
-    # Fudge the eroded points
+    # Check if result match transform direction
     j[np.abs(i) == 5] = 10000
     assert (np.all(result[j == 0] == 1))
     assert (np.all(result[np.abs(j) > 1] == 0))
@@ -136,15 +133,14 @@ def test_sobel_v_horizontal():
 
 def test_scharr_zeros():
     """Scharr on an array of all zeros."""
-    result = filters.scharr(np.zeros((10, 10)), np.ones((10, 10), bool))
+    result = filters.scharr(np.zeros((10, 10)), np.ones((10, 10), dtype=bool))
     assert (np.all(result < 1e-16))
 
 
 def test_scharr_mask():
     """Scharr on a masked array should be zero."""
-    np.random.seed(0)
     result = filters.scharr(np.random.uniform(size=(10, 10)),
-                            np.zeros((10, 10), bool))
+                            np.zeros((10, 10), dtype=bool))
     assert_allclose(result, 0)
 
 
@@ -153,7 +149,7 @@ def test_scharr_horizontal():
     i, j = np.mgrid[-5:6, -5:6]
     image = (i >= 0).astype(float)
     result = filters.scharr(image) * np.sqrt(2)
-    # Fudge the eroded points
+    # Check if result match transform direction
     i[np.abs(j) == 5] = 10000
     assert_allclose(result[i == 0], 1)
     assert (np.all(result[np.abs(i) > 1] == 0))
@@ -171,15 +167,15 @@ def test_scharr_vertical():
 
 def test_scharr_h_zeros():
     """Horizontal Scharr on an array of all zeros."""
-    result = filters.scharr_h(np.zeros((10, 10)), np.ones((10, 10), bool))
+    result = filters.scharr_h(np.zeros((10, 10)),
+                              np.ones((10, 10), dtype=bool))
     assert_allclose(result, 0)
 
 
 def test_scharr_h_mask():
     """Horizontal Scharr on a masked array should be zero."""
-    np.random.seed(0)
     result = filters.scharr_h(np.random.uniform(size=(10, 10)),
-                              np.zeros((10, 10), bool))
+                              np.zeros((10, 10), dtype=bool))
     assert_allclose(result, 0)
 
 
@@ -188,7 +184,7 @@ def test_scharr_h_horizontal():
     i, j = np.mgrid[-5:6, -5:6]
     image = (i >= 0).astype(float)
     result = filters.scharr_h(image)
-    # Fudge the eroded points
+    # Check if result match transform direction
     i[np.abs(j) == 5] = 10000
     assert (np.all(result[i == 0] == 1))
     assert (np.all(result[np.abs(i) > 1] == 0))
@@ -204,15 +200,15 @@ def test_scharr_h_vertical():
 
 def test_scharr_v_zeros():
     """Vertical Scharr on an array of all zeros."""
-    result = filters.scharr_v(np.zeros((10, 10)), np.ones((10, 10), bool))
+    result = filters.scharr_v(np.zeros((10, 10)),
+                              np.ones((10, 10), dtype=bool))
     assert_allclose(result, 0)
 
 
 def test_scharr_v_mask():
     """Vertical Scharr on a masked array should be zero."""
-    np.random.seed(0)
     result = filters.scharr_v(np.random.uniform(size=(10, 10)),
-                              np.zeros((10, 10), bool))
+                              np.zeros((10, 10), dtype=bool))
     assert_allclose(result, 0)
 
 
@@ -221,7 +217,7 @@ def test_scharr_v_vertical():
     i, j = np.mgrid[-5:6, -5:6]
     image = (j >= 0).astype(float)
     result = filters.scharr_v(image)
-    # Fudge the eroded points
+    # Check if result match transform direction
     j[np.abs(i) == 5] = 10000
     assert (np.all(result[j == 0] == 1))
     assert (np.all(result[np.abs(j) > 1] == 0))
@@ -237,15 +233,15 @@ def test_scharr_v_horizontal():
 
 def test_prewitt_zeros():
     """Prewitt on an array of all zeros."""
-    result = filters.prewitt(np.zeros((10, 10)), np.ones((10, 10), bool))
+    result = filters.prewitt(np.zeros((10, 10)),
+                             np.ones((10, 10), dtype=bool))
     assert_allclose(result, 0)
 
 
 def test_prewitt_mask():
     """Prewitt on a masked array should be zero."""
-    np.random.seed(0)
     result = filters.prewitt(np.random.uniform(size=(10, 10)),
-                             np.zeros((10, 10), bool))
+                             np.zeros((10, 10), dtype=bool))
     assert_allclose(np.abs(result), 0)
 
 
@@ -254,7 +250,7 @@ def test_prewitt_horizontal():
     i, j = np.mgrid[-5:6, -5:6]
     image = (i >= 0).astype(float)
     result = filters.prewitt(image) * np.sqrt(2)
-    # Fudge the eroded points
+    # Check if result match transform direction
     i[np.abs(j) == 5] = 10000
     assert (np.all(result[i == 0] == 1))
     assert_allclose(result[np.abs(i) > 1], 0, atol=1e-10)
@@ -272,15 +268,15 @@ def test_prewitt_vertical():
 
 def test_prewitt_h_zeros():
     """Horizontal prewitt on an array of all zeros."""
-    result = filters.prewitt_h(np.zeros((10, 10)), np.ones((10, 10), bool))
+    result = filters.prewitt_h(np.zeros((10, 10)),
+                               np.ones((10, 10), dtype=bool))
     assert_allclose(result, 0)
 
 
 def test_prewitt_h_mask():
     """Horizontal prewitt on a masked array should be zero."""
-    np.random.seed(0)
     result = filters.prewitt_h(np.random.uniform(size=(10, 10)),
-                               np.zeros((10, 10), bool))
+                               np.zeros((10, 10), dtype=bool))
     assert_allclose(result, 0)
 
 
@@ -289,7 +285,7 @@ def test_prewitt_h_horizontal():
     i, j = np.mgrid[-5:6, -5:6]
     image = (i >= 0).astype(float)
     result = filters.prewitt_h(image)
-    # Fudge the eroded points
+    # Check if result match transform direction
     i[np.abs(j) == 5] = 10000
     assert (np.all(result[i == 0] == 1))
     assert_allclose(result[np.abs(i) > 1], 0, atol=1e-10)
@@ -305,15 +301,15 @@ def test_prewitt_h_vertical():
 
 def test_prewitt_v_zeros():
     """Vertical prewitt on an array of all zeros."""
-    result = filters.prewitt_v(np.zeros((10, 10)), np.ones((10, 10), bool))
+    result = filters.prewitt_v(np.zeros((10, 10)),
+                               np.ones((10, 10), dtype=bool))
     assert_allclose(result, 0)
 
 
 def test_prewitt_v_mask():
     """Vertical prewitt on a masked array should be zero."""
-    np.random.seed(0)
     result = filters.prewitt_v(np.random.uniform(size=(10, 10)),
-                               np.zeros((10, 10), bool))
+                               np.zeros((10, 10), dtype=bool))
     assert_allclose(result, 0)
 
 
@@ -322,7 +318,7 @@ def test_prewitt_v_vertical():
     i, j = np.mgrid[-5:6, -5:6]
     image = (j >= 0).astype(float)
     result = filters.prewitt_v(image)
-    # Fudge the eroded points
+    # Check if result match transform direction
     j[np.abs(i) == 5] = 10000
     assert (np.all(result[j == 0] == 1))
     assert_allclose(result[np.abs(j) > 1], 0, atol=1e-10)
@@ -342,16 +338,16 @@ def test_laplace_zeros():
     image = np.zeros((9, 9))
     image[3:-3, 3:-3] = 1
     result = filters.laplace(image)
-    res_chk = np.array([[ 0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.],
-                        [ 0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.],
-                        [ 0.,  0.,  0., -1., -1., -1.,  0.,  0.,  0.],
-                        [ 0.,  0., -1.,  2.,  1.,  2., -1.,  0.,  0.],
-                        [ 0.,  0., -1.,  1.,  0.,  1., -1.,  0.,  0.],
-                        [ 0.,  0., -1.,  2.,  1.,  2., -1.,  0.,  0.],
-                        [ 0.,  0.,  0., -1., -1., -1.,  0.,  0.,  0.],
-                        [ 0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.],
-                        [ 0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.]])
-    assert_allclose(result, res_chk)
+    check_result = np.array([[0., 0., 0., 0., 0., 0., 0., 0., 0.],
+                             [0., 0., 0., 0., 0., 0., 0., 0., 0.],
+                             [0., 0., 0., -1., -1., -1., 0., 0., 0.],
+                             [0., 0., -1., 2., 1., 2., -1., 0., 0.],
+                             [0., 0., -1., 1., 0., 1., -1., 0., 0.],
+                             [0., 0., -1., 2., 1., 2., -1., 0., 0.],
+                             [0., 0., 0., -1., -1., -1., 0., 0., 0.],
+                             [0., 0., 0., 0., 0., 0., 0., 0., 0.],
+                             [0., 0., 0., 0., 0., 0., 0., 0., 0.]])
+    assert_allclose(result, check_result)
 
 
 def test_laplace_mask():
@@ -360,43 +356,142 @@ def test_laplace_mask():
     image = np.zeros((9, 9))
     image[3:-3, 3:-3] = 1
     # Define the mask
-    result = filters.laplace(image, ksize=3, mask=np.zeros((9, 9), bool))
+    result = filters.laplace(image, ksize=3, mask=np.zeros((9, 9), dtype=bool))
     assert (np.all(result == 0))
 
 
-@testing.parametrize("grad_func", (
-    filters.prewitt_h, filters.sobel_h, filters.scharr_h
-))
+def test_farid_zeros():
+    """Farid on an array of all zeros."""
+    result = filters.farid(np.zeros((10, 10)),
+                           mask=np.ones((10, 10), dtype=bool))
+    assert (np.all(result == 0))
+
+
+def test_farid_mask():
+    """Farid on a masked array should be zero."""
+    result = filters.farid(np.random.uniform(size=(10, 10)),
+                           mask=np.zeros((10, 10), dtype=bool))
+    assert (np.all(result == 0))
+
+
+def test_farid_horizontal():
+    """Farid on a horizontal edge should be a horizontal line."""
+    i, j = np.mgrid[-5:6, -5:6]
+    image = (i >= 0).astype(float)
+    result = filters.farid(image) * np.sqrt(2)
+    # Check if result match transform direction
+    i[np.abs(j) == 5] = 10000
+    assert (np.all(result[i == 0] == result[i == 0][0]))
+    assert_allclose(result[np.abs(i) > 2], 0, atol=1e-10)
+
+
+def test_farid_vertical():
+    """Farid on a vertical edge should be a vertical line."""
+    i, j = np.mgrid[-5:6, -5:6]
+    image = (j >= 0).astype(float)
+    result = filters.farid(image) * np.sqrt(2)
+    j[np.abs(i) == 5] = 10000
+    assert (np.all(result[j == 0] == result[j == 0][0]))
+    assert_allclose(result[np.abs(j) > 2], 0, atol=1e-10)
+
+
+def test_farid_h_zeros():
+    """Horizontal Farid on an array of all zeros."""
+    result = filters.farid_h(np.zeros((10, 10)),
+                             mask=np.ones((10, 10), dtype=bool))
+    assert (np.all(result == 0))
+
+
+def test_farid_h_mask():
+    """Horizontal Farid on a masked array should be zero."""
+    result = filters.farid_h(np.random.uniform(size=(10, 10)),
+                             mask=np.zeros((10, 10), dtype=bool))
+    assert (np.all(result == 0))
+
+
+def test_farid_h_horizontal():
+    """Horizontal Farid on an edge should be a horizontal line."""
+    i, j = np.mgrid[-5:6, -5:6]
+    image = (i >= 0).astype(float)
+    result = filters.farid_h(image)
+    # Check if result match transform direction
+    i[np.abs(j) == 5] = 10000
+    assert np.all(result[i == 0] == result[i == 0][0])
+    assert_allclose(result[np.abs(i) > 2], 0, atol=1e-10)
+
+
+def test_farid_h_vertical():
+    """Horizontal Farid on a vertical edge should be zero."""
+    i, j = np.mgrid[-5:6, -5:6]
+    image = (j >= 0).astype(float) * np.sqrt(2)
+    result = filters.farid_h(image)
+    assert_allclose(result, 0, atol=1e-10)
+
+
+def test_farid_v_zeros():
+    """Vertical Farid on an array of all zeros."""
+    result = filters.farid_v(np.zeros((10, 10)),
+                             mask=np.ones((10, 10), dtype=bool))
+    assert_allclose(result, 0, atol=1e-10)
+
+
+def test_farid_v_mask():
+    """Vertical Farid on a masked array should be zero."""
+    result = filters.farid_v(np.random.uniform(size=(10, 10)),
+                             mask=np.zeros((10, 10), dtype=bool))
+    assert_allclose(result, 0)
+
+
+def test_farid_v_vertical():
+    """Vertical Farid on an edge should be a vertical line."""
+    i, j = np.mgrid[-5:6, -5:6]
+    image = (j >= 0).astype(float)
+    result = filters.farid_v(image)
+    # Check if result match transform direction
+    j[np.abs(i) == 5] = 10000
+    assert (np.all(result[j == 0] == result[j == 0][0]))
+    assert_allclose(result[np.abs(j) > 2], 0, atol=1e-10)
+
+
+def test_farid_v_horizontal():
+    """vertical Farid on a horizontal edge should be zero."""
+    i, j = np.mgrid[-5:6, -5:6]
+    image = (i >= 0).astype(float)
+    result = filters.farid_v(image)
+    assert_allclose(result, 0, atol=1e-10)
+
+
+@testing.parametrize("grad_func", (filters.prewitt_h, filters.sobel_h,
+                                   filters.scharr_h))
 def test_horizontal_mask_line(grad_func):
     """Horizontal edge filters mask pixels surrounding input mask."""
     vgrad, _ = np.mgrid[:1:11j, :1:11j]  # vertical gradient with spacing 0.1
-    vgrad[5, :] = 1                      # bad horizontal line
+    vgrad[5, :] = 1  # bad horizontal line
 
     mask = np.ones_like(vgrad)
-    mask[5, :] = 0                       # mask bad line
+    mask[5, :] = 0  # mask bad line
 
     expected = np.zeros_like(vgrad)
-    expected[1:-1, 1:-1] = 0.2           # constant gradient for most of image,
-    expected[4:7, 1:-1] = 0              # but line and neighbors masked
+    expected[1:-1, 1:-1] = 0.2  # constant gradient for most of image,
+    expected[4:7, 1:-1] = 0  # but line and neighbors masked
 
     result = grad_func(vgrad, mask)
     assert_allclose(result, expected)
 
 
 @testing.parametrize("grad_func", (
-    filters.prewitt_v, filters.sobel_v, filters.scharr_v
-))
+    filters.prewitt_v, filters.sobel_v, filters.scharr_v))
 def test_vertical_mask_line(grad_func):
     """Vertical edge filters mask pixels surrounding input mask."""
     _, hgrad = np.mgrid[:1:11j, :1:11j]  # horizontal gradient with spacing 0.1
-    hgrad[:, 5] = 1                      # bad vertical line
+    hgrad[:, 5] = 1  # bad vertical line
 
     mask = np.ones_like(hgrad)
-    mask[:, 5] = 0                       # mask bad line
+    mask[:, 5] = 0  # mask bad line
 
     expected = np.zeros_like(hgrad)
-    expected[1:-1, 1:-1] = 0.2           # constant gradient for most of image,
-    expected[1:-1, 4:7] = 0              # but line and neighbors masked
+    expected[1:-1, 1:-1] = 0.2  # constant gradient for most of image,
+    expected[1:-1, 4:7] = 0  # but line and neighbors masked
 
     result = grad_func(hgrad, mask)
     assert_allclose(result, expected)
@@ -406,7 +501,8 @@ def test_range():
     """Output of edge detection should be in [0, 1]"""
     image = np.random.random((100, 100))
     for detector in (filters.sobel, filters.scharr,
-                     filters.prewitt, filters.roberts):
+                     filters.prewitt, filters.roberts,
+                     filters.farid):
         out = detector(image)
         assert_(out.min() >= 0,
                 "Minimum of `{0}` is smaller than zero".format(
