@@ -3,14 +3,15 @@ import math
 import numpy as np
 from scipy import ndimage as ndi
 from collections import OrderedDict
+from collections.abc import Iterable
 from ..exposure import histogram
-from .._shared.utils import assert_nD, warn, deprecated
+from .._shared.utils import check_nD, warn
 from ..transform import integral_image
 from ..util import crop, dtype_limits
+from ..filters._multiotsu import _find_threshold_multiotsu
 
 
 __all__ = ['try_all_threshold',
-           'threshold_adaptive',
            'threshold_otsu',
            'threshold_yen',
            'threshold_isodata',
@@ -21,7 +22,8 @@ __all__ = ['try_all_threshold',
            'threshold_niblack',
            'threshold_sauvola',
            'threshold_triangle',
-           'apply_hysteresis_threshold']
+           'apply_hysteresis_threshold',
+           'threshold_multiotsu']
 
 
 def _try_all(image, methods=None, figsize=None, num_cols=2, verbose=True):
@@ -48,8 +50,10 @@ def _try_all(image, methods=None, figsize=None, num_cols=2, verbose=True):
     """
     from matplotlib import pyplot as plt
 
+    # Handle default value
+    methods = methods or {}
+
     num_rows = math.ceil((len(methods) + 1.) / num_cols)
-    num_rows = int(num_rows)  # Python 2.7 support
     fig, ax = plt.subplots(num_rows, num_cols, figsize=figsize,
                            sharex=True, sharey=True)
     ax = ax.ravel()
@@ -59,8 +63,12 @@ def _try_all(image, methods=None, figsize=None, num_cols=2, verbose=True):
 
     i = 1
     for name, func in methods.items():
-        ax[i].imshow(func(image), cmap=plt.cm.gray)
         ax[i].set_title(name)
+        try:
+            ax[i].imshow(func(image), cmap=plt.cm.gray)
+        except Exception as e:
+            ax[i].text(0.5, 0.5, "%s" % type(e).__name__,
+                       ha="center", va="center", transform=ax[i].transAxes)
         i += 1
         if verbose:
             print(func.__orifunc__)
@@ -151,8 +159,8 @@ def threshold_local(image, block_size, method='gaussian', offset=0,
         Method used to determine adaptive threshold for local neighbourhood in
         weighted mean image.
 
-        * 'generic': use custom function (see `param` parameter)
-        * 'gaussian': apply gaussian filter (see `param` parameter for custom\
+        * 'generic': use custom function (see ``param`` parameter)
+        * 'gaussian': apply gaussian filter (see ``param`` parameter for custom\
                       sigma value)
         * 'mean': apply arithmetic mean filter
         * 'median': apply median rank filter
@@ -181,7 +189,7 @@ def threshold_local(image, block_size, method='gaussian', offset=0,
 
     References
     ----------
-    .. [1] http://docs.opencv.org/modules/imgproc/doc/miscellaneous_transformations.html?highlight=threshold#adaptivethreshold
+    .. [1] https://docs.opencv.org/modules/imgproc/doc/miscellaneous_transformations.html?highlight=threshold#adaptivethreshold
 
     Examples
     --------
@@ -195,7 +203,7 @@ def threshold_local(image, block_size, method='gaussian', offset=0,
     if block_size % 2 == 0:
         raise ValueError("The kwarg ``block_size`` must be odd! Given "
                          "``block_size`` {0} is even.".format(block_size))
-    assert_nD(image, 2)
+    check_nD(image, 2)
     thresh_image = np.zeros(image.shape, 'double')
     if method == 'generic':
         ndi.generic_filter(image, param, block_size,
@@ -225,16 +233,6 @@ def threshold_local(image, block_size, method='gaussian', offset=0,
     return thresh_image - offset
 
 
-@deprecated('threshold_local', removed_version='0.15')
-def threshold_adaptive(image, block_size, method='gaussian', offset=0,
-                       mode='reflect', param=None):
-    warn('The return value of `threshold_local` is a threshold image, while '
-         '`threshold_adaptive` returned the *thresholded* image.')
-    return image > threshold_local(image, block_size=block_size,
-                                   method=method, offset=offset, mode=mode,
-                                   param=param)
-
-
 def threshold_otsu(image, nbins=256):
     """Return threshold value based on Otsu's method.
 
@@ -255,11 +253,11 @@ def threshold_otsu(image, nbins=256):
     Raises
     ------
     ValueError
-         If `image` only contains a single grayscale value.
+         If ``image`` only contains a single grayscale value.
 
     References
     ----------
-    .. [1] Wikipedia, http://en.wikipedia.org/wiki/Otsu's_Method
+    .. [1] Wikipedia, https://en.wikipedia.org/wiki/Otsu's_Method
 
     Examples
     --------
@@ -283,7 +281,7 @@ def threshold_otsu(image, nbins=256):
                          "having more than one color. The input image seems "
                          "to have just one color {0}.".format(image.min()))
 
-    hist, bin_centers = histogram(image.ravel(), nbins)
+    hist, bin_centers = histogram(image.ravel(), nbins, source_range='image')
     hist = hist.astype(float)
 
     # class probabilities for all possible thresholds
@@ -294,8 +292,8 @@ def threshold_otsu(image, nbins=256):
     mean2 = (np.cumsum((hist * bin_centers)[::-1]) / weight2[::-1])[::-1]
 
     # Clip ends to align class 1 and class 2 variables:
-    # The last value of `weight1`/`mean1` should pair with zero values in
-    # `weight2`/`mean2`, which do not exist.
+    # The last value of ``weight1``/``mean1`` should pair with zero values in
+    # ``weight2``/``mean2``, which do not exist.
     variance12 = weight1[:-1] * weight2[1:] * (mean1[:-1] - mean2[1:]) ** 2
 
     idx = np.argmax(variance12)
@@ -324,10 +322,10 @@ def threshold_yen(image, nbins=256):
     ----------
     .. [1] Yen J.C., Chang F.J., and Chang S. (1995) "A New Criterion
            for Automatic Multilevel Thresholding" IEEE Trans. on Image
-           Processing, 4(3): 370-378. DOI:10.1109/83.366472
+           Processing, 4(3): 370-378. :DOI:`10.1109/83.366472`
     .. [2] Sezgin M. and Sankur B. (2004) "Survey over Image Thresholding
            Techniques and Quantitative Performance Evaluation" Journal of
-           Electronic Imaging, 13(1): 146-165, DOI:10.1117/1.1631315
+           Electronic Imaging, 13(1): 146-165, :DOI:`10.1117/1.1631315`
            http://www.busim.ee.boun.edu.tr/~sankur/SankurFolder/Threshold_survey.pdf
     .. [3] ImageJ AutoThresholder code, http://fiji.sc/wiki/index.php/Auto_Threshold
 
@@ -338,9 +336,9 @@ def threshold_yen(image, nbins=256):
     >>> thresh = threshold_yen(image)
     >>> binary = image <= thresh
     """
-    hist, bin_centers = histogram(image.ravel(), nbins)
+    hist, bin_centers = histogram(image.ravel(), nbins, source_range='image')
     # On blank images (e.g. filled with 0) with int dtype, `histogram()`
-    # returns `bin_centers` containing only one value. Speed up with it.
+    # returns ``bin_centers`` containing only one value. Speed up with it.
     if bin_centers.size == 1:
         return bin_centers[0]
 
@@ -350,8 +348,8 @@ def threshold_yen(image, nbins=256):
     P1_sq = np.cumsum(pmf ** 2)
     # Get cumsum calculated from end of squared array:
     P2_sq = np.cumsum(pmf[::-1] ** 2)[::-1]
-    # P2_sq indexes is shifted +1. I assume, with P1[:-1] it's help avoid '-inf'
-    # in crit. ImageJ Yen implementation replaces those values by zero.
+    # P2_sq indexes is shifted +1. I assume, with P1[:-1] it's help avoid
+    # '-inf' in crit. ImageJ Yen implementation replaces those values by zero.
     crit = np.log(((P1_sq[:-1] * P2_sq[1:]) ** -1) *
                   (P1[:-1] * (1.0 - P1[:-1])) ** 2)
     return bin_centers[crit.argmax()]
@@ -394,12 +392,12 @@ def threshold_isodata(image, nbins=256, return_all=False):
     .. [1] Ridler, TW & Calvard, S (1978), "Picture thresholding using an
            iterative selection method"
            IEEE Transactions on Systems, Man and Cybernetics 8: 630-632,
-           DOI:10.1109/TSMC.1978.4310039
+           :DOI:`10.1109/TSMC.1978.4310039`
     .. [2] Sezgin M. and Sankur B. (2004) "Survey over Image Thresholding
            Techniques and Quantitative Performance Evaluation" Journal of
            Electronic Imaging, 13(1): 146-165,
            http://www.busim.ee.boun.edu.tr/~sankur/SankurFolder/Threshold_survey.pdf
-           DOI:10.1117/1.1631315
+           :DOI:`10.1117/1.1631315`
     .. [3] ImageJ AutoThresholder code,
            http://fiji.sc/wiki/index.php/Auto_Threshold
 
@@ -410,7 +408,7 @@ def threshold_isodata(image, nbins=256, return_all=False):
     >>> thresh = threshold_isodata(image)
     >>> binary = image > thresh
     """
-    hist, bin_centers = histogram(image.ravel(), nbins)
+    hist, bin_centers = histogram(image.ravel(), nbins, source_range='image')
 
     # image only contains one unique value
     if len(bin_centers) == 1:
@@ -424,7 +422,7 @@ def threshold_isodata(image, nbins=256, return_all=False):
     # csuml and csumh contain the count of pixels in that bin or lower, and
     # in all bins strictly higher than that bin, respectively
     csuml = np.cumsum(hist)
-    csumh = np.cumsum(hist[::-1])[::-1] - hist
+    csumh = csuml[-1] - csuml
 
     # intensity_sum contains the total pixel intensity from each bin
     intensity_sum = hist * bin_centers
@@ -436,11 +434,12 @@ def threshold_isodata(image, nbins=256, return_all=False):
     # last bin of csumh, which is zero by construction.
     # So no worries about division by zero in the following lines, except
     # for the last bin, but we can ignore that because no valid threshold
-    # can be in the top bin. So we just patch up csumh[-1] to not cause 0/0
-    # errors.
-    csumh[-1] = 1
-    l = np.cumsum(intensity_sum) / csuml
-    h = (np.cumsum(intensity_sum[::-1])[::-1] - intensity_sum) / csumh
+    # can be in the top bin.
+    # To avoid the division by zero, we simply skip over the last element in
+    # all future computation.
+    csum_intensity = np.cumsum(intensity_sum)
+    lower = csum_intensity[:-1] / csuml[:-1]
+    higher = (csum_intensity[-1] - csum_intensity[:-1]) / csumh[:-1]
 
     # isodata finds threshold values that meet the criterion t = (l + m)/2
     # where l is the mean of all pixels <= t and h is the mean of all pixels
@@ -449,7 +448,7 @@ def threshold_isodata(image, nbins=256, return_all=False):
     # were calculated -- which is, of course, the histogram bin centers.
     # We only require this equality to be within the precision of the bin
     # width, of course.
-    all_mean = (l + h) / 2.0
+    all_mean = (lower + higher) / 2.0
     bin_width = bin_centers[1] - bin_centers[0]
 
     # Look only at thresholds that are below the actual all_mean value,
@@ -457,8 +456,8 @@ def threshold_isodata(image, nbins=256, return_all=False):
     # group. Otherwise can get thresholds that are not actually fixed-points
     # of the isodata algorithm. For float images, this matters less, since
     # there really can't be any guarantees anymore anyway.
-    distances = all_mean - bin_centers
-    thresholds = bin_centers[(distances >= 0) & (distances < bin_width)]
+    distances = all_mean - bin_centers[:-1]
+    thresholds = bin_centers[:-1][(distances >= 0) & (distances < bin_width)]
 
     if return_all:
         return thresholds
@@ -495,7 +494,7 @@ def _cross_entropy(image, threshold, bins=_DEFAULT_ENTROPY_BINS):
 
     Notes
     -----
-    See Li and Lee, 1993 [1]_; this is the objective function `threshold_li`
+    See Li and Lee, 1993 [1]_; this is the objective function ``threshold_li``
     minimizes. This function can be improved but this implementation most
     closely matches equation 8 in [1]_ and equations 1-3 in [2]_.
 
@@ -521,7 +520,8 @@ def _cross_entropy(image, threshold, bins=_DEFAULT_ENTROPY_BINS):
     return nu
 
 
-def threshold_li(image):
+def threshold_li(image, *, tolerance=None, initial_guess=None,
+                 iter_callback=None):
     """Compute threshold value by Li's iterative Minimum Cross Entropy method.
 
     Parameters
@@ -531,8 +531,23 @@ def threshold_li(image):
 
     tolerance : float, optional
         Finish the computation when the change in the threshold in an iteration
-        is less than this value. By default, this is half of the range of the
-        input image, divided by 256.
+        is less than this value. By default, this is half the smallest
+        difference between intensity values in ``image``.
+
+    initial_guess : float or Callable[[array[float]], float], optional
+        Li's iterative method uses gradient descent to find the optimal
+        threshold. If the image intensity histogram contains more than two
+        modes (peaks), the gradient descent could get stuck in a local optimum.
+        An initial guess for the iteration can help the algorithm find the
+        globally-optimal threshold. A float value defines a specific start
+        point, while a callable should take in an array of image intensities
+        and return a float value. Example valid callables include
+        ``numpy.mean`` (default), ``lambda arr: numpy.quantile(arr, 0.95)``,
+        or even :func:`skimage.filters.threshold_otsu`.
+
+    iter_callback : Callable[[float], Any], optional
+        A function that will be called on the threshold at every iteration of
+        the algorithm.
 
     Returns
     -------
@@ -544,14 +559,14 @@ def threshold_li(image):
     ----------
     .. [1] Li C.H. and Lee C.K. (1993) "Minimum Cross Entropy Thresholding"
            Pattern Recognition, 26(4): 617-625
-           DOI:10.1016/0031-3203(93)90115-D
+           :DOI:`10.1016/0031-3203(93)90115-D`
     .. [2] Li C.H. and Tam P.K.S. (1998) "An Iterative Algorithm for Minimum
            Cross Entropy Thresholding" Pattern Recognition Letters, 18(8): 771-776
-           DOI:10.1016/S0167-8655(98)00057-9
+           :DOI:`10.1016/S0167-8655(98)00057-9`
     .. [3] Sezgin M. and Sankur B. (2004) "Survey over Image Thresholding
            Techniques and Quantitative Performance Evaluation" Journal of
            Electronic Imaging, 13(1): 146-165
-           DOI:10.1117/1.1631315
+           :DOI:`10.1117/1.1631315`
     .. [4] ImageJ AutoThresholder code, http://fiji.sc/wiki/index.php/Auto_Threshold
 
     Examples
@@ -582,12 +597,34 @@ def threshold_li(image):
     # Li's algorithm requires positive image (because of log(mean))
     image_min = np.min(image)
     image -= image_min
-    image_range = np.max(image)
-    tolerance = 0.5 * image_range / 256
+    tolerance = tolerance or np.min(np.diff(np.unique(image))) / 2
 
-    # Initial estimate
-    t_curr = np.mean(image)
-    t_next = t_curr + 2 * tolerance
+    # Initial estimate for iteration. See "initial_guess" in the parameter list
+    if initial_guess is None:
+        t_next = np.mean(image)
+    elif callable(initial_guess):
+        t_next = initial_guess(image)
+    elif np.isscalar(initial_guess):  # convert to new, positive image range
+        t_next = initial_guess - image_min
+        image_max = np.max(image) + image_min
+        if not 0 < t_next < np.max(image):
+            msg = ('The initial guess for threshold_li must be within the '
+                   'range of the image. Got {} for image min {} and max {} '
+                   .format(initial_guess, image_min, image_max))
+            raise ValueError(msg)
+    else:
+        raise TypeError('Incorrect type for `initial_guess`; should be '
+                        'a floating point value, or a function mapping an '
+                        'array to a floating point value.')
+
+    # initial value for t_curr must be different from t_next by at
+    # least the tolerance. Since the image is positive, we ensure this
+    # by setting to a large-enough negative number
+    t_curr = -2 * tolerance
+
+    # Callback on initial iterations
+    if iter_callback is not None:
+        iter_callback(t_next + image_min)
 
     # Stop the iterations when the difference between the
     # new and old threshold values is less than the tolerance
@@ -600,6 +637,9 @@ def threshold_li(image):
         t_next = ((mean_back - mean_fore) /
                   (np.log(mean_back) - np.log(mean_fore)))
 
+        if iter_callback is not None:
+            iter_callback(t_next + image_min)
+
     threshold = t_next + image_min
     return threshold
 
@@ -607,8 +647,9 @@ def threshold_li(image):
 def threshold_minimum(image, nbins=256, max_iter=10000):
     """Return threshold value based on minimum method.
 
-    The histogram of the input `image` is computed and smoothed until there are
-    only two maxima. Then the minimum in between is the threshold value.
+    The histogram of the input ``image`` is computed and smoothed until
+    there are only two maxima. Then the minimum in between is the threshold
+    value.
 
     Parameters
     ----------
@@ -639,7 +680,7 @@ def threshold_minimum(image, nbins=256, max_iter=10000):
            vol. 55, pp. 532-537, 1993.
     .. [2] Prewitt, JMS & Mendelsohn, ML (1966), "The analysis of cell
            images", Annals of the New York Academy of Sciences 128: 1035-1053
-           DOI:10.1111/j.1749-6632.1965.tb11715.x
+           :DOI:`10.1111/j.1749-6632.1965.tb11715.x`
 
     Examples
     --------
@@ -666,9 +707,9 @@ def threshold_minimum(image, nbins=256, max_iter=10000):
 
         return maximum_idxs
 
-    hist, bin_centers = histogram(image.ravel(), nbins)
+    hist, bin_centers = histogram(image.ravel(), nbins, source_range='image')
 
-    smooth_hist = np.copy(hist).astype(np.float64)
+    smooth_hist = hist.astype(np.float64, copy=False)
 
     for counter in range(max_iter):
         smooth_hist = ndi.uniform_filter1d(smooth_hist, 3)
@@ -707,7 +748,7 @@ def threshold_mean(image):
     .. [1] C. A. Glasbey, "An analysis of histogram-based thresholding
         algorithms," CVGIP: Graphical Models and Image Processing,
         vol. 55, pp. 532-537, 1993.
-        DOI:10.1006/cgip.1993.1040
+        :DOI:`10.1006/cgip.1993.1040`
 
     Examples
     --------
@@ -741,7 +782,7 @@ def threshold_triangle(image, nbins=256):
     .. [1] Zack, G. W., Rogers, W. E. and Latt, S. A., 1977,
        Automatic Measurement of Sister Chromatid Exchange Frequency,
        Journal of Histochemistry and Cytochemistry 25 (7), pp. 741-753
-       DOI:10.1177/25.7.70454
+       :DOI:`10.1177/25.7.70454`
     .. [2] ImageJ AutoThresholder code,
        http://fiji.sc/wiki/index.php/Auto_Threshold
 
@@ -754,13 +795,13 @@ def threshold_triangle(image, nbins=256):
     """
     # nbins is ignored for integer arrays
     # so, we recalculate the effective nbins.
-    hist, bin_centers = histogram(image.ravel(), nbins)
+    hist, bin_centers = histogram(image.ravel(), nbins, source_range='image')
     nbins = len(hist)
 
     # Find peak, lowest and highest gray levels.
     arg_peak_height = np.argmax(hist)
     peak_height = hist[arg_peak_height]
-    arg_low_level, arg_high_level = np.where(hist>0)[0][[0, -1]]
+    arg_low_level, arg_high_level = np.where(hist > 0)[0][[0, -1]]
 
     # Flip is True if left tail is shorter.
     flip = arg_peak_height - arg_low_level < arg_high_level - arg_peak_height
@@ -796,24 +837,47 @@ def threshold_triangle(image, nbins=256):
     return bin_centers[arg_level]
 
 
+def _validate_window_size(axis_sizes):
+    """Ensure all sizes in ``axis_sizes`` are odd.
+
+    Parameters
+    ----------
+    axis_sizes : iterable of int
+
+    Raises
+    ------
+    ValueError
+        If any given axis size is even.
+    """
+    for axis_size in axis_sizes:
+        if axis_size % 2 == 0:
+            msg = ('Window size for `threshold_sauvola` or '
+                   '`threshold_niblack` must not be even on any dimension. '
+                   'Got {}'.format(axis_sizes))
+            raise ValueError(msg)
+
+
 def _mean_std(image, w):
     """Return local mean and standard deviation of each pixel using a
-    neighborhood defined by a rectangular window with size w times w.
+    neighborhood defined by a rectangular window size ``w``.
     The algorithm uses integral images to speedup computation. This is
-    used by threshold_niblack and threshold_sauvola.
+    used by :func:`threshold_niblack` and :func:`threshold_sauvola`.
 
     Parameters
     ----------
     image : ndarray
         Input image.
-    w : int
-        Odd window size (e.g. 3, 5, 7, ..., 21, ...).
+    w : int, or iterable of int
+        Window size specified as a single odd integer (3, 5, 7, …),
+        or an iterable of length ``image.ndim`` containing only odd
+        integers (e.g. ``(1, 5, 5)``).
 
     Returns
     -------
-    m : 2-D array of same size of image with local mean values.
-    s : 2-D array of same size of image with local standard
-        deviation values.
+    m : ndarray of float, same shape as ``image``
+        Local mean of the image.
+    s : ndarray of float, same shape as ``image``
+        Local standard deviation of the image.
 
     References
     ----------
@@ -821,29 +885,29 @@ def _mean_std(image, w):
            implementation of local adaptive thresholding techniques
            using integral images." in Document Recognition and
            Retrieval XV, (San Jose, USA), Jan. 2008.
-           DOI:10.1117/12.767755
+           :DOI:`10.1117/12.767755`
     """
-    if w == 1 or w % 2 == 0:
-        raise ValueError(
-            "Window size w = %s must be odd and greater than 1." % w)
+    if not isinstance(w, Iterable):
+        w = (w,) * image.ndim
+    _validate_window_size(w)
 
-    left_pad = w // 2 + 1
-    right_pad = w // 2
-    padded = np.pad(image.astype('float'), (left_pad, right_pad),
+    pad_width = tuple((k // 2 + 1, k // 2) for k in w)
+    padded = np.pad(image.astype('float'), pad_width,
                     mode='reflect')
     padded_sq = padded * padded
 
     integral = integral_image(padded)
     integral_sq = integral_image(padded_sq)
 
-    kern = np.zeros((w + 1,) * image.ndim)
+    kern = np.zeros(tuple(k + 1 for k in w))
     for indices in itertools.product(*([[0, -1]] * image.ndim)):
         kern[indices] = (-1) ** (image.ndim % 2 != np.sum(indices) % 2)
 
+    total_window_size = np.prod(w)
     sum_full = ndi.correlate(integral, kern, mode='constant')
-    m = crop(sum_full, (left_pad, right_pad)) / (w ** image.ndim)
+    m = crop(sum_full, pad_width) / total_window_size
     sum_sq_full = ndi.correlate(integral_sq, kern, mode='constant')
-    g2 = crop(sum_sq_full, (left_pad, right_pad)) / (w ** image.ndim)
+    g2 = crop(sum_sq_full, pad_width) / total_window_size
     # Note: we use np.clip because g2 is not guaranteed to be greater than
     # m*m when floating point error is considered
     s = np.sqrt(np.clip(g2 - m * m, 0, None))
@@ -865,10 +929,12 @@ def threshold_niblack(image, window_size=15, k=0.2):
 
     Parameters
     ----------
-    image: (N, M) ndarray
-        Grayscale input image.
-    window_size : int, optional
-        Odd size of pixel neighborhood window (e.g. 3, 5, 7...).
+    image: ndarray
+        Input image.
+    window_size : int, or iterable of int, optional
+        Window size specified as a single odd integer (3, 5, 7, …),
+        or an iterable of length ``image.ndim`` containing only odd
+        integers (e.g. ``(1, 5, 5)``).
     k : float, optional
         Value of parameter k in threshold formula.
 
@@ -882,16 +948,30 @@ def threshold_niblack(image, window_size=15, k=0.2):
     -----
     This algorithm is originally designed for text recognition.
 
+    The Bradley threshold is a particular case of the Niblack
+    one, being equivalent to
+
+    >>> from skimage import data
+    >>> image = data.page()
+    >>> q = 1
+    >>> threshold_image = threshold_niblack(image, k=0) * q
+
+    for some value ``q``. By default, Bradley and Roth use ``q=1``.
+
+
     References
     ----------
-    .. [1] Niblack, W (1986), An introduction to Digital Image
-           Processing, Prentice-Hall.
+    .. [1] W. Niblack, An introduction to Digital Image Processing,
+           Prentice-Hall, 1986.
+    .. [2] D. Bradley and G. Roth, "Adaptive thresholding using Integral
+           Image", Journal of Graphics Tools 12(2), pp. 13-21, 2007.
+           :DOI:`10.1080/2151237X.2007.10129236`
 
     Examples
     --------
     >>> from skimage import data
     >>> image = data.page()
-    >>> binary_image = threshold_niblack(image, window_size=7, k=0.1)
+    >>> threshold_image = threshold_niblack(image, window_size=7, k=0.1)
     """
     m, s = _mean_std(image, window_size)
     return m - k * s
@@ -914,10 +994,12 @@ def threshold_sauvola(image, window_size=15, k=0.2, r=None):
 
     Parameters
     ----------
-    image: (N, M) ndarray
-        Grayscale input image.
-    window_size : int, optional
-        Odd size of pixel neighborhood window (e.g. 3, 5, 7...).
+    image: ndarray
+        Input image.
+    window_size : int, or iterable of int, optional
+        Window size specified as a single odd integer (3, 5, 7, …),
+        or an iterable of length ``image.ndim`` containing only odd
+        integers (e.g. ``(1, 5, 5)``).
     k : float, optional
         Value of the positive parameter k.
     r : float, optional
@@ -939,7 +1021,7 @@ def threshold_sauvola(image, window_size=15, k=0.2, r=None):
     .. [1] J. Sauvola and M. Pietikainen, "Adaptive document image
            binarization," Pattern Recognition 33(2),
            pp. 225-236, 2000.
-           DOI:10.1016/S0031-3203(99)00055-2
+           :DOI:`10.1016/S0031-3203(99)00055-2`
 
     Examples
     --------
@@ -956,25 +1038,25 @@ def threshold_sauvola(image, window_size=15, k=0.2, r=None):
 
 
 def apply_hysteresis_threshold(image, low, high):
-    """Apply hysteresis thresholding to `image`.
+    """Apply hysteresis thresholding to ``image``.
 
-    This algorithm finds regions where `image` is greater than `high`
-    OR `image` is greater than `low` *and* that region is connected to
-    a region greater than `high`.
+    This algorithm finds regions where ``image`` is greater than ``high``
+    OR ``image`` is greater than ``low`` *and* that region is connected to
+    a region greater than ``high``.
 
     Parameters
     ----------
     image : array, shape (M,[ N, ..., P])
         Grayscale input image.
-    low : float, or array of same shape as `image`
+    low : float, or array of same shape as ``image``
         Lower threshold.
-    high : float, or array of same shape as `image`
+    high : float, or array of same shape as ``image``
         Higher threshold.
 
     Returns
     -------
-    thresholded : array of bool, same shape as `image`
-        Array in which `True` indicates the locations where `image`
+    thresholded : array of bool, same shape as ``image``
+        Array in which ``True`` indicates the locations where ``image``
         was above the hysteresis threshold.
 
     Examples
@@ -988,7 +1070,7 @@ def apply_hysteresis_threshold(image, low, high):
     .. [1] J. Canny. A computational approach to edge detection.
            IEEE Transactions on Pattern Analysis and Machine Intelligence.
            1986; vol. 8, pp.679-698.
-           DOI: 10.1109/TPAMI.1986.4767851
+           :DOI:`10.1109/TPAMI.1986.4767851`
     """
     low = np.clip(low, a_min=None, a_max=high)  # ensure low always below high
     mask_low = image > low
@@ -1000,3 +1082,99 @@ def apply_hysteresis_threshold(image, low, high):
     connected_to_high = sums > 0
     thresholded = connected_to_high[labels_low]
     return thresholded
+
+
+def threshold_multiotsu(image, classes=3, nbins=256):
+    r"""Generate `classes`-1 threshold values to divide gray levels in `image`.
+
+    The threshold values are chosen to maximize the total sum of pairwise
+    variances between the thresholded graylevel classes. See Notes and [1]_
+    for more details.
+
+    Parameters
+    ----------
+    image : (N, M) ndarray
+        Grayscale input image.
+    classes : int, optional
+        Number of classes to be thresholded, i.e. the number of resulting
+        regions.
+    nbins : int, optional
+        Number of bins used to calculate the histogram. This value is ignored
+        for integer arrays.
+
+    Returns
+    -------
+    idx_thresh : array
+        Array containing the threshold values for the desired classes.
+
+    Notes
+    -----
+    This implementation relies on a Cython function whose complexity
+    is :math:`O\left(\frac{Ch^{C-1}}{(C-1)!}\right)`, where :math:`h`
+    is the number of histogram bins and :math:`C` is the number of
+    classes desired.
+
+    References
+    ----------
+    .. [1] Liao, P-S., Chen, T-S. and Chung, P-C., "A fast algorithm for
+           multilevel thresholding", Journal of Information Science and
+           Engineering 17 (5): 713-727, 2001. Available at:
+           <http://ftp.iis.sinica.edu.tw/JISE/2001/200109_01.pdf>
+    .. [2] Tosa, Y., "Multi-Otsu Threshold", a java plugin for ImageJ.
+           Available at:
+           <http://imagej.net/plugins/download/Multi_OtsuThreshold.java>
+
+    Examples
+    --------
+    >>> from skimage.color import label2rgb
+    >>> from skimage import data
+    >>> image = data.camera()
+    >>> thresholds = threshold_multiotsu(image)
+    >>> regions = np.digitize(image, bins=thresholds)
+    >>> regions_colorized = label2rgb(regions)
+    """
+    # calculating the histogram and the probability of each gray level.
+    hist, bin_centers = histogram(image.ravel(),
+                                  nbins=nbins,
+                                  source_range='image')
+    prob = hist / image.size
+    # histogram ignores nbins for integer arrays.
+    nbins = len(bin_centers)
+
+    # Compute the zeroth (momP, cumulative probability) and first
+    # (momS, mean) moments
+
+    # building the lookup tables.
+    # step 1: calculating the diagonal.
+    prob[0] = 0
+    momP = np.diagflat(prob)
+    momS = np.diagflat(np.arange(nbins) * prob)
+
+    # step 2: calculating the first row.
+    momP[1, 2:] = prob[1] + np.cumsum(prob[2:])
+    momS[1, 2:] = prob[1] + np.cumsum(np.arange(2, nbins) * prob[2:])
+
+    # step 3: the other rows are recursively computed as:
+    # A[i, j] = A[1, j] - A[1, i-1] for A in {momP, momS};  i > 1 and j > i.
+    upper_tri = np.triu_indices_from(momP[2:, 2:], 1)
+
+    momP[2:, 2:][upper_tri] = (momP[1, 2:][upper_tri[1]]
+                               - np.repeat(momP[1, 1:-2],
+                                           np.arange(nbins - 3, 0, -1)))
+    momS[2:, 2:][upper_tri] = (momS[1, 2:][upper_tri[1]]
+                               - np.repeat(momS[1, 1:-2],
+                                           np.arange(nbins - 3, 0, -1)))
+
+    # step 4: calculating the between class variance.
+    var_btwcls = np.zeros_like(momP)
+    idx = momP > 0
+    var_btwcls[idx] = momS[idx] ** 2 / momP[idx]
+
+    # finding max threshold candidates, depending on classes.
+    # number of thresholds is equal to number of classes - 1.
+    aux_thresh = _find_threshold_multiotsu(var_btwcls, classes, nbins)
+
+    # correcting threshold values.
+    idx_thresh = bin_centers[:-1][aux_thresh.astype('int')]
+
+    return idx_thresh
