@@ -44,7 +44,7 @@ def test_rag_merge():
     g.merge_nodes(1, 4)
     g.merge_nodes(2, 3)
     n = g.merge_nodes(3, 4, in_place=False)
-    assert sorted(g.node[n]['labels']) == list(range(5))
+    assert sorted(g.nodes[n]['labels']) == list(range(5))
     assert list(g.edges()) == []
 
 
@@ -111,20 +111,20 @@ def test_rag_error():
     labels[5:, :] = 1
     with testing.raises(ValueError):
         graph.rag_mean_color(img, labels,
-                             2, 'non existant mode')
+                             2, 'non existent mode')
 
 
 def _weight_mean_color(graph, src, dst, n):
-    diff = graph.node[dst]['mean color'] - graph.node[n]['mean color']
+    diff = graph.nodes[dst]['mean color'] - graph.nodes[n]['mean color']
     diff = np.linalg.norm(diff)
     return {'weight': diff}
 
 
 def _pre_merge_mean_color(graph, src, dst):
-    graph.node[dst]['total color'] += graph.node[src]['total color']
-    graph.node[dst]['pixel count'] += graph.node[src]['pixel count']
-    graph.node[dst]['mean color'] = (graph.node[dst]['total color'] /
-                                     graph.node[dst]['pixel count'])
+    graph.nodes[dst]['total color'] += graph.nodes[src]['total color']
+    graph.nodes[dst]['pixel count'] += graph.nodes[src]['pixel count']
+    graph.nodes[dst]['mean color'] = (graph.nodes[dst]['total color'] /
+                                      graph.nodes[dst]['pixel count'])
 
 
 def merge_hierarchical_mean_color(labels, rag, thresh, rag_copy=True,
@@ -215,8 +215,8 @@ def test_rag_boundary():
     labels[8:, 8:] = 4
 
     g = graph.rag_boundary(labels, edge_map, connectivity=1)
-    assert set(g.nodes()) == set([1, 2, 3, 4])
-    assert set(g.edges()) == set([(1, 2), (1, 3), (2, 4), (3, 4)])
+    assert set(g.nodes()) == {1, 2, 3, 4}
+    assert set(g.edges()) == {(1, 2), (1, 3), (2, 4), (3, 4)}
     assert g[1][3]['weight'] == 0.25
     assert g[2][4]['weight'] == 0.34375
     assert g[1][3]['count'] == 16
