@@ -14,9 +14,9 @@ def configuration(parent_package='', top_path=None):
 
     # This function tries to create C files from the given .pyx files.  If
     # it fails, try to build with pre-generated .c files.
-    cython(['_spath.pyx'], working_path=base_path)
-    cython(['_mcp.pyx'], working_path=base_path)
-    cython(['heap.pyx'], working_path=base_path)
+    cython(['_spath.pyx',
+            '_mcp.pyx',
+            'heap.pyx'], working_path=base_path)
 
     config.add_extension('_spath', sources=['_spath.c'],
                          include_dirs=[get_numpy_include_dirs()])
