@@ -1,7 +1,6 @@
 __all__ = ['imread', 'imsave']
 
 import numpy as np
-from six import string_types
 from PIL import Image
 
 from ...util import img_as_ubyte, img_as_uint
@@ -24,14 +23,14 @@ def imread(fname, dtype=None, img_num=None, **kwargs):
 
     Notes
     -----
-    Files are read using the Python Imaging Libary.
+    Files are read using the Python Imaging Library.
     See PIL docs [1]_ for a list of supported formats.
 
     References
     ----------
     .. [1] http://pillow.readthedocs.org/en/latest/handbook/image-file-formats.html
     """
-    if isinstance(fname, string_types):
+    if isinstance(fname, str):
         with open(fname, 'rb') as f:
             im = Image.open(f)
             return pil_to_ndarray(im, dtype=dtype, img_num=img_num)
@@ -140,7 +139,8 @@ def _palette_is_grayscale(pil_image):
     is_grayscale : bool
         True if all colors in image palette are gray.
     """
-    assert pil_image.mode == 'P'
+    if pil_image.mode != 'P':
+        raise ValueError('pil_image.mode must be equal to "P".')
     # get palette as an array with R, G, B columns
     palette = np.asarray(pil_image.getpalette()).reshape((256, 3))
     # Not all palette colors are used; unused colors have junk values.
@@ -225,7 +225,7 @@ def imsave(fname, arr, format_str=None, **kwargs):
 
     Notes
     -----
-    Use the Python Imaging Libary.
+    Use the Python Imaging Library.
     See PIL docs [1]_ for a list of other supported formats.
     All images besides single channel PNGs are converted using `img_as_uint8`.
     Single Channel PNGs have the following behavior:
@@ -237,10 +237,10 @@ def imsave(fname, arr, format_str=None, **kwargs):
     .. [1] http://pillow.readthedocs.org/en/latest/handbook/image-file-formats.html
     """
     # default to PNG if file-like object
-    if not isinstance(fname, string_types) and format_str is None:
+    if not isinstance(fname, str) and format_str is None:
         format_str = "PNG"
     # Check for png in filename
-    if (isinstance(fname, string_types)
+    if (isinstance(fname, str)
             and fname.lower().endswith(".png")):
         format_str = "PNG"
 
