@@ -6,48 +6,32 @@ from skimage import data_dir
 from skimage.io import imread, imsave, use_plugin, reset_plugins
 
 from skimage._shared import testing
-from skimage._shared.testing import (assert_array_almost_equal, TestCase,
-                                     expected_warnings)
-
-
-try:
-    import imageio as _imageio
-except ImportError:
-    imageio_available = False
-else:
-    imageio_available = True
+from skimage._shared.testing import assert_array_almost_equal, TestCase
+from skimage._shared._warnings import expected_warnings
 
 
 def setup():
-    if imageio_available:
-        np.random.seed(0)
-        use_plugin('imageio')
+    use_plugin('imageio')
 
 
 def teardown():
     reset_plugins()
 
 
-@testing.skipif(not imageio_available, reason="imageio not installed")
-def test_imageio_flatten():
-    # a color image is flattened (as_gray in .16)
-    with expected_warnings(['`flatten` has been deprecated']):
-        img = imread(os.path.join(data_dir, 'color.png'), flatten=True)
+def test_imageio_as_gray():
+    img = imread(os.path.join(data_dir, 'color.png'), as_gray=True)
     assert img.ndim == 2
     assert img.dtype == np.float64
-    with expected_warnings(['`flatten` has been deprecated']):
-        img = imread(os.path.join(data_dir, 'camera.png'), flatten=True)
-    # check that flattening does not occur for an image that is grey already.
+    img = imread(os.path.join(data_dir, 'camera.png'), as_gray=True)
+    # check that conversion does not happen for a gray image
     assert np.sctype2char(img.dtype) in np.typecodes['AllInteger']
 
 
-@testing.skipif(not imageio_available, reason="imageio not installed")
 def test_imageio_palette():
     img = imread(os.path.join(data_dir, 'palette_color.png'))
     assert img.ndim == 3
 
 
-@testing.skipif(not imageio_available, reason="imageio not installed")
 def test_imageio_truncated_jpg():
     # imageio>2.0 uses Pillow / PIL to try and load the file.
     # Oddly, PIL explicitly raises a SyntaxError when the file read fails.
@@ -66,9 +50,9 @@ class TestSave(TestCase):
 
         assert_array_almost_equal((x * scaling).astype(np.int32), y)
 
-    @testing.skipif(not imageio_available, reason="imageio not installed")
     def test_imsave_roundtrip(self):
         dtype = np.uint8
+        np.random.seed(0)
         for shape in [(10, 10), (10, 10, 3), (10, 10, 4)]:
             x = np.ones(shape, dtype=dtype) * np.random.rand(*shape)
 
@@ -77,3 +61,10 @@ class TestSave(TestCase):
             else:
                 x = (x * 255).astype(dtype)
                 yield self.roundtrip, x
+
+
+def test_return_class():
+    testing.assert_equal(
+        type(imread(os.path.join(data_dir, 'color.png'))),
+        np.ndarray
+    )
