@@ -1,27 +1,17 @@
-# coding: utf-8
-from __future__ import division
-from math import sqrt, atan2, pi as PI
-import itertools
 from warnings import warn
+from math import sqrt, atan2, pi as PI
 import numpy as np
 from scipy import ndimage as ndi
 
 from ._label import label
 from . import _moments
 
-
 from functools import wraps
 
+
 __all__ = ['regionprops', 'perimeter']
 
 
-XY_TO_RC_DEPRECATION_MESSAGE = (
-    'regionprops and image moments (including moments, normalized moments, '
-    'central moments, and inertia tensor) of 2D images will change from xy '
-    'coordinates to rc coordinates in version 0.16.\nSee '
-    'http://scikit-image.org/docs/0.14.x/release_notes_and_installation.html#deprecations '
-    'for details on how to avoid this message.'
-)
 STREL_4 = np.array([[0, 1, 0],
                     [1, 1, 1],
                     [0, 1, 0]], dtype=np.uint8)
@@ -46,7 +36,11 @@ PROPS = {
     'FilledImage': 'filled_image',
     'HuMoments': 'moments_hu',
     'Image': 'image',
+    'InertiaTensor': 'inertia_tensor',
+    'InertiaTensorEigvals': 'inertia_tensor_eigvals',
+    'IntensityImage': 'intensity_image',
     'Label': 'label',
+    'LocalCentroid': 'local_centroid',
     'MajorAxisLength': 'major_axis_length',
     'MaxIntensity': 'max_intensity',
     'MeanIntensity': 'mean_intensity',
@@ -58,15 +52,63 @@ PROPS = {
     'Perimeter': 'perimeter',
     # 'PixelIdxList',
     # 'PixelList',
+    'Slice': 'slice',
     'Solidity': 'solidity',
     # 'SubarrayIdx'
     'WeightedCentralMoments': 'weighted_moments_central',
     'WeightedCentroid': 'weighted_centroid',
     'WeightedHuMoments': 'weighted_moments_hu',
+    'WeightedLocalCentroid': 'weighted_local_centroid',
     'WeightedMoments': 'weighted_moments',
     'WeightedNormalizedMoments': 'weighted_moments_normalized'
 }
 
+OBJECT_COLUMNS = {
+    'image', 'coords', 'convex_image', 'slice',
+    'filled_image', 'intensity_image'
+}
+
+COL_DTYPES = {
+    'area': int,
+    'bbox': int,
+    'bbox_area': int,
+    'moments_central': float,
+    'centroid': int,
+    'convex_area': int,
+    'convex_image': object,
+    'coords': object,
+    'eccentricity': float,
+    'equivalent_diameter': float,
+    'euler_number': int,
+    'extent': float,
+    'filled_area': int,
+    'filled_image': object,
+    'moments_hu': float,
+    'image': object,
+    'inertia_tensor': float,
+    'inertia_tensor_eigvals': float,
+    'intensity_image': object,
+    'label': int,
+    'local_centroid': int,
+    'major_axis_length': float,
+    'max_intensity': float,
+    'mean_intensity': float,
+    'min_intensity': float,
+    'minor_axis_length': float,
+    'moments': float,
+    'moments_normalized': float,
+    'orientation': float,
+    'perimeter': float,
+    'slice': object,
+    'solidity': float,
+    'weighted_moments_central': float,
+    'weighted_centroid': int,
+    'weighted_moments_hu': float,
+    'weighted_local_centroid': int,
+    'weighted_moments': int,
+    'weighted_moments_normalized': float
+}
+
 PROP_VALS = set(PROPS.values())
 
 
@@ -94,48 +136,36 @@ def only2d(method):
     return func2d
 
 
-class _RegionProperties(object):
+class RegionProperties:
     """Please refer to `skimage.measure.regionprops` for more information
     on the available region properties.
     """
 
     def __init__(self, slice, label, label_image, intensity_image,
-                 cache_active, coordinates):
+                 cache_active):
 
         if intensity_image is not None:
             if not intensity_image.shape == label_image.shape:
                 raise ValueError('Label and intensity image must have the'
-                                 'same shape.')
+                                 ' same shape.')
 
         self.label = label
 
         self._slice = slice
+        self.slice = slice
         self._label_image = label_image
         self._intensity_image = intensity_image
 
         self._cache_active = cache_active
         self._cache = {}
         self._ndim = label_image.ndim
-        # Note: in PR 2603, we added support for nD moments in regionprops.
-        # Many properties used xy coordinates, instead of rc. This attribute
-        # helps with the deprecation process and should be removed in 0.16.
-        if label_image.ndim > 2 or coordinates == 'rc':
-            self._use_xy_warning = False
-            self._transpose_moments = False
-        elif coordinates == 'xy':
-            self._use_xy_warning = False  # don't warn if 'xy' given explicitly
-            self._transpose_moments = True
-        elif coordinates is None:
-            self._use_xy_warning = True
-            self._transpose_moments = True
-        else:
-            raise ValueError('Incorrect value for regionprops coordinates: %s.'
-                             ' Possible values are: "rc", "xy", or None')
 
+    @property
     @_cached
     def area(self):
         return np.sum(self.image)
 
+    @property
     def bbox(self):
         """
         Returns
@@ -143,29 +173,35 @@ class _RegionProperties(object):
         A tuple of the bounding box's start coordinates for each dimension,
         followed by the end coordinates for each dimension
         """
-        return tuple([self._slice[i].start for i in range(self._ndim)] +
-                     [self._slice[i].stop for i in range(self._ndim)])
+        return tuple([self.slice[i].start for i in range(self._ndim)] +
+                     [self.slice[i].stop for i in range(self._ndim)])
 
+    @property
     def bbox_area(self):
         return self.image.size
 
+    @property
     def centroid(self):
         return tuple(self.coords.mean(axis=0))
 
+    @property
     @_cached
     def convex_area(self):
         return np.sum(self.convex_image)
 
+    @property
     @_cached
     def convex_image(self):
         from ..morphology.convex_hull import convex_hull_image
         return convex_hull_image(self.image)
 
+    @property
     def coords(self):
         indices = np.nonzero(self.image)
-        return np.vstack([indices[i] + self._slice[i].start
+        return np.vstack([indices[i] + self.slice[i].start
                           for i in range(self._ndim)]).T
 
+    @property
     @only2d
     def eccentricity(self):
         l1, l2 = self.inertia_tensor_eigvals
@@ -173,146 +209,163 @@ class _RegionProperties(object):
             return 0
         return sqrt(1 - l2 / l1)
 
+    @property
     def equivalent_diameter(self):
         if self._ndim == 2:
             return sqrt(4 * self.area / PI)
         elif self._ndim == 3:
             return (6 * self.area / PI) ** (1. / 3)
 
+    @property
     def euler_number(self):
         euler_array = self.filled_image != self.image
         _, num = label(euler_array, connectivity=self._ndim, return_num=True,
                        background=0)
         return -num + 1
 
+    @property
     def extent(self):
         return self.area / self.image.size
 
+    @property
     def filled_area(self):
         return np.sum(self.filled_image)
 
+    @property
     @_cached
     def filled_image(self):
         structure = np.ones((3,) * self._ndim)
         return ndi.binary_fill_holes(self.image, structure)
 
+    @property
     @_cached
     def image(self):
-        return self._label_image[self._slice] == self.label
+        return self._label_image[self.slice] == self.label
 
+    @property
     @_cached
     def inertia_tensor(self):
         mu = self.moments_central
         return _moments.inertia_tensor(self.image, mu)
 
+    @property
     @_cached
     def inertia_tensor_eigvals(self):
         return _moments.inertia_tensor_eigvals(self.image,
                                                T=self.inertia_tensor)
 
+    @property
     @_cached
     def intensity_image(self):
         if self._intensity_image is None:
             raise AttributeError('No intensity image specified.')
-        return self._intensity_image[self._slice] * self.image
+        return self._intensity_image[self.slice] * self.image
 
     def _intensity_image_double(self):
         return self.intensity_image.astype(np.double)
 
+    @property
     def local_centroid(self):
         M = self.moments
-        if self._transpose_moments:
-            M = M.T
         return tuple(M[tuple(np.eye(self._ndim, dtype=int))] /
                      M[(0,) * self._ndim])
 
+    @property
     def max_intensity(self):
         return np.max(self.intensity_image[self.image])
 
+    @property
     def mean_intensity(self):
         return np.mean(self.intensity_image[self.image])
 
+    @property
     def min_intensity(self):
         return np.min(self.intensity_image[self.image])
 
+    @property
     def major_axis_length(self):
         l1 = self.inertia_tensor_eigvals[0]
         return 4 * sqrt(l1)
 
+    @property
     def minor_axis_length(self):
         l2 = self.inertia_tensor_eigvals[-1]
         return 4 * sqrt(l2)
 
+    @property
     @_cached
     def moments(self):
         M = _moments.moments(self.image.astype(np.uint8), 3)
-        if self._use_xy_warning:
-            warn(XY_TO_RC_DEPRECATION_MESSAGE)
-        if self._transpose_moments:
-            M = M.T
         return M
 
+    @property
     @_cached
     def moments_central(self):
         mu = _moments.moments_central(self.image.astype(np.uint8),
                                       self.local_centroid, order=3)
-        if self._use_xy_warning:
-            warn(XY_TO_RC_DEPRECATION_MESSAGE)
-        if self._transpose_moments:
-            mu = mu.T
         return mu
 
+    @property
     @only2d
     def moments_hu(self):
         return _moments.moments_hu(self.moments_normalized)
 
+    @property
     @_cached
     def moments_normalized(self):
         return _moments.moments_normalized(self.moments_central, 3)
 
+    @property
     @only2d
     def orientation(self):
         a, b, b, c = self.inertia_tensor.flat
-        sign = -1 if self._transpose_moments else 1
         if a - c == 0:
             if b < 0:
                 return -PI / 4.
             else:
                 return PI / 4.
         else:
-            return sign * 0.5 * atan2(-2 * b, c - a)
+            return 0.5 * atan2(-2 * b, c - a)
 
+    @property
     @only2d
     def perimeter(self):
         return perimeter(self.image, 4)
 
+    @property
     def solidity(self):
         return self.area / self.convex_area
 
+    @property
     def weighted_centroid(self):
         ctr = self.weighted_local_centroid
         return tuple(idx + slc.start
-                     for idx, slc in zip(ctr, self._slice))
+                     for idx, slc in zip(ctr, self.slice))
 
+    @property
     def weighted_local_centroid(self):
         M = self.weighted_moments
         return (M[tuple(np.eye(self._ndim, dtype=int))] /
                 M[(0,) * self._ndim])
 
+    @property
     @_cached
     def weighted_moments(self):
         return _moments.moments(self._intensity_image_double(), 3)
 
+    @property
     @_cached
     def weighted_moments_central(self):
         ctr = self.weighted_local_centroid
         return _moments.moments_central(self._intensity_image_double(),
                                         center=ctr, order=3)
 
+    @property
     @only2d
     def weighted_moments_hu(self):
         return _moments.moments_hu(self.weighted_moments_normalized)
 
+    @property
     @_cached
     def weighted_moments_normalized(self):
         return _moments.moments_normalized(self.weighted_moments_central, 3)
@@ -340,11 +393,11 @@ class _RegionProperties(object):
         value = getattr(self, key, None)
         if value is not None:
             return value
-        else:  # backwards compatability
+        else:  # backwards compatibility
             return getattr(self, PROPS[key])
 
     def __eq__(self, other):
-        if not isinstance(other, _RegionProperties):
+        if not isinstance(other, RegionProperties):
             return False
 
         for key in PROP_VALS:
@@ -358,16 +411,230 @@ class _RegionProperties(object):
         return True
 
 
+# For compatibility with code written prior to 0.16
+_RegionProperties = RegionProperties
+
+
+def _props_to_dict(regions, properties=('label', 'bbox'), separator='-'):
+    """Convert image region properties list into a column dictionary.
+
+    Parameters
+    ----------
+    regions : (N,) list
+        List of RegionProperties objects as returned by :func:`regionprops`.
+    properties : tuple or list of str, optional
+        Properties that will be included in the resulting dictionary
+        For a list of available properties, please see :func:`regionprops`.
+        Users should remember to add "label" to keep track of region
+        identities.
+    separator : str, optional
+        For non-scalar properties not listed in OBJECT_COLUMNS, each element
+        will appear in its own column, with the index of that element separated
+        from the property name by this separator. For example, the inertia
+        tensor of a 2D region will appear in four columns:
+        ``inertia_tensor-0-0``, ``inertia_tensor-0-1``, ``inertia_tensor-1-0``,
+        and ``inertia_tensor-1-1`` (where the separator is ``-``).
+
+        Object columns are those that cannot be split in this way because the
+        number of columns would change depending on the object. For example,
+        ``image`` and ``coords``.
+
+    Returns
+    -------
+    out_dict : dict
+        Dictionary mapping property names to an array of values of that
+        property, one value per region. This dictionary can be used as input to
+        pandas ``DataFrame`` to map property names to columns in the frame and
+        regions to rows.
+
+    Notes
+    -----
+    Each column contains either a scalar property, an object property, or an
+    element in a multidimensional array.
+
+    Properties with scalar values for each region, such as "eccentricity", will
+    appear as a float or int array with that property name as key.
+
+    Multidimensional properties *of fixed size* for a given image dimension,
+    such as "centroid" (every centroid will have three elements in a 3D image,
+    no matter the region size), will be split into that many columns, with the
+    name {property_name}{separator}{element_num} (for 1D properties),
+    {property_name}{separator}{elem_num0}{separator}{elem_num1} (for 2D
+    properties), and so on.
+
+    For multidimensional properties that don't have a fixed size, such as
+    "image" (the image of a region varies in size depending on the region
+    size), an object array will be used, with the corresponding property name
+    as the key.
+
+    Examples
+    --------
+    >>> from skimage import data, util, measure
+    >>> image = data.coins()
+    >>> label_image = measure.label(image > 110, connectivity=image.ndim)
+    >>> proplist = regionprops(label_image, image)
+    >>> props = _props_to_dict(proplist, properties=['label', 'inertia_tensor',
+    ...                                              'inertia_tensor_eigvals'])
+    >>> props  # doctest: +ELLIPSIS +SKIP
+    {'label': array([ 1,  2, ...]), ...
+     'inertia_tensor-0-0': array([  4.012...e+03,   8.51..., ...]), ...
+     ...,
+     'inertia_tensor_eigvals-1': array([  2.67...e+02,   2.83..., ...])}
+
+    The resulting dictionary can be directly passed to pandas, if installed, to
+    obtain a clean DataFrame:
+
+    >>> import pandas as pd  # doctest: +SKIP
+    >>> data = pd.DataFrame(props)  # doctest: +SKIP
+    >>> data.head()  # doctest: +SKIP
+       label  inertia_tensor-0-0  ...  inertia_tensor_eigvals-1
+    0      1         4012.909888  ...                267.065503
+    1      2            8.514739  ...                  2.834806
+    2      3            0.666667  ...                  0.000000
+    3      4            0.000000  ...                  0.000000
+    4      5            0.222222  ...                  0.111111
+
+    """
+
+    out = {}
+    n = len(regions)
+    for prop in properties:
+        dtype = COL_DTYPES[prop]
+        column_buffer = np.zeros(n, dtype=dtype)
+        r = regions[0][prop]
+
+        # scalars and objects are dedicated one column per prop
+        # array properties are raveled into multiple columns
+        # for more info, refer to notes 1
+        if np.isscalar(r) or prop in OBJECT_COLUMNS:
+            for i in range(n):
+                column_buffer[i] = regions[i][prop]
+            out[prop] = np.copy(column_buffer)
+        else:
+            if isinstance(r, np.ndarray):
+                shape = r.shape
+            else:
+                shape = (len(r),)
+
+            for ind in np.ndindex(shape):
+                for k in range(n):
+                    loc = ind if len(ind) > 1 else ind[0]
+                    column_buffer[k] = regions[k][prop][loc]
+                modified_prop = separator.join(map(str, (prop,) + ind))
+                out[modified_prop] = np.copy(column_buffer)
+    return out
+
+
+def regionprops_table(label_image, intensity_image=None,
+                      properties=('label', 'bbox'),
+                      *,
+                      cache=True, separator='-'):
+    """Compute image properties and return them as a pandas-compatible table.
+
+    The table is a dictionary mapping column names to value arrays. See Notes
+    section below for details.
+
+    Parameters
+    ----------
+    label_image : (N, M) ndarray
+        Labeled input image. Labels with value 0 are ignored.
+    intensity_image : (N, M) ndarray, optional
+        Intensity (i.e., input) image with same size as labeled image.
+        Default is None.
+    properties : tuple or list of str, optional
+        Properties that will be included in the resulting dictionary
+        For a list of available properties, please see :func:`regionprops`.
+        Users should remember to add "label" to keep track of region
+        identities.
+    cache : bool, optional
+        Determine whether to cache calculated properties. The computation is
+        much faster for cached properties, whereas the memory consumption
+        increases.
+    separator : str, optional
+        For non-scalar properties not listed in OBJECT_COLUMNS, each element
+        will appear in its own column, with the index of that element separated
+        from the property name by this separator. For example, the inertia
+        tensor of a 2D region will appear in four columns:
+        ``inertia_tensor-0-0``, ``inertia_tensor-0-1``, ``inertia_tensor-1-0``,
+        and ``inertia_tensor-1-1`` (where the separator is ``-``).
+
+        Object columns are those that cannot be split in this way because the
+        number of columns would change depending on the object. For example,
+        ``image`` and ``coords``.
+
+    Returns
+    -------
+    out_dict : dict
+        Dictionary mapping property names to an array of values of that
+        property, one value per region. This dictionary can be used as input to
+        pandas ``DataFrame`` to map property names to columns in the frame and
+        regions to rows.
+
+    Notes
+    -----
+    Each column contains either a scalar property, an object property, or an
+    element in a multidimensional array.
+
+    Properties with scalar values for each region, such as "eccentricity", will
+    appear as a float or int array with that property name as key.
+
+    Multidimensional properties *of fixed size* for a given image dimension,
+    such as "centroid" (every centroid will have three elements in a 3D image,
+    no matter the region size), will be split into that many columns, with the
+    name {property_name}{separator}{element_num} (for 1D properties),
+    {property_name}{separator}{elem_num0}{separator}{elem_num1} (for 2D
+    properties), and so on.
+
+    For multidimensional properties that don't have a fixed size, such as
+    "image" (the image of a region varies in size depending on the region
+    size), an object array will be used, with the corresponding property name
+    as the key.
+
+    Examples
+    --------
+    >>> from skimage import data, util, measure
+    >>> image = data.coins()
+    >>> label_image = measure.label(image > 110, connectivity=image.ndim)
+    >>> props = regionprops_table(label_image, image,
+    ...                           properties=['label', 'inertia_tensor',
+    ...                                       'inertia_tensor_eigvals'])
+    >>> props  # doctest: +ELLIPSIS +SKIP
+    {'label': array([ 1,  2, ...]), ...
+     'inertia_tensor-0-0': array([  4.012...e+03,   8.51..., ...]), ...
+     ...,
+     'inertia_tensor_eigvals-1': array([  2.67...e+02,   2.83..., ...])}
+
+    The resulting dictionary can be directly passed to pandas, if installed, to
+    obtain a clean DataFrame:
+
+    >>> import pandas as pd  # doctest: +SKIP
+    >>> data = pd.DataFrame(props)  # doctest: +SKIP
+    >>> data.head()  # doctest: +SKIP
+       label  inertia_tensor-0-0  ...  inertia_tensor_eigvals-1
+    0      1         4012.909888  ...                267.065503
+    1      2            8.514739  ...                  2.834806
+    2      3            0.666667  ...                  0.000000
+    3      4            0.000000  ...                  0.000000
+    4      5            0.222222  ...                  0.111111
+
+    [5 rows x 7 columns]
+
+    """
+    regions = regionprops(label_image, intensity_image=intensity_image,
+                          cache=cache)
+    return _props_to_dict(regions, properties=properties, separator=separator)
+
+
 def regionprops(label_image, intensity_image=None, cache=True,
                 coordinates=None):
-    """Measure properties of labeled image regions.
+    r"""Measure properties of labeled image regions.
 
     Parameters
     ----------
     label_image : (N, M) ndarray
         Labeled input image. Labels with value 0 are ignored.
 
-        .. versionchanged:: 0.14.2
+        .. versionchanged:: 0.14.1
             Previously, ``label_image`` was processed by ``numpy.squeeze`` and
             so any number of singleton dimensions was allowed. This resulted in
             inconsistent handling of images with singleton dimensions. To
@@ -380,9 +647,20 @@ def regionprops(label_image, intensity_image=None, cache=True,
         Determine whether to cache calculated properties. The computation is
         much faster for cached properties, whereas the memory consumption
         increases.
-    coordinates : 'rc' or 'xy', optional
-        Coordinate conventions for 2D images. (Only 'rc' coordinates are
-        supported for 3D images.)
+    coordinates : DEPRECATED
+        This argument is deprecated and will be removed in a future version
+        of scikit-image.
+
+        See :ref:`Coordinate conventions <numpy-images-coordinate-conventions>`
+        for more details.
+
+        .. deprecated:: 0.16.0
+            Use "rc" coordinates everywhere. It may be sufficient to call
+            ``numpy.transpose`` on your label image to get the same values as
+            0.15 and earlier. However, for some properties, the transformation
+            will be less trivial. For example, the new orientation is
+            :math:`\frac{\pi}{2}` plus the old orientation.
+
 
     Returns
     -------
@@ -395,7 +673,7 @@ def regionprops(label_image, intensity_image=None, cache=True,
     The following properties can be accessed as attributes or keys:
 
     **area** : int
-        Number of pixels of region.
+        Number of pixels of the region.
     **bbox** : tuple
         Bounding box ``(min_row, min_col, max_row, max_col)``.
         Pixels belonging to the bounding box are in the half-open interval
@@ -405,7 +683,8 @@ def regionprops(label_image, intensity_image=None, cache=True,
     **centroid** : array
         Centroid coordinate tuple ``(row, col)``.
     **convex_area** : int
-        Number of pixels of convex hull image.
+        Number of pixels of convex hull image, which is the smallest convex
+        polygon that encloses the region.
     **convex_image** : (H, J) ndarray
         Binary convex hull image which has the same size as bounding box.
     **coords** : (N, 2) ndarray
@@ -425,16 +704,17 @@ def regionprops(label_image, intensity_image=None, cache=True,
         Ratio of pixels in the region to pixels in the total bounding box.
         Computed as ``area / (rows * cols)``
     **filled_area** : int
-        Number of pixels of filled region.
+        Number of pixels of the region will all the holes filled in. Describes
+        the area of the filled_image.
     **filled_image** : (H, J) ndarray
         Binary region image with filled holes which has the same size as
         bounding box.
     **image** : (H, J) ndarray
         Sliced binary region image which has the same size as bounding box.
-    **inertia_tensor** : (2, 2) ndarray
+    **inertia_tensor** : ndarray
         Inertia tensor of the region for the rotation around its mass.
     **inertia_tensor_eigvals** : tuple
-        The two eigen values of the inertia tensor in decreasing order.
+        The eigenvalues of the inertia tensor in decreasing order.
     **intensity_image** : ndarray
         Image inside region bounding box.
     **label** : int
@@ -457,34 +737,33 @@ def regionprops(label_image, intensity_image=None, cache=True,
     **moments** : (3, 3) ndarray
         Spatial moments up to 3rd order::
 
-            m_ji = sum{ array(x, y) * x^j * y^i }
+            m_ij = sum{ array(row, col) * row^i * col^j }
 
-        where the sum is over the `x`, `y` coordinates of the region.
+        where the sum is over the `row`, `col` coordinates of the region.
     **moments_central** : (3, 3) ndarray
         Central moments (translation invariant) up to 3rd order::
 
-            mu_ji = sum{ array(x, y) * (x - x_c)^j * (y - y_c)^i }
+            mu_ij = sum{ array(row, col) * (row - row_c)^i * (col - col_c)^j }
 
-        where the sum is over the `x`, `y` coordinates of the region,
-        and `x_c` and `y_c` are the coordinates of the region's centroid.
+        where the sum is over the `row`, `col` coordinates of the region,
+        and `row_c` and `col_c` are the coordinates of the region's centroid.
     **moments_hu** : tuple
         Hu moments (translation, scale and rotation invariant).
     **moments_normalized** : (3, 3) ndarray
         Normalized moments (translation and scale invariant) up to 3rd order::
 
-            nu_ji = mu_ji / m_00^[(i+j)/2 + 1]
+            nu_ij = mu_ij / m_00^[(i+j)/2 + 1]
 
         where `m_00` is the zeroth spatial moment.
     **orientation** : float
-        In 'rc' coordinates, angle between the 0th axis (rows) and the major
+        Angle between the 0th axis (rows) and the major
         axis of the ellipse that has the same second moments as the region,
         ranging from `-pi/2` to `pi/2` counter-clockwise.
-
-        In `xy` coordinates, as above but the angle is now measured from the
-        "x" or horizontal axis.
     **perimeter** : float
         Perimeter of object which approximates the contour as a line
         through the centers of border pixels using a 4-connectivity.
+    **slice** : tuple of slices
+        A slice to extract the object from the source image.
     **solidity** : float
         Ratio of pixels in the region to pixels of the convex hull image.
     **weighted_centroid** : array
@@ -496,17 +775,17 @@ def regionprops(label_image, intensity_image=None, cache=True,
     **weighted_moments** : (3, 3) ndarray
         Spatial moments of intensity image up to 3rd order::
 
-            wm_ji = sum{ array(x, y) * x^j * y^i }
+            wm_ij = sum{ array(row, col) * row^i * col^j }
 
-        where the sum is over the `x`, `y` coordinates of the region.
+        where the sum is over the `row`, `col` coordinates of the region.
     **weighted_moments_central** : (3, 3) ndarray
         Central moments (translation invariant) of intensity image up to
         3rd order::
 
-            wmu_ji = sum{ array(x, y) * (x - x_c)^j * (y - y_c)^i }
+            wmu_ij = sum{ array(row, col) * (row - row_c)^i * (col - col_c)^j }
 
-        where the sum is over the `x`, `y` coordinates of the region,
-        and `x_c` and `y_c` are the coordinates of the region's weighted
+        where the sum is over the `row`, `col` coordinates of the region,
+        and `row_c` and `col_c` are the coordinates of the region's weighted
         centroid.
     **weighted_moments_hu** : tuple
         Hu moments (translation, scale and rotation invariant) of intensity
@@ -515,7 +794,7 @@ def regionprops(label_image, intensity_image=None, cache=True,
         Normalized moments (translation and scale invariant) of intensity
         image up to 3rd order::
 
-            wnu_ji = wmu_ji / wm_00^[(i+j)/2 + 1]
+            wnu_ij = wmu_ij / wm_00^[(i+j)/2 + 1]
 
         where ``wm_00`` is the zeroth spatial moment (intensity-weighted area).
 
@@ -537,7 +816,7 @@ def regionprops(label_image, intensity_image=None, cache=True,
     .. [3] T. H. Reiss. Recognizing Planar Objects Using Invariant Image
            Features, from Lecture notes in computer science, p. 676. Springer,
            Berlin, 1993.
-    .. [4] http://en.wikipedia.org/wiki/Image_moment
+    .. [4] https://en.wikipedia.org/wiki/Image_moment
 
     Examples
     --------
@@ -561,6 +840,22 @@ def regionprops(label_image, intensity_image=None, cache=True,
     if not np.issubdtype(label_image.dtype, np.integer):
         raise TypeError('Label image must be of integer type.')
 
+    if coordinates is not None:
+        if coordinates == 'rc':
+            msg = ('The coordinates keyword argument to skimage.measure.'
+                   'regionprops is deprecated. All features are now computed '
+                   'in rc (row-column) coordinates. Please remove '
+                   '`coordinates="rc"` from all calls to regionprops before '
+                   'updating scikit-image.')
+            warn(msg, stacklevel=2, category=FutureWarning)
+        else:
+            msg = ('Values other than "rc" for the "coordinates" argument '
+                   'to skimage.measure.regionprops are no longer supported. '
+                   'You should update your code to use "rc" coordinates and '
+                   'stop using the "coordinates" argument, or use skimage '
+                   'version 0.15.x or earlier.')
+            raise ValueError(msg)
+
     regions = []
 
     objects = ndi.find_objects(label_image)
@@ -570,8 +865,8 @@ def regionprops(label_image, intensity_image=None, cache=True,
 
         label = i + 1
 
-        props = _RegionProperties(sl, label, label_image, intensity_image,
-                                  cache, coordinates=coordinates)
+        props = RegionProperties(sl, label, label_image, intensity_image,
+                                 cache)
         regions.append(props)
 
     return regions
@@ -582,10 +877,12 @@ def perimeter(image, neighbourhood=4):
 
     Parameters
     ----------
-    image : array
-        Binary image.
+    image : (N, M) ndarray
+        2D binary image.
     neighbourhood : 4 or 8, optional
-        Neighborhood connectivity for border pixel determination.
+        Neighborhood connectivity for border pixel determination. It is used to
+        compute the contour. A higher neighbourhood widens the border on which
+        the perimeter is computed.
 
     Returns
     -------
@@ -597,7 +894,23 @@ def perimeter(image, neighbourhood=4):
     .. [1] K. Benkrid, D. Crookes. Design and FPGA Implementation of
            a Perimeter Estimator. The Queen's University of Belfast.
            http://www.cs.qub.ac.uk/~d.crookes/webpubs/papers/perimeter.doc
+
+    Examples
+    --------
+    >>> from skimage import data, util
+    >>> from skimage.measure import label
+    >>> # coins image (binary)
+    >>> img_coins = data.coins() > 110
+    >>> # total perimeter of all objects in the image
+    >>> perimeter(img_coins, neighbourhood=4)  # doctest: +ELLIPSIS
+    7796.867...
+    >>> perimeter(img_coins, neighbourhood=8)  # doctest: +ELLIPSIS
+    8806.268...
+
     """
+    if image.ndim != 2:
+        raise NotImplementedError('`perimeter` supports 2D images only')
+
     if neighbourhood == 4:
         strel = STREL_4
     else:
@@ -621,7 +934,7 @@ def perimeter(image, neighbourhood=4):
     # but that was measured as taking much longer than bincount + np.dot (5x
     # as much time)
     perimeter_histogram = np.bincount(perimeter_image.ravel(), minlength=50)
-    total_perimeter = np.dot(perimeter_histogram, perimeter_weights)
+    total_perimeter = perimeter_histogram @ perimeter_weights
     return total_perimeter
 
 
@@ -629,10 +942,10 @@ def _parse_docs():
     import re
     import textwrap
 
-    doc = regionprops.__doc__
-    matches = re.finditer('\*\*(\w+)\*\* \:.*?\n(.*?)(?=\n    [\*\S]+)',
+    doc = regionprops.__doc__ or ''
+    matches = re.finditer(r'\*\*(\w+)\*\* \:.*?\n(.*?)(?=\n    [\*\S]+)',
                           doc, flags=re.DOTALL)
-    prop_doc = dict((m.group(1), textwrap.dedent(m.group(2))) for m in matches)
+    prop_doc = {m.group(1): textwrap.dedent(m.group(2)) for m in matches}
 
     return prop_doc
 
@@ -640,15 +953,11 @@ def _parse_docs():
 def _install_properties_docs():
     prop_doc = _parse_docs()
 
-    for p in [member for member in dir(_RegionProperties)
+    for p in [member for member in dir(RegionProperties)
               if not member.startswith('_')]:
-        try:
-            getattr(_RegionProperties, p).__doc__ = prop_doc[p]
-        except AttributeError:
-            # For Python 2.x
-            getattr(_RegionProperties, p).im_func.__doc__ = prop_doc[p]
-
-        setattr(_RegionProperties, p, property(getattr(_RegionProperties, p)))
+        getattr(RegionProperties, p).__doc__ = prop_doc[p]
 
 
-_install_properties_docs()
+if __debug__:
+    # don't install docstrings when in optimized/non-debug mode
+    _install_properties_docs()
