@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 """ Create lookup tables for the marching cubes algorithm, by parsing
-the file "LookUpTable.h". This prints a text to the stdout wich
+the file "LookUpTable.h". This prints a text to the stdout which
 can then be copied to luts.py.
 
 The luts are tuples of shape and base64 encoded bytes.
@@ -13,14 +13,6 @@ import numpy as np
 import sys
 import base64
 
-# Get base64 encode/decode functions
-if sys.version_info >= (3, ):
-    base64encode = base64.encodebytes
-    base64decode = base64.decodebytes
-else:
-    base64encode = base64.encodestring
-    base64decode = base64.decodestring
-
 
 def create_luts(fname):
 
@@ -113,8 +105,8 @@ def get_table(lines1, needle, i):
     code = '\n'.join(lines2)
     code = code.split('=',1)[1]
     array = eval(code)
-    array64 = base64encode(array.tostring()).decode('utf-8')
-    # Reverse: bytes = base64decode(text.encode('utf-8'))
+    array64 = base64.encodebytes(array.tostring()).decode('utf-8')
+    # Reverse: bytes = base64.decodebytes(text.encode('utf-8'))
     text = '%s = %s, """\n%s"""' % (name, str(array.shape), array64)
 
     # Build actual lines
