@@ -2,22 +2,96 @@
 Algorithms for computing the skeleton of a binary image
 """
 
-import sys
-from six.moves import range
+
 import numpy as np
+from ..util import img_as_ubyte, crop
 from scipy import ndimage as ndi
 
+from .._shared.utils import check_nD, warn
 from ._skeletonize_cy import (_fast_skeletonize, _skeletonize_loop,
                               _table_lookup_index)
-
-from .._shared.utils import assert_nD
+from ._skeletonize_3d_cy import _compute_thin_image
 
 
-# --------- Skeletonization by morphological thinning ---------
+def skeletonize(image, *, method=None):
+    """Compute the skeleton of a binary image.
+
+    Thinning is used to reduce each connected component in a binary image
+    to a single-pixel wide skeleton.
+
+    Parameters
+    ----------
+    image : ndarray, 2D or 3D
+        A binary image containing the objects to be skeletonized. Zeros
+        represent background, nonzero values are foreground.
+    method : {'zhang', 'lee'}, optional
+        Which algorithm to use. Zhang's algorithm [Zha84]_ only works for
+        2D images, and is the default for 2D. Lee's algorithm [Lee94]_
+        works for 2D or 3D images and is the default for 3D.
+
+    Returns
+    -------
+    skeleton : ndarray
+        The thinned image.
+
+    See also
+    --------
+    medial_axis
+
+    References
+    ----------
+    .. [Lee94] T.-C. Lee, R.L. Kashyap and C.-N. Chu, Building skeleton models
+           via 3-D medial surface/axis thinning algorithms.
+           Computer Vision, Graphics, and Image Processing, 56(6):462-478, 1994.
+
+    .. [Zha84] A fast parallel algorithm for thinning digital patterns,
+           T. Y. Zhang and C. Y. Suen, Communications of the ACM,
+           March 1984, Volume 27, Number 3.
 
 
-def skeletonize(image):
-    """Return the skeleton of a binary image.
+    Examples
+    --------
+    >>> X, Y = np.ogrid[0:9, 0:9]
+    >>> ellipse = (1./3 * (X - 4)**2 + (Y - 4)**2 < 3**2).astype(np.uint8)
+    >>> ellipse
+    array([[0, 0, 0, 1, 1, 1, 0, 0, 0],
+           [0, 0, 1, 1, 1, 1, 1, 0, 0],
+           [0, 0, 1, 1, 1, 1, 1, 0, 0],
+           [0, 0, 1, 1, 1, 1, 1, 0, 0],
+           [0, 0, 1, 1, 1, 1, 1, 0, 0],
+           [0, 0, 1, 1, 1, 1, 1, 0, 0],
+           [0, 0, 1, 1, 1, 1, 1, 0, 0],
+           [0, 0, 1, 1, 1, 1, 1, 0, 0],
+           [0, 0, 0, 1, 1, 1, 0, 0, 0]], dtype=uint8)
+    >>> skel = skeletonize(ellipse)
+    >>> skel.astype(np.uint8)
+    array([[0, 0, 0, 0, 0, 0, 0, 0, 0],
+           [0, 0, 0, 0, 0, 0, 0, 0, 0],
+           [0, 0, 0, 0, 0, 0, 0, 0, 0],
+           [0, 0, 0, 0, 1, 0, 0, 0, 0],
+           [0, 0, 0, 0, 1, 0, 0, 0, 0],
+           [0, 0, 0, 0, 1, 0, 0, 0, 0],
+           [0, 0, 0, 0, 1, 0, 0, 0, 0],
+           [0, 0, 0, 0, 0, 0, 0, 0, 0],
+           [0, 0, 0, 0, 0, 0, 0, 0, 0]], dtype=uint8)
+
+    """
+
+    if image.ndim == 2 and (method is None or method == 'zhang'):
+        skeleton = skeletonize_2d(image)
+    elif image.ndim == 3 and method == 'zhang':
+        raise ValueError('skeletonize method "zhang" only works for 2D '
+                         'images.')
+    elif image.ndim == 3 or (image.ndim == 2 and method == 'lee'):
+        skeleton = skeletonize_3d(image)
+    else:
+        raise ValueError('skeletonize requires a 2D or 3D image as input, '
+                         'got {}D.'.format(image.ndim))
+    return skeleton
+
+
+def skeletonize_2d(image):
+    """Return the skeleton of a 2D binary image.
 
     Thinning is used to reduce each connected component in a binary image
     to a single-pixel wide skeleton.
@@ -204,7 +278,7 @@ def thin(image, max_iter=None):
 
     See also
     --------
-    skeletonize, skeletonize_3d, medial_axis
+    skeletonize, medial_axis
 
     Notes
     -----
@@ -220,11 +294,11 @@ def thin(image, max_iter=None):
     ----------
     .. [1] Z. Guo and R. W. Hall, "Parallel thinning with
            two-subiteration algorithms," Comm. ACM, vol. 32, no. 3,
-           pp. 359-373, 1989. DOI:10.1145/62065.62074
+           pp. 359-373, 1989. :DOI:`10.1145/62065.62074`
     .. [2] Lam, L., Seong-Whan Lee, and Ching Y. Suen, "Thinning
            Methodologies-A Comprehensive Survey," IEEE Transactions on
            Pattern Analysis and Machine Intelligence, Vol 14, No. 9,
-           p. 879, 1992. DOI:10.1109/34.161346
+           p. 879, 1992. :DOI:`10.1109/34.161346`
 
     Examples
     --------
@@ -249,22 +323,24 @@ def thin(image, max_iter=None):
            [0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0]], dtype=uint8)
     """
-    # check parameters
-    max_iter = max_iter or sys.maxsize
     # check that image is 2d
-    assert_nD(image, 2)
+    check_nD(image, 2)
 
     # convert image to uint8 with values in {0, 1}
     skel = np.asanyarray(image, dtype=bool).astype(np.uint8)
 
     # neighborhood mask
-    mask = np.array([[ 8,  4,  2],
-                     [16,  0,  1],
-                     [32, 64,128]], dtype=np.uint8)
+    mask = np.array([[ 8,  4,   2],
+                     [16,  0,   1],
+                     [32, 64, 128]], dtype=np.uint8)
 
     # iterate until convergence, up to the iteration limit
-    for i in range(max_iter):
-        before = np.sum(skel)  # count points before thinning
+    max_iter = max_iter or np.inf
+    n_iter = 0
+    n_pts_old, n_pts_new = np.inf, np.sum(skel)
+    while n_pts_old != n_pts_new and n_iter < max_iter:
+        n_pts_old = n_pts_new
+
         # perform the two "subiterations" described in the paper
         for lut in [G123_LUT, G123P_LUT]:
             # correlate image with neighborhood mask
@@ -274,11 +350,8 @@ def thin(image, max_iter=None):
             # perform deletion
             skel[D] = 0
 
-        after = np.sum(skel)  # count points after thinning
-
-        if before == after:
-            # iteration had no effect: finish
-            break
+        n_pts_new = np.sum(skel)  # count points after thinning
+        n_iter += 1
 
     return skel.astype(np.bool)
 
@@ -429,7 +502,7 @@ def medial_axis(image, mask=None, return_distance=False):
     _skeletonize_loop(result, i, j, order, table)
 
     result = result.astype(bool)
-    if not mask is None:
+    if mask is not None:
         result[~mask] = image[~mask]
     if return_distance:
         return result, store_distance
@@ -501,3 +574,86 @@ def _table_lookup(image, table):
         indexer = _table_lookup_index(np.ascontiguousarray(image, np.uint8))
     image = table[indexer]
     return image
+
+
+def skeletonize_3d(image, *, img=None):
+    """Compute the skeleton of a binary image.
+
+    Thinning is used to reduce each connected component in a binary image
+    to a single-pixel wide skeleton.
+
+    Parameters
+    ----------
+    image : ndarray, 2D or 3D
+        A binary image containing the objects to be skeletonized. Zeros
+        represent background, nonzero values are foreground.
+
+    Other Parameters
+    ----------------
+    img : DEPRECATED
+        Synonym for `image`.
+
+        .. deprecated:: 0.16
+           Will be removed in 0.17.
+
+    Returns
+    -------
+    skeleton : ndarray
+        The thinned image.
+
+    See also
+    --------
+    skeletonize, medial_axis
+
+    Notes
+    -----
+    The method of [Lee94]_ uses an octree data structure to examine a 3x3x3
+    neighborhood of a pixel. The algorithm proceeds by iteratively sweeping
+    over the image, and removing pixels at each iteration until the image
+    stops changing. Each iteration consists of two steps: first, a list of
+    candidates for removal is assembled; then pixels from this list are
+    rechecked sequentially, to better preserve connectivity of the image.
+
+    The algorithm this function implements is different from the algorithms
+    used by either `skeletonize` or `medial_axis`, thus for 2D images the
+    results produced by this function are generally different.
+
+    References
+    ----------
+    .. [Lee94] T.-C. Lee, R.L. Kashyap and C.-N. Chu, Building skeleton models
+           via 3-D medial surface/axis thinning algorithms.
+           Computer Vision, Graphics, and Image Processing, 56(6):462-478, 1994.
+
+    """
+    if img is not None:
+        image = img
+        warn('Using img as a keyword argument to skeletonize_3d is deprecated.'
+             ' Use image instead.')
+    # make sure the image is 3D or 2D
+    if image.ndim < 2 or image.ndim > 3:
+        raise ValueError("skeletonize_3d can only handle 2D or 3D images; "
+                         "got image.ndim = %s instead." % image.ndim)
+    image = np.ascontiguousarray(image)
+    image = img_as_ubyte(image, force_copy=False)
+
+    # make an in image 3D and pad it w/ zeros to simplify dealing w/ boundaries
+    # NB: careful here to not clobber the original *and* minimize copying
+    image_o = image
+    if image.ndim == 2:
+        image_o = image[np.newaxis, ...]
+    image_o = np.pad(image_o, pad_width=1, mode='constant')
+
+    # normalize to binary
+    maxval = image_o.max()
+    image_o[image_o != 0] = 1
+
+    # do the computation
+    image_o = np.asarray(_compute_thin_image(image_o))
+
+    # crop it back and restore the original intensity range
+    image_o = crop(image_o, crop_width=1)
+    if image.ndim == 2:
+        image_o = image_o[0]
+    image_o *= maxval
+
+    return image_o
