@@ -8,12 +8,12 @@ from .selem import _default_selem
 # Our function names don't exactly correspond to ndimages.
 # This dictionary translates from our names to scipy's.
 funcs = ('erosion', 'dilation', 'opening', 'closing')
-skimage2ndimage = dict((x, 'grey_' + x) for x in funcs)
+skimage2ndimage = {x: 'grey_' + x for x in funcs}
 
 # These function names are the same in ndimage.
 funcs = ('binary_erosion', 'binary_dilation', 'binary_opening',
          'binary_closing', 'black_tophat', 'white_tophat')
-skimage2ndimage.update(dict((x, x) for x in funcs))
+skimage2ndimage.update({x: x for x in funcs})
 
 
 def default_selem(func):
@@ -49,20 +49,25 @@ def _check_dtype_supported(ar):
 
 
 def remove_small_objects(ar, min_size=64, connectivity=1, in_place=False):
-    """Remove connected components smaller than the specified size.
+    """Remove objects smaller than the specified size.
+
+    Expects ar to be an array with labeled objects, and removes objects
+    smaller than min_size. If `ar` is bool, the image is first labeled.
+    This leads to potentially different behavior for bool and 0-and-1
+    arrays.
 
     Parameters
     ----------
     ar : ndarray (arbitrary shape, int or bool type)
-        The array containing the connected components of interest. If the array
-        type is int, it is assumed that it contains already-labeled objects.
-        The ints must be non-negative.
+        The array containing the objects of interest. If the array type is
+        int, the ints must be non-negative.
     min_size : int, optional (default: 64)
-        The smallest allowable connected component size.
+        The smallest allowable object size.
     connectivity : int, {1, 2, ..., ar.ndim}, optional (default: 1)
-        The connectivity defining the neighborhood of a pixel.
+        The connectivity defining the neighborhood of a pixel. Used during
+        labelling if `ar` is bool.
     in_place : bool, optional (default: False)
-        If `True`, remove the connected components in the input array itself.
+        If ``True``, remove the objects in the input array itself.
         Otherwise, make a copy.
 
     Raises
@@ -134,9 +139,8 @@ def remove_small_objects(ar, min_size=64, connectivity=1, in_place=False):
     return out
 
 
-def remove_small_holes(ar, area_threshold=64, connectivity=1, in_place=False,
-                       min_size=None):
-    """Remove continguous holes smaller than the specified size.
+def remove_small_holes(ar, area_threshold=64, connectivity=1, in_place=False):
+    """Remove contiguous holes smaller than the specified size.
 
     Parameters
     ----------
@@ -151,7 +155,6 @@ def remove_small_holes(ar, area_threshold=64, connectivity=1, in_place=False,
         If `True`, remove the connected components in the input array itself.
         Otherwise, make a copy.
 
-
     Raises
     ------
     TypeError
@@ -202,11 +205,6 @@ def remove_small_holes(ar, area_threshold=64, connectivity=1, in_place=False,
         warn("Any labeled images will be returned as a boolean array. "
              "Did you mean to use a boolean array?", UserWarning)
 
-    if min_size is not None:
-        warn("the min_size argument is deprecated and will be removed in " +
-             "0.16. Use area_threshold instead.")
-        area_threshold = min_size
-
     if in_place:
         out = ar
     else:
