@@ -12,12 +12,16 @@ def configuration(parent_package='', top_path=None):
     config = Configuration('morphology', parent_package, top_path)
     config.add_data_dir('tests')
 
-    cython(['_watershed.pyx'], working_path=base_path)
-    cython(['_skeletonize_cy.pyx'], working_path=base_path)
-    cython(['_convex_hull.pyx'], working_path=base_path)
-    cython(['_greyreconstruct.pyx'], working_path=base_path)
+    cython(['_watershed.pyx',
+            '_skeletonize_cy.pyx',
+            '_convex_hull.pyx',
+            '_greyreconstruct.pyx',
+            '_extrema_cy.pyx'], working_path=base_path)
+    # _skeletonize_3d uses c++, so it must be cythonized separately
     cython(['_skeletonize_3d_cy.pyx.in'], working_path=base_path)
     cython(['_extrema_cy.pyx'], working_path=base_path)
+    cython(['_flood_fill_cy.pyx'], working_path=base_path)
+    cython(['_max_tree.pyx'], working_path=base_path)
 
     config.add_extension('_watershed', sources=['_watershed.c'],
                          include_dirs=[get_numpy_include_dirs()])
@@ -27,10 +31,16 @@ def configuration(parent_package='', top_path=None):
                          include_dirs=[get_numpy_include_dirs()])
     config.add_extension('_greyreconstruct', sources=['_greyreconstruct.c'],
                          include_dirs=[get_numpy_include_dirs()])
-    config.add_extension('_skeletonize_3d_cy', sources=['_skeletonize_3d_cy.c'],
+    config.add_extension('_max_tree', sources=['_max_tree.c'],
                          include_dirs=[get_numpy_include_dirs()])
+    config.add_extension('_skeletonize_3d_cy',
+                         sources=['_skeletonize_3d_cy.cpp'],
+                         include_dirs=[get_numpy_include_dirs()],
+                         language='c++')
     config.add_extension('_extrema_cy', sources=['_extrema_cy.c'],
                          include_dirs=[get_numpy_include_dirs()])
+    config.add_extension('_flood_fill_cy', sources=['_flood_fill_cy.c'],
+                         include_dirs=[get_numpy_include_dirs()])
 
     return config
 
