@@ -1,9 +1,5 @@
-# -*- coding: utf-8 -*-
-# deconvolution.py --- Image deconvolution
-
 """Implementations restoration functions"""
 
-from __future__ import division
 
 import numpy as np
 import numpy.random as npr
@@ -15,7 +11,7 @@ __keywords__ = "restoration, image, deconvolution"
 
 
 def wiener(image, psf, balance, reg=None, is_real=True, clip=True):
-    """Wiener-Hunt deconvolution
+    r"""Wiener-Hunt deconvolution
 
     Return the deconvolution with a Wiener-Hunt approach (i.e. with
     Fourier diagonalisation).
@@ -82,8 +78,8 @@ def wiener(image, psf, balance, reg=None, is_real=True, clip=True):
        \Lambda_H^\dagger F y
 
     where :math:`F` and :math:`F^\dagger` are the Fourier and inverse
-    Fourier transfroms respectively, :math:`\Lambda_H` the transfer
-    function (or the Fourier transfrom of the PSF, see [Hunt] below)
+    Fourier transforms respectively, :math:`\Lambda_H` the transfer
+    function (or the Fourier transform of the PSF, see [Hunt] below)
     and :math:`\Lambda_D` the filter to penalize the restored image
     frequencies (Laplacian by default, that is penalization of high
     frequency). The parameter :math:`\lambda` tunes the balance
@@ -108,7 +104,7 @@ def wiener(image, psf, balance, reg=None, is_real=True, clip=True):
            spread function parameters for Wiener-Hunt deconvolution",
            J. Opt. Soc. Am. A 27, 1593-1607 (2010)
 
-           http://www.opticsinfobase.org/josaa/abstract.cfm?URI=josaa-27-7-1593
+           https://www.osapublishing.org/josaa/abstract.cfm?URI=josaa-27-7-1593
 
            http://research.orieux.fr/files/papers/OGR-JOSA10.pdf
 
@@ -162,7 +158,7 @@ def unsupervised_wiener(image, psf, reg=None, user_params=None, is_real=True,
     reg : ndarray, optional
        The regularisation operator. The Laplacian by default. It can
        be an impulse response or a transfer function, as for the psf.
-    user_params : dict
+    user_params : dict, optional
        Dictionary of parameters for the Gibbs sampler. See below.
     clip : boolean, optional
        True by default. If true, pixel values of the result above 1 or
@@ -230,7 +226,7 @@ def unsupervised_wiener(image, psf, reg=None, user_params=None, is_real=True,
            spread function parameters for Wiener-Hunt deconvolution",
            J. Opt. Soc. Am. A 27, 1593-1607 (2010)
 
-           http://www.opticsinfobase.org/josaa/abstract.cfm?URI=josaa-27-7-1593
+           https://www.osapublishing.org/josaa/abstract.cfm?URI=josaa-27-7-1593
 
            http://research.orieux.fr/files/papers/OGR-JOSA10.pdf
     """
@@ -264,7 +260,7 @@ def unsupervised_wiener(image, psf, reg=None, user_params=None, is_real=True,
     areg2 = np.abs(reg) ** 2
     atf2 = np.abs(trans_fct) ** 2
 
-    # The Fourier transfrom may change the image.size attribut, so we
+    # The Fourier transform may change the image.size attribute, so we
     # store it.
     if is_real:
         data_spectrum = uft.urfft2(image.astype(np.float))
@@ -339,7 +335,7 @@ def richardson_lucy(image, psf, iterations=50, clip=True):
        Input degraded image (can be N dimensional).
     psf : ndarray
        The point spread function.
-    iterations : int
+    iterations : int, optional
        Number of iterations. This parameter plays the role of
        regularisation.
     clip : boolean, optional
@@ -363,7 +359,7 @@ def richardson_lucy(image, psf, iterations=50, clip=True):
 
     References
     ----------
-    .. [1] http://en.wikipedia.org/wiki/Richardson%E2%80%93Lucy_deconvolution
+    .. [1] https://en.wikipedia.org/wiki/Richardson%E2%80%93Lucy_deconvolution
     """
     # compute the times for direct convolution and the fft method. The fft is of
     # complexity O(N log(N)) for each dimension and the direct method does
@@ -382,7 +378,7 @@ def richardson_lucy(image, psf, iterations=50, clip=True):
 
     image = image.astype(np.float)
     psf = psf.astype(np.float)
-    im_deconv = 0.5 * np.ones(image.shape)
+    im_deconv = np.full(image.shape, 0.5)
     psf_mirror = psf[::-1, ::-1]
 
     for _ in range(iterations):
