@@ -11,6 +11,7 @@ from ._chan_vese import chan_vese
 from .morphsnakes import (morphological_geodesic_active_contour,
                           morphological_chan_vese, inverse_gaussian_gradient,
                           circle_level_set, checkerboard_level_set)
+from ..morphology import flood, flood_fill
 
 
 __all__ = ['random_walker',
@@ -30,5 +31,7 @@ __all__ = ['random_walker',
            'morphological_chan_vese',
            'inverse_gaussian_gradient',
            'circle_level_set',
-           'checkerboard_level_set'
+           'checkerboard_level_set',
+           'flood',
+           'flood_fill',
            ]
