@@ -2,7 +2,7 @@ import numpy as np
 from ..measure import label
 
 
-def clear_border(labels, buffer_size=0, bgval=0, in_place=False):
+def clear_border(labels, buffer_size=0, bgval=0, in_place=False, mask=None):
     """Clear objects connected to the label image border.
 
     Parameters
@@ -16,7 +16,10 @@ def clear_border(labels, buffer_size=0, bgval=0, in_place=False):
         Cleared objects are set to this value.
     in_place : bool, optional
         Whether or not to manipulate the labels array in-place.
-
+    mask : ndarray of bool, same shape as `image`, optional.
+        Image data mask. Objects in labels image overlapping with
+        False pixels of mask will be removed. If defined, the 
+        argument buffer_size will be ignored.
     Returns
     -------
     out : (M[, N[, ..., P]]) array
@@ -27,38 +30,59 @@ def clear_border(labels, buffer_size=0, bgval=0, in_place=False):
     >>> import numpy as np
     >>> from skimage.segmentation import clear_border
     >>> labels = np.array([[0, 0, 0, 0, 0, 0, 0, 1, 0],
-    ...                    [0, 0, 0, 0, 1, 0, 0, 0, 0],
-    ...                    [1, 0, 0, 1, 0, 1, 0, 0, 0],
-    ...                    [0, 0, 1, 1, 1, 1, 1, 0, 0],
+    ...                    [1, 1, 0, 0, 1, 0, 0, 1, 0],
+    ...                    [1, 1, 0, 1, 0, 1, 0, 0, 0],
+    ...                    [0, 0, 0, 1, 1, 1, 1, 0, 0],
     ...                    [0, 1, 1, 1, 1, 1, 1, 1, 0],
     ...                    [0, 0, 0, 0, 0, 0, 0, 0, 0]])
     >>> clear_border(labels)
     array([[0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 1, 0, 0, 0, 0],
            [0, 0, 0, 1, 0, 1, 0, 0, 0],
-           [0, 0, 1, 1, 1, 1, 1, 0, 0],
+           [0, 0, 0, 1, 1, 1, 1, 0, 0],
+           [0, 1, 1, 1, 1, 1, 1, 1, 0],
+           [0, 0, 0, 0, 0, 0, 0, 0, 0]])
+    >>> mask = np.array([[0, 0, 1, 1, 1, 1, 1, 1, 1],
+    ...                  [0, 0, 1, 1, 1, 1, 1, 1, 1],
+    ...                  [1, 1, 1, 1, 1, 1, 1, 1, 1],
+    ...                  [1, 1, 1, 1, 1, 1, 1, 1, 1],
+    ...                  [1, 1, 1, 1, 1, 1, 1, 1, 1],
+    ...                  [1, 1, 1, 1, 1, 1, 1, 1, 1]]).astype(np.bool)
+    >>> clear_border(labels, mask=mask)
+    array([[0, 0, 0, 0, 0, 0, 0, 1, 0],
+           [0, 0, 0, 0, 1, 0, 0, 1, 0],
+           [0, 0, 0, 1, 0, 1, 0, 0, 0],
+           [0, 0, 0, 1, 1, 1, 1, 0, 0],
            [0, 1, 1, 1, 1, 1, 1, 1, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0]])
 
     """
     image = labels
 
-    if any( ( buffer_size >= s for s in image.shape)):
+    if any((buffer_size >= s for s in image.shape)) and mask is None:
+        # ignore buffer_size if mask
         raise ValueError("buffer size may not be greater than image size")
 
-    # create borders with buffer_size
-    borders = np.zeros_like(image, dtype=np.bool_)
-    ext = buffer_size + 1
-    slstart = slice(ext)
-    slend   = slice(-ext, None)
-    slices  = [slice(s) for s in image.shape]
-    for d in range(image.ndim):
-        slicedim = list(slices)
-        slicedim[d] = slstart
-        borders[tuple(slicedim)] = True
-        slicedim[d] = slend
-        borders[tuple(slicedim)] = True
-
+    if mask is not None:
+        err_msg = "image and mask should have the same shape but are {} and {}"
+        assert image.shape == mask.shape, \
+               err_msg.format(image.shape, mask.shape)
+        if mask.dtype != np.bool_:
+            raise TypeError("mask should be of type bool.")
+        borders = ~mask
+    else:
+        # create borders with buffer_size
+        borders = np.zeros_like(image, dtype=np.bool_)
+        ext = buffer_size + 1
+        slstart = slice(ext)
+        slend = slice(-ext, None)
+        slices = [slice(s) for s in image.shape]
+        for d in range(image.ndim):
+            slicedim = list(slices)
+            slicedim[d] = slstart
+            borders[tuple(slicedim)] = True
+            slicedim[d] = slend
+            borders[tuple(slicedim)] = True
     # Re-label, in case we are dealing with a binary image
     # and to get consistent labeling
     labels = label(image, background=0)
