@@ -1,7 +1,4 @@
-# coding=utf-8
-from __future__ import division
-
-import collections as coll
+from collections.abc import Iterable
 import numpy as np
 from scipy import ndimage as ndi
 
@@ -130,7 +127,7 @@ def slic(image, n_segments=100, compactness=10., max_iter=10, sigma=0,
     elif isinstance(spacing, (list, tuple)):
         spacing = np.array(spacing, dtype=np.double)
 
-    if not isinstance(sigma, coll.Iterable):
+    if not isinstance(sigma, Iterable):
         sigma = np.array([sigma, sigma, sigma], dtype=np.double)
         sigma /= spacing.astype(np.double)
     elif isinstance(sigma, (list, tuple)):
