@@ -1,7 +1,8 @@
 import numpy as np
-from numpy.testing import assert_array_equal, assert_allclose
 from skimage.segmentation import find_boundaries, mark_boundaries
 
+from skimage._shared.testing import assert_array_equal, assert_allclose
+
 
 white = (1, 1, 1)
 
@@ -107,17 +108,13 @@ def test_mark_boundaries_subpixel():
     marked_proj = np.round(np.mean(marked, axis=-1), 2)
 
     ref_result = np.array(
-            [[ 0.55,  0.63,  0.72,  0.69,  0.6 ,  0.55,  0.54],
-             [ 0.45,  0.58,  0.72,  1.  ,  1.  ,  1.  ,  0.69],
-             [ 0.42,  0.54,  0.65,  1.  ,  0.44,  1.  ,  0.89],
-             [ 0.69,  1.  ,  1.  ,  1.  ,  0.69,  1.  ,  0.83],
-             [ 0.96,  1.  ,  0.38,  1.  ,  0.79,  1.  ,  0.53],
-             [ 0.89,  1.  ,  1.  ,  1.  ,  0.38,  1.  ,  0.16],
-             [ 0.57,  0.78,  0.93,  1.  ,  0.07,  1.  ,  0.09],
-             [ 0.2 ,  0.52,  0.92,  1.  ,  1.  ,  1.  ,  0.54],
-             [ 0.02,  0.35,  0.83,  0.9 ,  0.78,  0.81,  0.87]])
+        [[ 0.55,  0.63,  0.72,  0.69,  0.6 ,  0.55,  0.54],
+         [ 0.45,  0.58,  0.72,  1.  ,  1.  ,  1.  ,  0.69],
+         [ 0.42,  0.54,  0.65,  1.  ,  0.44,  1.  ,  0.89],
+         [ 0.69,  1.  ,  1.  ,  1.  ,  0.69,  1.  ,  0.83],
+         [ 0.96,  1.  ,  0.38,  1.  ,  0.79,  1.  ,  0.53],
+         [ 0.89,  1.  ,  1.  ,  1.  ,  0.38,  1.  ,  0.16],
+         [ 0.57,  0.78,  0.93,  1.  ,  0.07,  1.  ,  0.09],
+         [ 0.2 ,  0.52,  0.92,  1.  ,  1.  ,  1.  ,  0.54],
+         [ 0.02,  0.35,  0.83,  0.9 ,  0.78,  0.81,  0.87]])
     assert_allclose(marked_proj, ref_result, atol=0.01)
-
-
-if __name__ == "__main__":
-    np.testing.run_module_suite()
