@@ -1,4 +1,3 @@
-import six
 import numpy as np
 from scipy import ndimage as ndi
 
@@ -17,28 +16,14 @@ HOMOGRAPHY_TRANSFORMS = (
 )
 
 
-def _multichannel_default(multichannel, ndim):
-    if multichannel is not None:
-        return multichannel
-    else:
-        warn('The default multichannel argument (None) is deprecated.  Please '
-             'specify either True or False explicitly.  multichannel will '
-             'default to False starting with release 0.16.')
-        # utility for maintaining previous color image default behavior
-        if ndim == 3:
-            return True
-        else:
-            return False
-
-
-def resize(image, output_shape, order=1, mode=None, cval=0, clip=True,
-           preserve_range=False, anti_aliasing=None, anti_aliasing_sigma=None):
+def resize(image, output_shape, order=1, mode='reflect', cval=0, clip=True,
+           preserve_range=False, anti_aliasing=True, anti_aliasing_sigma=None):
     """Resize image to match a certain size.
 
-    Performs interpolation to up-size or down-size images. Note that anti-
-    aliasing should be enabled when down-sizing images to avoid aliasing
-    artifacts. For down-sampling N-dimensional images with an integer factor
-    also see `skimage.transform.downscale_local_mean`.
+    Performs interpolation to up-size or down-size N-dimensional images. Note
+    that anti-aliasing should be enabled when down-sizing images to avoid
+    aliasing artifacts. For down-sampling with an integer factor also see
+    `skimage.transform.downscale_local_mean`.
 
     Parameters
     ----------
@@ -62,8 +47,7 @@ def resize(image, output_shape, order=1, mode=None, cval=0, clip=True,
         be in the range 0-5. See `skimage.transform.warp` for detail.
     mode : {'constant', 'edge', 'symmetric', 'reflect', 'wrap'}, optional
         Points outside the boundaries of the input are filled according
-        to the given mode.  Modes match the behaviour of `numpy.pad`.  The
-        default mode is 'constant'.
+        to the given mode.  Modes match the behaviour of `numpy.pad`.
     cval : float, optional
         Used in conjunction with mode 'constant', the value outside
         the image boundaries.
@@ -74,14 +58,16 @@ def resize(image, output_shape, order=1, mode=None, cval=0, clip=True,
     preserve_range : bool, optional
         Whether to keep the original range of values. Otherwise, the input
         image is converted according to the conventions of `img_as_float`.
+        Also see https://scikit-image.org/docs/dev/user_guide/data_types.html
     anti_aliasing : bool, optional
         Whether to apply a Gaussian filter to smooth the image prior to
         down-scaling. It is crucial to filter when down-sampling the image to
         avoid aliasing artifacts.
     anti_aliasing_sigma : {float, tuple of floats}, optional
         Standard deviation for Gaussian filtering to avoid aliasing artifacts.
-        By default, this value is chosen as (1 - s) / 2 where s is the
-        down-scaling factor.
+        By default, this value is chosen as (s - 1) / 2 where s is the
+        down-scaling factor, where s > 1. For the up-size case, s < 1, no
+        anti-aliasing is performed prior to rescaling.
 
     Notes
     -----
@@ -96,20 +82,10 @@ def resize(image, output_shape, order=1, mode=None, cval=0, clip=True,
     >>> from skimage import data
     >>> from skimage.transform import resize
     >>> image = data.camera()
-    >>> resize(image, (100, 100), mode='reflect').shape
+    >>> resize(image, (100, 100)).shape
     (100, 100)
 
     """
-    if mode is None:
-        mode = 'constant'
-        warn("The default mode, 'constant', will be changed to 'reflect' in "
-             "skimage 0.15.")
-
-    if anti_aliasing is None:
-        anti_aliasing = False
-        warn("Anti-aliasing will be enabled by default in skimage 0.15 to "
-             "avoid aliasing artifacts when down-sampling images.")
-
     output_shape = tuple(output_shape)
     output_ndim = len(output_shape)
     input_shape = image.shape
@@ -140,8 +116,24 @@ def resize(image, output_shape, order=1, mode=None, cval=0, clip=True,
                 warn("Anti-aliasing standard deviation greater than zero but "
                      "not down-sampling along all axes")
 
+        # Translate modes used by np.pad to those used by ndi.gaussian_filter
+        np_pad_to_ndimage = {
+            'constant': 'constant',
+            'edge': 'nearest',
+            'symmetric': 'reflect',
+            'reflect': 'mirror',
+            'wrap': 'wrap'
+        }
+        try:
+            ndi_mode = np_pad_to_ndimage[mode]
+        except KeyError:
+            raise ValueError("Unknown mode, or cannot translate mode. The "
+                             "mode should be one of 'constant', 'edge', "
+                             "'symmetric', 'reflect', or 'wrap'. See the "
+                             "documentation of numpy.pad for more info.")
+
         image = ndi.gaussian_filter(image, anti_aliasing_sigma,
-                                    cval=cval, mode=mode)
+                                    cval=cval, mode=ndi_mode)
 
     # 2-dimensional interpolation
     if len(output_shape) == 2 or (len(output_shape) == 3 and
@@ -164,6 +156,11 @@ def resize(image, output_shape, order=1, mode=None, cval=0, clip=True,
             tform = AffineTransform()
             tform.estimate(src_corners, dst_corners)
 
+        # Make sure the transform is exactly metric, to ensure fast warping.
+        tform.params[2] = (0, 0, 1)
+        tform.params[0, 1] = 0
+        tform.params[1, 0] = 0
+
         out = warp(image, tform, output_shape=output_shape, order=order,
                    mode=mode, cval=cval, clip=clip,
                    preserve_range=preserve_range)
@@ -187,15 +184,15 @@ def resize(image, output_shape, order=1, mode=None, cval=0, clip=True,
     return out
 
 
-def rescale(image, scale, order=1, mode=None, cval=0, clip=True,
-            preserve_range=False, multichannel=None,
-            anti_aliasing=None, anti_aliasing_sigma=None):
+def rescale(image, scale, order=1, mode='reflect', cval=0, clip=True,
+            preserve_range=False, multichannel=False,
+            anti_aliasing=True, anti_aliasing_sigma=None):
     """Scale image by a certain factor.
 
-    Performs interpolation to up-scale or down-scale images. Note that anti-
-    aliasing should be enabled when down-sizing images to avoid aliasing
-    artifacts. For down-sampling N-dimensional images with an integer factor
-    also see `skimage.transform.downscale_local_mean`.
+    Performs interpolation to up-scale or down-scale N-dimensional images.
+    Note that anti-aliasing should be enabled when down-sizing images to avoid
+    aliasing artifacts. For down-sampling with an integer factor also see
+    `skimage.transform.downscale_local_mean`.
 
     Parameters
     ----------
@@ -217,8 +214,7 @@ def rescale(image, scale, order=1, mode=None, cval=0, clip=True,
         be in the range 0-5. See `skimage.transform.warp` for detail.
     mode : {'constant', 'edge', 'symmetric', 'reflect', 'wrap'}, optional
         Points outside the boundaries of the input are filled according
-        to the given mode.  Modes match the behaviour of `numpy.pad`.  The
-        default mode is 'constant'.
+        to the given mode.  Modes match the behaviour of `numpy.pad`.
     cval : float, optional
         Used in conjunction with mode 'constant', the value outside
         the image boundaries.
@@ -229,11 +225,11 @@ def rescale(image, scale, order=1, mode=None, cval=0, clip=True,
     preserve_range : bool, optional
         Whether to keep the original range of values. Otherwise, the input
         image is converted according to the conventions of `img_as_float`.
+        Also see
+        https://scikit-image.org/docs/dev/user_guide/data_types.html
     multichannel : bool, optional
         Whether the last axis of the image is to be interpreted as multiple
-        channels or another spatial dimension. By default, is set to True for
-        3D (2D+color) inputs, and False for others. Starting in release 0.16,
-        this will always default to False.
+        channels or another spatial dimension.
     anti_aliasing : bool, optional
         Whether to apply a Gaussian filter to smooth the image prior to
         down-scaling. It is crucial to filter when down-sampling the image to
@@ -256,13 +252,12 @@ def rescale(image, scale, order=1, mode=None, cval=0, clip=True,
     >>> from skimage import data
     >>> from skimage.transform import rescale
     >>> image = data.camera()
-    >>> rescale(image, 0.1, mode='reflect').shape
+    >>> rescale(image, 0.1).shape
     (51, 51)
-    >>> rescale(image, 0.5, mode='reflect').shape
+    >>> rescale(image, 0.5).shape
     (256, 256)
 
     """
-    multichannel = _multichannel_default(multichannel, image.ndim)
     scale = np.atleast_1d(scale)
     if len(scale) > 1:
         if ((not multichannel and len(scale) != image.ndim) or
@@ -325,6 +320,8 @@ def rotate(image, angle, resize=False, center=None, order=1, mode='constant',
     preserve_range : bool, optional
         Whether to keep the original range of values. Otherwise, the input
         image is converted according to the conventions of `img_as_float`.
+        Also see
+        https://scikit-image.org/docs/dev/user_guide/data_types.html
 
     Notes
     -----
@@ -376,13 +373,16 @@ def rotate(image, angle, resize=False, center=None, order=1, mode='constant',
         maxr = corners[:, 1].max()
         out_rows = maxr - minr + 1
         out_cols = maxc - minc + 1
-        output_shape = np.ceil((out_rows, out_cols))
+        output_shape = np.around((out_rows, out_cols))
 
         # fit output image in new shape
         translation = (minc, minr)
         tform4 = SimilarityTransform(translation=translation)
         tform = tform4 + tform
 
+    # Make sure the transform is exactly affine, to ensure fast warping.
+    tform.params[2] = (0, 0, 1)
+
     return warp(image, tform, output_shape=output_shape, order=order,
                 mode=mode, cval=cval, clip=clip, preserve_range=preserve_range)
 
@@ -393,10 +393,9 @@ def downscale_local_mean(image, factors, cval=0, clip=True):
     The image is padded with `cval` if it is not perfectly divisible by the
     integer factors.
 
-    In contrast to the 2-D interpolation in `skimage.transform.resize` and
-    `skimage.transform.rescale` this function may be applied to N-dimensional
-    images and calculates the local mean of elements in each block of size
-    `factors` in the input image.
+    In contrast to interpolation in `skimage.transform.resize` and
+    `skimage.transform.rescale` this function calculates the local mean of
+    elements in each block of size `factors` in the input image.
 
     Parameters
     ----------
@@ -407,11 +406,18 @@ def downscale_local_mean(image, factors, cval=0, clip=True):
     cval : float, optional
         Constant padding value if image is not perfectly divisible by the
         integer factors.
+    clip : bool, optional
+        Unused, but kept here for API consistency with the other transforms
+        in this module. (The local mean will never fall outside the range
+        of values in the input image, assuming the provided `cval` also
+        falls within that range.)
 
     Returns
     -------
     image : ndarray
         Down-sampled image with same number of dimensions as input image.
+        For integer inputs, the output dtype will be ``float64``.
+        See :func:`numpy.mean` for details.
 
     Examples
     --------
@@ -448,7 +454,7 @@ def _swirl_mapping(xy, center, rotation, strength, radius):
 
 
 def swirl(image, center=None, strength=1, radius=100, rotation=0,
-          output_shape=None, order=1, mode=None, cval=0, clip=True,
+          output_shape=None, order=1, mode='reflect', cval=0, clip=True,
           preserve_range=False):
     """Perform a swirl transformation.
 
@@ -493,13 +499,10 @@ def swirl(image, center=None, strength=1, radius=100, rotation=0,
     preserve_range : bool, optional
         Whether to keep the original range of values. Otherwise, the input
         image is converted according to the conventions of `img_as_float`.
+        Also see
+        https://scikit-image.org/docs/dev/user_guide/data_types.html
 
     """
-    if mode is None:
-        warn('The default of `mode` in `skimage.transform.swirl` '
-             'will change to `reflect` in version 0.15.')
-        mode = 'constant'
-
     if center is None:
         center = np.array(image.shape)[:2][::-1] / 2
 
@@ -727,6 +730,8 @@ def warp(image, inverse_map, map_args={}, output_shape=None, order=1,
     preserve_range : bool, optional
         Whether to keep the original range of values. Otherwise, the input
         image is converted according to the conventions of `img_as_float`.
+        Also see
+        https://scikit-image.org/docs/dev/user_guide/data_types.html
 
     Returns
     -------
@@ -797,6 +802,10 @@ def warp(image, inverse_map, map_args={}, output_shape=None, order=1,
     >>> warped = warp(cube, coords)
 
     """
+
+    if image.size == 0:
+        raise ValueError("Cannot warp empty image with dimensions", image.shape)
+
     image = convert_to_float(image, preserve_range)
 
     input_shape = np.array(image.shape)
@@ -836,20 +845,22 @@ def warp(image, inverse_map, map_args={}, output_shape=None, order=1,
               inverse_map.__name__ == 'inverse' and
               get_bound_method_class(inverse_map) in HOMOGRAPHY_TRANSFORMS):
             # inverse_map is the inverse of a homography
-            matrix = np.linalg.inv(six.get_method_self(inverse_map).params)
+            matrix = np.linalg.inv(inverse_map.__self__.params)
 
         if matrix is not None:
-            matrix = matrix.astype(np.double)
+            matrix = matrix.astype(image.dtype)
+            ctype = 'float32_t' if image.dtype == np.float32 else 'float64_t'
             if image.ndim == 2:
-                warped = _warp_fast(image, matrix,
-                                    output_shape=output_shape,
-                                    order=order, mode=mode, cval=cval)
+                warped = _warp_fast[ctype](image, matrix,
+                                           output_shape=output_shape,
+                                           order=order, mode=mode, cval=cval)
             elif image.ndim == 3:
                 dims = []
                 for dim in range(image.shape[2]):
-                    dims.append(_warp_fast(image[..., dim], matrix,
-                                           output_shape=output_shape,
-                                           order=order, mode=mode, cval=cval))
+                    dims.append(_warp_fast[ctype](image[..., dim], matrix,
+                                                  output_shape=output_shape,
+                                                  order=order, mode=mode,
+                                                  cval=cval))
                 warped = np.dstack(dims)
 
     if warped is None:
@@ -896,3 +907,166 @@ def warp(image, inverse_map, map_args={}, output_shape=None, order=1,
     _clip_warp_output(image, warped, order, mode, cval, clip)
 
     return warped
+
+
+def _linear_polar_mapping(output_coords, k_angle, k_radius, center):
+    """Inverse mapping function to convert from cartesion to polar coordinates
+
+    Parameters
+    ----------
+    output_coords : ndarray
+        `(M, 2)` array of `(col, row)` coordinates in the output image
+    k_angle : float
+        Scaling factor that relates the intended number of rows in the output
+        image to angle: ``k_angle = nrows / (2 * np.pi)``
+    k_radius : float
+        Scaling factor that relates the radius of the circle bounding the
+        area to be transformed to the intended number of columns in the output
+        image: ``k_radius = ncols / radius``
+    center : tuple (row, col)
+        Coordinates that represent the center of the circle that bounds the
+        area to be transformed in an input image.
+
+    Returns
+    -------
+    coords : ndarray
+        `(M, 2)` array of `(col, row)` coordinates in the input image that
+        correspond to the `output_coords` given as input.
+    """
+    angle = output_coords[:, 1] / k_angle
+    rr = ((output_coords[:, 0] / k_radius) * np.sin(angle)) + center[0]
+    cc = ((output_coords[:, 0] / k_radius) * np.cos(angle)) + center[1]
+    coords = np.column_stack((cc, rr))
+    return coords
+
+
+def _log_polar_mapping(output_coords, k_angle, k_radius, center):
+    """Inverse mapping function to convert from cartesion to polar coordinates
+
+    Parameters
+    ----------
+    output_coords : ndarray
+        `(M, 2)` array of `(col, row)` coordinates in the output image
+    k_angle : float
+        Scaling factor that relates the intended number of rows in the output
+        image to angle: ``k_angle = nrows / (2 * np.pi)``
+    k_radius : float
+        Scaling factor that relates the radius of the circle bounding the
+        area to be transformed to the intended number of columns in the output
+        image: ``k_radius = width / np.log(radius)``
+    center : tuple (row, col)
+        Coordinates that represent the center of the circle that bounds the
+        area to be transformed in an input image.
+
+    Returns
+    -------
+    coords : ndarray
+        `(M, 2)` array of `(col, row)` coordinates in the input image that
+        correspond to the `output_coords` given as input.
+    """
+    angle = output_coords[:, 1] / k_angle
+    rr = ((np.exp(output_coords[:, 0] / k_radius)) * np.sin(angle)) + center[0]
+    cc = ((np.exp(output_coords[:, 0] / k_radius)) * np.cos(angle)) + center[1]
+    coords = np.column_stack((cc, rr))
+    return coords
+
+
+def warp_polar(image, center=None, *, radius=None, output_shape=None,
+               scaling='linear', multichannel=False, **kwargs):
+    """Remap image to polor or log-polar coordinates space.
+
+    Parameters
+    ----------
+    image : ndarray
+        Input image. Only 2-D arrays are accepted by default. If
+        `multichannel=True`, 3-D arrays are accepted and the last axis is
+        interpreted as multiple channels.
+    center : tuple (row, col), optional
+        Point in image that represents the center of the transformation (i.e.,
+        the origin in cartesian space). Values can be of type `float`.
+        If no value is given, the center is assumed to be the center point
+        of the image.
+    radius : float, optional
+        Radius of the circle that bounds the area to be transformed.
+    output_shape : tuple (row, col), optional
+    scaling : {'linear', 'log'}, optional
+        Specify whether the image warp is polar or log-polar. Defaults to
+        'linear'.
+    multichannel : bool, optional
+        Whether the image is a 3-D array in which the third axis is to be
+        interpreted as multiple channels. If set to `False` (default), only 2-D
+        arrays are accepted.
+    **kwargs : keyword arguments
+        Passed to `transform.warp`.
+
+    Returns
+    -------
+    warped : ndarray
+        The polar or log-polar warped image.
+
+    Examples
+    --------
+    Perform a basic polar warp on a grayscale image:
+
+    >>> from skimage import data
+    >>> from skimage.transform import warp_polar
+    >>> image = data.checkerboard()
+    >>> warped = warp_polar(image)
+
+    Perform a log-polar warp on a grayscale image:
+
+    >>> warped = warp_polar(image, scaling='log')
+
+    Perform a log-polar warp on a grayscale image while specifying center,
+    radius, and output shape:
+
+    >>> warped = warp_polar(image, (100,100), radius=100,
+    ...                     output_shape=image.shape, scaling='log')
+
+    Perform a log-polar warp on a color image:
+
+    >>> image = data.astronaut()
+    >>> warped = warp_polar(image, scaling='log', multichannel=True)
+    """
+    if image.ndim != 2 and not multichannel:
+        raise ValueError("Input array must be 2 dimensions "
+                         "when `multichannel=False`,"
+                         " got {}".format(image.ndim))
+
+    if image.ndim != 3 and multichannel:
+        raise ValueError("Input array must be 3 dimensions "
+                         "when `multichannel=True`,"
+                         " got {}".format(image.ndim))
+
+    if center is None:
+        center = (np.array(image.shape)[:2] / 2) - 0.5
+
+    if radius is None:
+        w, h = np.array(image.shape)[:2] / 2
+        radius = np.sqrt(w ** 2 + h ** 2)
+
+    if output_shape is None:
+        height = 360
+        width = int(np.ceil(radius))
+        output_shape = (height, width)
+    else:
+        output_shape = safe_as_int(output_shape)
+        height = output_shape[0]
+        width = output_shape[1]
+
+    if scaling == 'linear':
+        k_radius = width / radius
+        map_func = _linear_polar_mapping
+    elif scaling == 'log':
+        k_radius = width / np.log(radius)
+        map_func = _log_polar_mapping
+    else:
+        raise ValueError("Scaling value must be in {'linear', 'log'}")
+
+    k_angle = height / (2 * np.pi)
+    warp_args = {'k_angle': k_angle, 'k_radius': k_radius, 'center': center}
+
+    warped = warp(image, map_func, map_args=warp_args,
+                  output_shape=output_shape, **kwargs)
+
+    return warped
