@@ -3,12 +3,10 @@ import numpy as np
 from scipy import ndimage as ndi
 from ..transform import resize
 from ..util import img_as_float
-from ._warps import _multichannel_default
 
 
 def _smooth(image, sigma, mode, cval, multichannel=None):
     """Return image with each channel smoothed by the Gaussian filter."""
-    multichannel = _multichannel_default(multichannel, image.ndim)
     smoothed = np.empty(image.shape, dtype=np.double)
 
     # apply Gaussian filter to all channels independently
@@ -25,7 +23,7 @@ def _check_factor(factor):
 
 
 def pyramid_reduce(image, downscale=2, sigma=None, order=1,
-                   mode='reflect', cval=0, multichannel=None):
+                   mode='reflect', cval=0, multichannel=False):
     """Smooth and then downsample image.
 
     Parameters
@@ -48,9 +46,7 @@ def pyramid_reduce(image, downscale=2, sigma=None, order=1,
         Value to fill past edges of input if mode is 'constant'.
     multichannel : bool, optional
         Whether the last axis of the image is to be interpreted as multiple
-        channels or another spatial dimension. By default, is set to True for
-        3D (2D+color) inputs, and False for others. Starting in release 0.16,
-        this will always default to False.
+        channels or another spatial dimension.
 
     Returns
     -------
@@ -62,7 +58,6 @@ def pyramid_reduce(image, downscale=2, sigma=None, order=1,
     .. [1] http://web.mit.edu/persci/people/adelson/pub_pdfs/pyramid83.pdf
 
     """
-    multichannel = _multichannel_default(multichannel, image.ndim)
     _check_factor(downscale)
 
     image = img_as_float(image)
@@ -83,7 +78,7 @@ def pyramid_reduce(image, downscale=2, sigma=None, order=1,
 
 
 def pyramid_expand(image, upscale=2, sigma=None, order=1,
-                   mode='reflect', cval=0, multichannel=None):
+                   mode='reflect', cval=0, multichannel=False):
     """Upsample and then smooth image.
 
     Parameters
@@ -106,10 +101,7 @@ def pyramid_expand(image, upscale=2, sigma=None, order=1,
         Value to fill past edges of input if mode is 'constant'.
     multichannel : bool, optional
         Whether the last axis of the image is to be interpreted as multiple
-        channels or another spatial dimension. By default, is set to True for
-        3D (2D+color) inputs, and False for others. Starting in release 0.16,
-        this will always default to False.
-
+        channels or another spatial dimension.
 
     Returns
     -------
@@ -121,7 +113,6 @@ def pyramid_expand(image, upscale=2, sigma=None, order=1,
     .. [1] http://web.mit.edu/persci/people/adelson/pub_pdfs/pyramid83.pdf
 
     """
-    multichannel = _multichannel_default(multichannel, image.ndim)
     _check_factor(upscale)
 
     image = img_as_float(image)
@@ -142,7 +133,7 @@ def pyramid_expand(image, upscale=2, sigma=None, order=1,
 
 
 def pyramid_gaussian(image, max_layer=-1, downscale=2, sigma=None, order=1,
-                     mode='reflect', cval=0, multichannel=None):
+                     mode='reflect', cval=0, multichannel=False):
     """Yield images of the Gaussian pyramid formed by the input image.
 
     Recursively applies the `pyramid_reduce` function to the image, and yields
@@ -157,7 +148,7 @@ def pyramid_gaussian(image, max_layer=-1, downscale=2, sigma=None, order=1,
     ----------
     image : ndarray
         Input image.
-    max_layer : int
+    max_layer : int, optional
         Number of layers for the pyramid. 0th layer is the original image.
         Default is -1 which builds all possible layers.
     downscale : float, optional
@@ -176,10 +167,7 @@ def pyramid_gaussian(image, max_layer=-1, downscale=2, sigma=None, order=1,
         Value to fill past edges of input if mode is 'constant'.
     multichannel : bool, optional
         Whether the last axis of the image is to be interpreted as multiple
-        channels or another spatial dimension. By default, is set to True for
-        3D (2D+color) inputs, and False for others. Starting in release 0.16,
-        this will always default to False.
-
+        channels or another spatial dimension.
 
     Returns
     -------
@@ -222,7 +210,7 @@ def pyramid_gaussian(image, max_layer=-1, downscale=2, sigma=None, order=1,
 
 
 def pyramid_laplacian(image, max_layer=-1, downscale=2, sigma=None, order=1,
-                      mode='reflect', cval=0, multichannel=None):
+                      mode='reflect', cval=0, multichannel=False):
     """Yield images of the laplacian pyramid formed by the input image.
 
     Each layer contains the difference between the downsampled and the
@@ -240,7 +228,7 @@ def pyramid_laplacian(image, max_layer=-1, downscale=2, sigma=None, order=1,
     ----------
     image : ndarray
         Input image.
-    max_layer : int
+    max_layer : int, optional
         Number of layers for the pyramid. 0th layer is the original image.
         Default is -1 which builds all possible layers.
     downscale : float, optional
@@ -259,10 +247,7 @@ def pyramid_laplacian(image, max_layer=-1, downscale=2, sigma=None, order=1,
         Value to fill past edges of input if mode is 'constant'.
     multichannel : bool, optional
         Whether the last axis of the image is to be interpreted as multiple
-        channels or another spatial dimension. By default, is set to True for
-        3D (2D+color) inputs, and False for others. Starting in release 0.16,
-        this will always default to False.
-
+        channels or another spatial dimension.
 
     Returns
     -------
@@ -275,7 +260,6 @@ def pyramid_laplacian(image, max_layer=-1, downscale=2, sigma=None, order=1,
     .. [2] http://sepwww.stanford.edu/data/media/public/sep/morgan/texturematch/paper_html/node3.html
 
     """
-    multichannel = _multichannel_default(multichannel, image.ndim)
     _check_factor(downscale)
 
     # cast to float for consistent data type in pyramid
