@@ -1,31 +1,38 @@
-from __future__ import division
 import numpy as np
 from warnings import warn
 
-__all__ = ['img_as_float', 'img_as_int', 'img_as_uint', 'img_as_ubyte',
+
+__all__ = ['img_as_float32', 'img_as_float64', 'img_as_float',
+           'img_as_int', 'img_as_uint', 'img_as_ubyte',
            'img_as_bool', 'dtype_limits']
 
+# For integers Numpy uses `_integer_types` basis internally, and builds a leaky
+# `np.XintYY` abstraction on top of it. This leads to situations when, for
+# example, there are two np.Xint64 dtypes with the same attributes but
+# different object references. In order to avoid any potential issues,
+# we use the basis dtypes here. For more information, see:
+# - https://github.com/scikit-image/scikit-image/issues/3043
+# For convenience, for these dtypes we indicate also the possible bit depths
+# (some of them are platform specific). For the details, see:
+# http://www.unix.org/whitepapers/64bit.html
+_integer_types = (np.byte, np.ubyte,          # 8 bits
+                  np.short, np.ushort,        # 16 bits
+                  np.intc, np.uintc,          # 16 or 32 or 64 bits
+                  np.int_, np.uint,           # 32 or 64 bits
+                  np.longlong, np.ulonglong)  # 64 bits
+_integer_ranges = {t: (np.iinfo(t).min, np.iinfo(t).max)
+                   for t in _integer_types}
 dtype_range = {np.bool_: (False, True),
                np.bool8: (False, True),
-               np.uint8: (0, 255),
-               np.uint16: (0, 65535),
-               np.uint32: (0, 2**32 - 1),
-               np.uint64: (0, 2**64 - 1),
-               np.int8: (-128, 127),
-               np.int16: (-32768, 32767),
-               np.int32: (-2**31, 2**31 - 1),
-               np.int64: (-2**63, 2**63 - 1),
                np.float16: (-1, 1),
                np.float32: (-1, 1),
                np.float64: (-1, 1)}
+dtype_range.update(_integer_ranges)
 
-_supported_types = (np.bool_, np.bool8,
-                    np.uint8, np.uint16, np.uint32, np.uint64,
-                    np.int8, np.int16, np.int32, np.int64,
-                    np.float16, np.float32, np.float64)
+_supported_types = list(dtype_range.keys())
 
 
-def dtype_limits(image, clip_negative=None):
+def dtype_limits(image, clip_negative=False):
     """Return intensity limits, i.e. (min, max) tuple, of the image's dtype.
 
     Parameters
@@ -35,23 +42,137 @@ def dtype_limits(image, clip_negative=None):
     clip_negative : bool, optional
         If True, clip the negative range (i.e. return 0 for min intensity)
         even if the image dtype allows negative values.
-        The default behavior (None) is equivalent to True.
 
     Returns
     -------
     imin, imax : tuple
         Lower and upper intensity limits.
     """
-    if clip_negative is None:
-        clip_negative = True
-        warn('The default of `clip_negative` in `skimage.util.dtype_limits` '
-             'will change to `False` in version 0.15.')
     imin, imax = dtype_range[image.dtype.type]
     if clip_negative:
         imin = 0
     return imin, imax
 
 
+def _dtype_itemsize(itemsize, *dtypes):
+    """Return first of `dtypes` with itemsize greater than `itemsize`
+
+    Parameters
+    ----------
+    itemsize: int
+        The data type object element size.
+
+    Other Parameters
+    ----------------
+    *dtypes:
+        Any Object accepted by `np.dtype` to be converted to a data
+        type object
+
+    Returns
+    -------
+    dtype: data type object
+        First of `dtypes` with itemsize greater than `itemsize`.
+
+    """
+    return next(dt for dt in dtypes if np.dtype(dt).itemsize >= itemsize)
+
+
+def _dtype_bits(kind, bits, itemsize=1):
+    """Return dtype of `kind` that can store a `bits` wide unsigned int
+
+    Parameters:
+    kind: str
+        Data type kind.
+    bits: int
+        Desired number of bits.
+    itemsize: int
+        The data type object element size.
+
+    Returns
+    -------
+    dtype: data type object
+        Data type of `kind` that can store a `bits` wide unsigned int
+
+    """
+
+    s = next(i for i in (itemsize, ) + (2, 4, 8) if
+             bits < (i * 8) or (bits == (i * 8) and kind == 'u'))
+
+    return np.dtype(kind + str(s))
+
+
+def _scale(a, n, m, copy=True):
+    """Scale an array of unsigned/positive integers from `n` to `m` bits.
+
+    Numbers can be represented exactly only if `m` is a multiple of `n`.
+
+    Parameters
+    ----------
+    a : ndarray
+        Input image array.
+    n : int
+        Number of bits currently used to encode the values in `a`.
+    m : int
+        Desired number of bits to encode the values in `out`.
+    copy : bool, optional
+        If True, allocates and returns new array. Otherwise, modifies
+        `a` in place.
+
+    Returns
+    -------
+    out : array
+        Output image array. Has the same kind as `a`.
+    """
+    kind = a.dtype.kind
+    if n > m and a.max() < 2 ** m:
+        mnew = int(np.ceil(m / 2) * 2)
+        if mnew > m:
+            dtype = "int{}".format(mnew)
+        else:
+            dtype = "uint{}".format(mnew)
+        n = int(np.ceil(n / 2) * 2)
+        warn("Downcasting {} to {} without scaling because max "
+             "value {} fits in {}".format(a.dtype, dtype, a.max(), dtype),
+             stacklevel=3)
+        return a.astype(_dtype_bits(kind, m))
+    elif n == m:
+        return a.copy() if copy else a
+    elif n > m:
+        # downscale with precision loss
+        if copy:
+            b = np.empty(a.shape, _dtype_bits(kind, m))
+            np.floor_divide(a, 2**(n - m), out=b, dtype=a.dtype,
+                            casting='unsafe')
+            return b
+        else:
+            a //= 2**(n - m)
+            return a
+    elif m % n == 0:
+        # exact upscale to a multiple of `n` bits
+        if copy:
+            b = np.empty(a.shape, _dtype_bits(kind, m))
+            np.multiply(a, (2**m - 1) // (2**n - 1), out=b, dtype=b.dtype)
+            return b
+        else:
+            a = a.astype(_dtype_bits(kind, m, a.dtype.itemsize), copy=False)
+            a *= (2**m - 1) // (2**n - 1)
+            return a
+    else:
+        # upscale to a multiple of `n` bits,
+        # then downscale with precision loss
+        o = (m // n + 1) * n
+        if copy:
+            b = np.empty(a.shape, _dtype_bits(kind, o))
+            np.multiply(a, (2**o - 1) // (2**n - 1), out=b, dtype=b.dtype)
+            b //= 2**(o - m)
+            return b
+        else:
+            a = a.astype(_dtype_bits(kind, o, a.dtype.itemsize), copy=False)
+            a *= (2**o - 1) // (2**n - 1)
+            a //= 2**(o - m)
+            return a
+
+
 def convert(image, dtype, force_copy=False, uniform=False):
     """
     Convert an image to the requested data-type.
@@ -81,10 +202,17 @@ def convert(image, dtype, force_copy=False, uniform=False):
         rounded to the nearest integers, which minimizes back and forth
         conversion errors.
 
+    .. versionchanged :: 0.15
+        ``convert`` no longer warns about possible precision or sign
+        information loss. See discussions on these warnings at:
+        https://github.com/scikit-image/scikit-image/issues/2602
+        https://github.com/scikit-image/scikit-image/issues/543#issuecomment-208202228
+        https://github.com/scikit-image/scikit-image/pull/3575
+
     References
     ----------
     .. [1] DirectX data conversion rules.
-           http://msdn.microsoft.com/en-us/library/windows/desktop/dd607323%28v=vs.85%29.aspx
+           https://msdn.microsoft.com/en-us/library/windows/desktop/dd607323%28v=vs.85%29.aspx
     .. [2] Data Conversions. In "OpenGL ES 2.0 Specification v2.0.25",
            pp 7-8. Khronos Group, 2010.
     .. [3] Proper treatment of pixels as integers. A.W. Paeth.
@@ -103,7 +231,16 @@ def convert(image, dtype, force_copy=False, uniform=False):
     itemsize_in = dtypeobj_in.itemsize
     itemsize_out = dtypeobj_out.itemsize
 
-    if dtype_in == dtype_out:
+    # Below, we do an `issubdtype` check.  Its purpose is to find out
+    # whether we can get away without doing any image conversion.  This happens
+    # when:
+    #
+    # - the output and input dtypes are the same or
+    # - when the output is specified as a type, and the input dtype
+    #   is a subclass of that type (e.g. `np.floating` will allow
+    #   `float32` and `float64` arrays through)
+
+    if np.issubdtype(dtype_in, np.obj2sctype(dtype)):
         if force_copy:
             image = image.copy()
         return image
@@ -112,103 +249,6 @@ def convert(image, dtype, force_copy=False, uniform=False):
         raise ValueError("Can not convert from {} to {}."
                          .format(dtypeobj_in, dtypeobj_out))
 
-    def sign_loss():
-        warn("Possible sign loss when converting negative image of type "
-             "{} to positive image of type {}."
-             .format(dtypeobj_in, dtypeobj_out))
-
-    def prec_loss():
-        warn("Possible precision loss when converting from {} to {}"
-             .format(dtypeobj_in, dtypeobj_out))
-
-    def _dtype_itemsize(itemsize, *dtypes):
-        # Return first of `dtypes` with itemsize greater than `itemsize`
-        return next(dt for dt in dtypes if np.dtype(dt).itemsize >= itemsize)
-
-    def _dtype_bits(kind, bits, itemsize=1):
-        # Return dtype of `kind` that can store a `bits` wide unsigned int
-        def compare(x, y, kind='u'):
-            if kind == 'u':
-                return x <= y
-            else:
-                return x < y
-
-        s = next(i for i in (itemsize, ) + (2, 4, 8) if compare(bits, i * 8,
-                                                                kind=kind))
-        return np.dtype(kind + str(s))
-
-    def _scale(a, n, m, copy=True):
-        """Scale an array of unsigned/positive integers from `n` to `m` bits.
-
-        Numbers can be represented exactly only if `m` is a multiple of `n`.
-
-        Parameters
-        ----------
-        a : ndarray
-            Input image array.
-        n : int
-            Number of bits currently used to encode the values in `a`.
-        m : int
-            Desired number of bits to encode the values in `out`.
-        copy : bool, optional
-            If True, allocates and returns new array. Otherwise, modifies
-            `a` in place.
-
-        Returns
-        -------
-        out : array
-            Output image array. Has the same kind as `a`.
-        """
-        kind = a.dtype.kind
-        if n > m and a.max() < 2 ** m:
-            mnew = int(np.ceil(m / 2) * 2)
-            if mnew > m:
-                dtype = "int{}".format(mnew)
-            else:
-                dtype = "uint{}".format(mnew)
-            n = int(np.ceil(n / 2) * 2)
-            warn("Downcasting {} to {} without scaling because max "
-                 "value {} fits in {}".format(a.dtype, dtype, a.max(), dtype))
-            return a.astype(_dtype_bits(kind, m))
-        elif n == m:
-            return a.copy() if copy else a
-        elif n > m:
-            # downscale with precision loss
-            prec_loss()
-            if copy:
-                b = np.empty(a.shape, _dtype_bits(kind, m))
-                np.floor_divide(a, 2**(n - m), out=b, dtype=a.dtype,
-                                casting='unsafe')
-                return b
-            else:
-                a //= 2**(n - m)
-                return a
-        elif m % n == 0:
-            # exact upscale to a multiple of `n` bits
-            if copy:
-                b = np.empty(a.shape, _dtype_bits(kind, m))
-                np.multiply(a, (2**m - 1) // (2**n - 1), out=b, dtype=b.dtype)
-                return b
-            else:
-                a = a.astype(_dtype_bits(kind, m, a.dtype.itemsize), copy=False)
-                a *= (2**m - 1) // (2**n - 1)
-                return a
-        else:
-            # upscale to a multiple of `n` bits,
-            # then downscale with precision loss
-            prec_loss()
-            o = (m // n + 1) * n
-            if copy:
-                b = np.empty(a.shape, _dtype_bits(kind, o))
-                np.multiply(a, (2**o - 1) // (2**n - 1), out=b, dtype=b.dtype)
-                b //= 2**(o - m)
-                return b
-            else:
-                a = a.astype(_dtype_bits(kind, o, a.dtype.itemsize), copy=False)
-                a *= (2**o - 1) // (2**n - 1)
-                a //= 2**(o - m)
-                return a
-
     if kind_in in 'ui':
         imin_in = np.iinfo(dtype_in).min
         imax_in = np.iinfo(dtype_in).max
@@ -218,9 +258,6 @@ def convert(image, dtype, force_copy=False, uniform=False):
 
     # any -> binary
     if kind_out == 'b':
-        if kind_in in "fi":
-            sign_loss()
-        prec_loss()
         return image > dtype_in(dtype_range[dtype_in][1] / 2)
 
     # binary -> any
@@ -232,54 +269,57 @@ def convert(image, dtype, force_copy=False, uniform=False):
 
     # float -> any
     if kind_in == 'f':
-        if np.min(image) < -1.0 or np.max(image) > 1.0:
-            raise ValueError("Images of type float must be between -1 and 1.")
         if kind_out == 'f':
             # float -> float
-            if itemsize_in > itemsize_out:
-                prec_loss()
             return image.astype(dtype_out)
 
+        if np.min(image) < -1.0 or np.max(image) > 1.0:
+            raise ValueError("Images of type float must be between -1 and 1.")
         # floating point -> integer
-        prec_loss()
         # use float type that can represent output integer type
-        image = image.astype(_dtype_itemsize(itemsize_out, dtype_in,
-                                             np.float32, np.float64))
+        computation_type = _dtype_itemsize(itemsize_out, dtype_in,
+                                           np.float32, np.float64)
+
         if not uniform:
             if kind_out == 'u':
-                image *= imax_out
+                image_out = np.multiply(image, imax_out,
+                                        dtype=computation_type)
             else:
-                image *= imax_out - imin_out
-                image -= 1.0
-                image /= 2.0
-            np.rint(image, out=image)
-            np.clip(image, imin_out, imax_out, out=image)
+                image_out = np.multiply(image, (imax_out - imin_out) / 2,
+                                        dtype=computation_type)
+                image_out -= 1.0 / 2.
+            np.rint(image_out, out=image_out)
+            np.clip(image_out, imin_out, imax_out, out=image_out)
         elif kind_out == 'u':
-            image *= imax_out + 1
-            np.clip(image, 0, imax_out, out=image)
+            image_out = np.multiply(image, imax_out + 1,
+                                    dtype=computation_type)
+            np.clip(image_out, 0, imax_out, out=image_out)
         else:
-            image *= (imax_out - imin_out + 1.0) / 2.0
-            np.floor(image, out=image)
-            np.clip(image, imin_out, imax_out, out=image)
-        return image.astype(dtype_out)
+            image_out = np.multiply(image, (imax_out - imin_out + 1.0) / 2.0,
+                                    dtype=computation_type)
+            np.floor(image_out, out=image_out)
+            np.clip(image_out, imin_out, imax_out, out=image_out)
+        return image_out.astype(dtype_out)
 
     # signed/unsigned int -> float
     if kind_out == 'f':
-        if itemsize_in >= itemsize_out:
-            prec_loss()
         # use float type that can exactly represent input integers
-        image = image.astype(_dtype_itemsize(itemsize_in, dtype_out,
-                                             np.float32, np.float64))
+        computation_type = _dtype_itemsize(itemsize_in, dtype_out,
+                                           np.float32, np.float64)
+
         if kind_in == 'u':
-            image /= imax_in
+            # using np.divide or np.multiply doesn't copy the data
+            # until the computation time
+            image = np.multiply(image, 1. / imax_in,
+                                dtype=computation_type)
             # DirectX uses this conversion also for signed ints
             # if imin_in:
             #     np.maximum(image, -1.0, out=image)
         else:
-            image *= 2.0
-            image += 1.0
-            image /= imax_in - imin_in
-        return image.astype(dtype_out)
+            image = np.add(image, 0.5, dtype=computation_type)
+            image *= 2 / (imax_in - imin_in)
+
+        return np.asarray(image, dtype_out)
 
     # unsigned int -> signed/unsigned int
     if kind_in == 'u':
@@ -293,7 +333,6 @@ def convert(image, dtype, force_copy=False, uniform=False):
 
     # signed int -> unsigned int
     if kind_out == 'u':
-        sign_loss()
         image = _scale(image, 8 * itemsize_in - 1, 8 * itemsize_out)
         result = np.empty(image.shape, dtype_out)
         np.maximum(image, 0, out=result, dtype=image.dtype, casting='unsafe')
@@ -310,7 +349,33 @@ def convert(image, dtype, force_copy=False, uniform=False):
     return image.astype(dtype_out)
 
 
-def img_as_float(image, force_copy=False):
+def img_as_float32(image, force_copy=False):
+    """Convert an image to single-precision (32-bit) floating point format.
+
+    Parameters
+    ----------
+    image : ndarray
+        Input image.
+    force_copy : bool, optional
+        Force a copy of the data, irrespective of its current dtype.
+
+    Returns
+    -------
+    out : ndarray of float32
+        Output image.
+
+    Notes
+    -----
+    The range of a floating point image is [0.0, 1.0] or [-1.0, 1.0] when
+    converting from unsigned or signed datatypes, respectively.
+    If the input image has a float type, intensity values are not modified
+    and can be outside the ranges [0.0, 1.0] or [-1.0, 1.0].
+
+    """
+    return convert(image, np.float32, force_copy)
+
+
+def img_as_float64(image, force_copy=False):
     """Convert an image to double-precision (64-bit) floating point format.
 
     Parameters
@@ -336,6 +401,35 @@ def img_as_float(image, force_copy=False):
     return convert(image, np.float64, force_copy)
 
 
+def img_as_float(image, force_copy=False):
+    """Convert an image to floating point format.
+
+    This function is similar to `img_as_float64`, but will not convert
+    lower-precision floating point arrays to `float64`.
+
+    Parameters
+    ----------
+    image : ndarray
+        Input image.
+    force_copy : bool, optional
+        Force a copy of the data, irrespective of its current dtype.
+
+    Returns
+    -------
+    out : ndarray of float
+        Output image.
+
+    Notes
+    -----
+    The range of a floating point image is [0.0, 1.0] or [-1.0, 1.0] when
+    converting from unsigned or signed datatypes, respectively.
+    If the input image has a float type, intensity values are not modified
+    and can be outside the ranges [0.0, 1.0] or [-1.0, 1.0].
+
+    """
+    return convert(image, np.floating, force_copy)
+
+
 def img_as_uint(image, force_copy=False):
     """Convert an image to 16-bit unsigned integer format.
 
