@@ -1,4 +1,5 @@
-from numpy.testing import assert_array_equal, assert_allclose, assert_raises
+from skimage._shared import testing
+from skimage._shared.testing import assert_array_equal, assert_allclose
 
 import numpy as np
 from skimage.data import camera
@@ -113,12 +114,14 @@ def test_localvar():
 
     # Ensure local variance bounds checking works properly
     bad_local_vars = np.zeros_like(data)
-    assert_raises(ValueError, random_noise, data, mode='localvar', seed=seed,
-                  local_vars=bad_local_vars)
+    with testing.raises(ValueError):
+        random_noise(data, mode='localvar', seed=seed,
+                     local_vars=bad_local_vars)
     bad_local_vars += 0.1
     bad_local_vars[0, 0] = -1
-    assert_raises(ValueError, random_noise, data, mode='localvar', seed=seed,
-                  local_vars=bad_local_vars)
+    with testing.raises(ValueError):
+        random_noise(data, mode='localvar', seed=seed,
+                     local_vars=bad_local_vars)
 
 
 def test_speckle():
@@ -207,8 +210,5 @@ def test_clip_speckle():
 
 def test_bad_mode():
     data = np.zeros((64, 64))
-    assert_raises(KeyError, random_noise, data, 'perlin')
-
-
-if __name__ == '__main__':
-    np.testing.run_module_suite()
+    with testing.raises(KeyError):
+        random_noise(data, 'perlin')
