@@ -1,6 +1,7 @@
 import numpy as np
-from numpy.testing import assert_equal, assert_raises
 from skimage.util import unique_rows
+from skimage._shared import testing
+from skimage._shared.testing import assert_equal
 
 
 def test_discontiguous_array():
@@ -28,13 +29,11 @@ def test_float_array():
 
 def test_1d_array():
     ar = np.array([1, 0, 1, 1], np.uint8)
-    assert_raises(ValueError, unique_rows, ar)
+    with testing.raises(ValueError):
+        unique_rows(ar)
 
 
 def test_3d_array():
     ar = np.arange(8).reshape((2, 2, 2))
-    assert_raises(ValueError, unique_rows, ar)
-
-
-if __name__ == '__main__':
-    np.testing.run_module_suite()
+    with testing.raises(ValueError):
+        unique_rows(ar)
